//
// GKOS Driver/Monitor with QWERTY and COM port input
//
// Seppo Tiainen 2007, http://gkos.net
//
// - Thanks to Alan Baylis for his code blocks 
//   (2004) that gave my project a jump start!
//
// Designed by using the great Dev-C++ tool
//

#include <windows.h>
#include <string>
#include <commctrl.h>
#include <stdio.h>
#include "gkosw.h"

// For file I/O
//#include <iostream>
#include <fstream> //<iostream> is actually included in <fstream>
//#include <string>

// ini file IO:
#include <iostream>
using namespace std;

// Set Process priority
HANDLE hProcess;
static int gStat;


void gSendRest(); // Send single characters if no shortcut were found
void gSendShortcut();
void SENDSTR (char *tx_str);

static int  KeyCode = 1024;
static void gSK(int gKeyCode);

const char ClassName[] = "GkoswWindowClass";
HWND hWndEditBox, hWndStatusBar, hWndToolBar;


//bitmap
static HINSTANCE hInstBmp = NULL;
HBITMAP hbmGkos; // Devices
HBITMAP hbmGkosA; // Letters
HBITMAP hbmGkosCtr; // Controls
HBITMAP hbmGkosNum; // Numbers & Navi
HBITMAP hbmGkosSym; // Symbols
HBITMAP hbmGkosInfo; // Info
BITMAP bm;
HWND hgWnd;
static int gDriver = 0;  // Mode of operation Driver=1/Monitor=0
static int BmpSelect = 1; // screen bmp select 1 D, 2 A, 3 SYM, 4 1, 5 Ctrl
static int BmpSelectTEMP = 1;  // to select helper based on shifts
static int gEditBox = 1;// 1 = small editbox, 2 = full editbox, 0 = no editbox
//-------

HHOOK MouseHook;
static bool HookFailed = FALSE;

// Saved parameters for window position:
	unsigned int gWinPosX = 100; // Window position X
	unsigned int gWinPosY = 100;// Window position Y


// timer to update keyboard status every 10 ms
const UINT idTimer1 = 1;
UINT nTimerDelay = 10; // 1000 = once per second, 10 = once per 10 ms

static int QwertySet = 1; // 1 = SDF/JKL(often works),
                          // 2 = ZXC/Arrows(seldom works)
                          // 3 = ZXC/NumPad 123(seldom works)
                          // 4 = 45t/i90 special relaxed position of hands
                          // 5 = NumPad 7 8 9 1 2 3
                          // 6 = NumPad , / -  7 4 1 Modified KP-2000 NumPad:
                          // Key '/' moved to key '8' and Key ',' moved to key ENTER     
                          // 7 = NumPad  0 1 4 - 9 / Standard NumPad (difficut to hold)
                          // (0 = Using COM input)
static bool ComMode = FALSE; // COM or QWERTY input

static void SetLanguage(int Language);
static void SetPCkeyboard(int PCkeyboard);
static bool USmode = TRUE;
static int  Language = 1;   // 1 = en, 2 = fi, 3 = se (xx), 4 = de (yy)
static int  NativeLang = 1; // (default) ALT 123abc: Native <=> English
static int  PCkeyboard = 2; // 1 = US, 2,3 = FIN/SWE,  4 = GER 
static bool SimpleMode = FALSE; //TRUE; // Simple mode = no timers used for chord detection
static bool GkosON = TRUE; // TRUE:  GKOS chord typing and Keyboard Hook active
                           // FALSE: QWERTY typing and Keyboard Hook inactive
static int gRef = 127; // 0...63
static int ChordValue = 256; // 0...255
static int FinalChord = 256; // 0...63
static int ChordValue0 = 256; // 0...63 // The present chord, before release
static int ChordValue1 = 256; // 0...63 // In CheckBack
static int ChordValue2 = 256; // 0...63 // - '' -
static int gkChord = 256; // 0...63
static void SetTables(void);
static void gkOut(int gkChord);
static void InstallHook(void);
static void UnInstallHook(void);
//static bool NationalSC = FALSE;
static void gSK(int gKeyCode);
static void gSK2 (int gRef);
static void gCheckComInput(int KeyCodeC); // calls gCheckInput(...)
static void gCheckInput(int KeyCodeX);
static int gChord(); // chord value paramC/2 ago
static int gChordNow(); // Chord value now
static void gCheckBack();
static int gCheckFastHit(); // A single fast hit must be treated separately

bool Monitor = FALSE;

static int  iChar[257];  // 
static int  iCharUS[257];  // 
static int  iShifts[257];   // shift needed to get the symbol on PC key
static int  iShiftsUS[257]; // shift needed to get the symbol on PC key
static int  gCharRef[1024];    // ChordValue => GKOS Ref 
static char* gChar[257];   // Character to show on GKOS Monitor Display - FIN
static char* gCharUS[257]; // Character to show on GKOS Monitor Display - US
static void ClearShifts(); // Do this before exiting the program

static int XShifts = 0; // User selected by Chordon ? \  => 2 = AltGr
                        // User selected by Chordon Sh \ => 8 = Shift+AltGr

static char AscToGKOS[256]; 	// for Chordon Shortcuts 
static char* Ch[1024];   // the 781 2-byte Chordons (.a !v Si ?t ...)
//static char* SC[1024]; 		// for 781 Chordon Shortcuts EN
//static char* SCfin[1024];   // for 781 Chordon Shortcuts FI

   // New set of shortcut parameters:
   int Lc = 2, Le = 32, Lf = 32, Lx = 32, Ly = 32; // max Lengt of Chordons and word shortcuts (en, fi)   //
   // [number of][lenght]         
   char Chrd[802][5];  // Chords for shortcuts
   char SCen[802][33]; // English shortcuts
   char SCfi[802][33]; // Finnish Shortcus
   char SCxx[802][33]; // xxx Shortcuts
   char SCyy[802][33]; // yyy Shortcuts

// more languages...
static int sc1;
static int sc2;

//----params in functions
static char A[2048];

static char* tx_char;
static char* tmp_char;

//static int Nchar; // declared within the function


//-------------------------------------
//static int  KeyCode = 1024;
static int  KeyUpCode = 1024; // Simple mode code
static int  bVkValue = 1024;
static int  pcKeyCode = 1024;
static int  pcShifts =1024;  // what shifts are needed to get the symbol on PC key

// Key-down timers
static int TimerA = 1024;
static int TimerB = 1024;
static int TimerC = 1024;
static int TimerD = 1024;
static int TimerE = 1024;
static int TimerF = 1024;
static int TimerRpt = 1024; // Autorepeat timer
static int gMinTimer = 1024;
static int gMaxTimer = 1024;
static int TimerMin = 8; //  8 value around 8 Used while determining paramC
//static int TimerMinTmp= 8;
static int paramC = 1024; //  Chord length, must be > TimerMin
static int gMinimumTime();
static int gMaximumTime(); // maximum key-down time at key release
static int gMaximumTimer = 0;
static void ClearTimers();
// Automatic repeat
static int AutoRptCnt = 50;
static int AutoRptDelay = 70;  // Autorepeat Delay, present value
static int AutoRptDelayF = 60;  // Autorepeat Delay for Full Mode (Timer based)
static int AutoRptDelayS = 50;  // Autorepeat Delay for Simple Mode
static bool AutoRpt = FALSE;
static bool AutoStop = FALSE; // prevent extra chars after autorpt key release 
static int ChordValueRpt = 32; // 0...63 // Autorepeat chord value
static int RptCharRef;
static void gShiftON();
static void gShiftOFF();
static void gShiftsOFF();
static bool AutoShiftsCleared = TRUE;
// Tybot
static int TybotCnt = 10;
static int TybotA = 0; // Tybot KeyA, 0 = up, 1 = down, 2 = keep down 
static int TybotB = 0; // Tybot KeyB
static int TybotC = 0; // Tybot KeyC
static int TybotD = 0; // Tybot KeyD
static int TybotE = 0; // Tybot KeyE
static int TybotF = 0; // Tybot KeyF
static bool HQ = FALSE;

// Key-down status
static bool Adown = FALSE;
static bool Bdown = FALSE;
static bool Cdown = FALSE;
static bool Ddown = FALSE;
static bool Edown = FALSE;
static bool Fdown = FALSE;
static bool AnyUp = FALSE; // Any Key goes Up => Check Chord
static bool AllUp = TRUE; // AllKeysUp <AKU> signal
static bool AKUsent = FALSE;
// Used for COM input
static bool AdownPrev = FALSE;
static bool BdownPrev = FALSE;
static bool CdownPrev = FALSE;
static bool DdownPrev = FALSE;
static bool EdownPrev = FALSE;
static bool FdownPrev = FALSE;



//VOID keybd_event(BYTE, BYTE, DWORD, DWORD);
BYTE bVk; BYTE bScan; DWORD dwFlags; DWORD dwExtraInfo;
static void SetCapsLock( BOOL bState );


//VOID mouse_event(DWORD dwMFlags, DWORD dx, DWORD dy, DWORD dwData,ULONG_PTR dwExtraInfo);
VOID mouse_event(DWORD, DWORD, DWORD, DWORD,ULONG_PTR);
DWORD dwMFlags; DWORD dx; DWORD dy; DWORD dwData; ULONG_PTR dwMExtraInfo;
// QWERTY keys as mouse buttons:
void LeftClickDown();
void LeftClickUp();
void RightClickDown();
void RightClickUp();
static bool mLdown = FALSE;  // Mouse button status (G and H keys) 
static bool mRdown = FALSE;
// GKOS keys as mouse
static bool gMouseMode = FALSE; 
static bool gMouseSHIFT = FALSE; // Shifts can be used with mouse too
static bool gMouseCtrl = FALSE; 
static bool gMouseAlt = FALSE; 
static bool gMouseScroll = FALSE;
static void gMouseShiftOFF();
static void gMouseCtrlOFF();
static void gMouseAltOFF();
 
//mouse (COM port in)
static bool btLDownDelta = FALSE;
static bool btLUpDelta = TRUE;
static bool btRDownDelta = FALSE;
static bool btRUpDelta = TRUE;
static void gSendMouse();
//Scroll by kbd
static void ScrollUp(); // SYMB DownArrow
static void ScrollDown(); // SYMB UpArrow
static void ScrollUpFast(); // SYMB DownArrow
static void ScrollDownFast(); // SYMB UpArrow
static void ScrollUpSlow(); // SYMB DownArrow
static void ScrollDownSlow(); // SYMB UpArrow

// Grammatics
static bool AccentedVovels = TRUE; // for decision on o/. a/
static bool AccentedVovelsTEMP = TRUE; // during word
static bool AccentedVovelsSPACE = TRUE; // during word change

//HQ
static bool qConvert = FALSE;
static bool qDown = FALSE;
static int qCode1 = 0;
static int qCode2 = 0;
static bool qSpace = FALSE;
static bool qShift = FALSE;
static bool qCAPS = FALSE;
static bool qCtrl = FALSE;
static bool qAlt = FALSE;
static int SpaceShift = 0; // Shift (CAPS) up to next Space
static int Space123 = 0;   // Use 123 mode up to next Space


//keyboard
static bool gShift = FALSE;
static bool gCAPS = FALSE;
static bool gCtrl = FALSE;
static bool gAlt = FALSE;
static bool gNumLock = FALSE;
static bool gScroll = FALSE;
static bool gAltGr = FALSE;
static bool gSYMB = FALSE;
static bool g123abc = FALSE;
static bool gESC = FALSE;
static bool gFUNC = FALSE;  // Shift for function keys F1...F12
static bool gWIN = FALSE;   // Shift for Windows keys (s, m, ....)
static bool gSPE = FALSE;   // Shift for Special functions
//static void gPressKey(int pcKeyCode);
//static void pcKeyDown(int pcKeyCode);
//static void pcKeyUp(int pcKeyCode);
static void gPressKey(int bVkValue);
static void pcKeyDown(int bVkValue);
static void pcKeyUp(int bVkValue);

static bool Painting = FALSE;
static void SetNumLock(BOOL bState);
static void SetScroll(BOOL bState);

//PC kbd
static bool pcCAPS = FALSE;
static bool pcAltGr = FALSE;

static int MouseData = 0; // Flags mouse data bytes 1...4. Else == 0
static int ChordonData = 0; // Flags Chordon shortcut data bytes 1...4. Else == 0
static int MouseByte[4]; // The 4 mouse data bytes
static int ChordonByte[3]; // The 3 chordon shortcut data bytes !a [AKU]

// COM ---
static int OpenPort();
static COMSTAT status;
static DWORD errors;
static char buffer[1024];
static int buffsize = sizeof(buffer);
static DWORD actual;
static bool Chords2COM = TRUE; // Indicates that Chord values+192 are
       // received on the COM port (every 10ms), not final KeyCodes.
       // Default must be TRUE for the status display to be correct.
static char pcCommPort[6] = "COM1";
static char PortNum[6] = "COM1";
static char tempstring[80];

static char tempstr; // temporary string
static int tempint; // temporary integer
static char buffer2[255];
static int buff2size = sizeof(buffer2);


static DCB dcb;
static HANDLE hCom;
static BOOL fSuccess;
static bool Done;

// Tools -----
static void gTybot();




//====================================functions ================================

// ----- gkosw.ini file -----
//SetLanguage(int Language);
//SetPCkeyboard(int PCkeyboard);
//bool USmode = TRUE;
//int  Language = 1;   // 1 = en, 2 = fi, 3 = se, 4 = de
//int  PCkeyboard = 2; // 1 = US, 2,3 = FIN/SWE,  4 = GER 
//bool SimpleMode = FALSE; //TRUE; // Simple mode = no timers used for chord detection
//int QwertySet = 1; // 1 = SDF/JKL(often works),
                     // 2 = ZXC/Arrows(seldom works)
                     // 3 = ZXC/NumPad 123(seldom works)
                     // 0 = Using COM input
//char PortNum[6] = "COM1";

int gReadParams()
{

	char rChar1[13]; // 'Do_not_edit'
    char rChar2[6]; // PortNum[6] e.g. COM1
    char rChar3[6]; // spare
    char rChar4[6]; // spare
    char rChar5[6]; // spare
	unsigned int rInt1;
	unsigned int rInt2;
	unsigned int rInt3;
	unsigned int rInt4;  
	unsigned int rInt5;  // spare ...
	unsigned int rInt6;
	unsigned int rInt7;
	unsigned int rInt8;
    unsigned int rInt9; // Window position X
	unsigned int rInt10;// Window position Y
	long rLong1;
	long rLong2;
	long rLong3;
	long rLong4;
	long rLong5;
	int rBool1;
	int rBool2;
	int rBool3; // spare
	int rBool4; // ...
	int rBool5;
	int rBool6;
	int rBool7;
	int rBool8;
	int rBool9;
	int rBool10;

	FILE *GKOSparams = fopen("gkosw.ini", "r+");
	
	if (!GKOSparams){
    MessageBox(NULL, "File gkosw.ini for parameters will be created at exit.","Running this program first time...", MB_OK | MB_ICONINFORMATION);                     
                     return 1;}
	
	fscanf(GKOSparams, "%s\n", rChar1);
	fscanf(GKOSparams, "%s\n", rChar2); // PortNum
	fscanf(GKOSparams, "%s\n", rChar3); //123456788
	fscanf(GKOSparams, "%s\n", rChar4); //123456788
	fscanf(GKOSparams, "%s\n", rChar5); //123456788
	fscanf(GKOSparams, "%d\n", &rInt1); // Language
	fscanf(GKOSparams, "%d\n", &rInt2); // QwertySet
	fscanf(GKOSparams, "%d\n", &rInt3); // PCkeyboard
	fscanf(GKOSparams, "%d\n", &rInt4); // NativeLang
	fscanf(GKOSparams, "%d\n", &rInt5);
	fscanf(GKOSparams, "%d\n", &rInt6);
	fscanf(GKOSparams, "%d\n", &rInt7);
	fscanf(GKOSparams, "%d\n", &rInt8);
	fscanf(GKOSparams, "%d\n", &rInt9); // Window position X
	fscanf(GKOSparams, "%d\n", &rInt10);// Window position Y
	fscanf(GKOSparams, "%d\n", &rLong1);// Date
	fscanf(GKOSparams, "%d\n", &rLong2);
	fscanf(GKOSparams, "%d\n", &rLong3);
	fscanf(GKOSparams, "%d\n", &rLong4);
	fscanf(GKOSparams, "%d\n", &rLong5);
	fscanf(GKOSparams, "%d\n", &rBool1); // USmode
	fscanf(GKOSparams, "%d\n", &rBool2); // SinpleMode
	fscanf(GKOSparams, "%d\n", &rBool3); // ComMode
	fscanf(GKOSparams, "%d\n", &rBool4);
	fscanf(GKOSparams, "%d\n", &rBool5);
	fscanf(GKOSparams, "%d\n", &rBool6);
	fscanf(GKOSparams, "%d\n", &rBool7);
	fscanf(GKOSparams, "%d\n", &rBool8);
	fscanf(GKOSparams, "%d\n", &rBool9);
	fscanf(GKOSparams, "%d\n", &rBool10);

	fclose(GKOSparams);

    strcpy(PortNum, rChar2);      
    Language = rInt1;
    QwertySet = rInt2;
    PCkeyboard = rInt3;
               if (PCkeyboard >= 10){PCkeyboard = 2;}
    NativeLang = rInt4;
    gWinPosX = rInt9;           
               if (gWinPosX == 0){gWinPosX = 100;}
    gWinPosY = rInt10;           
               if (gWinPosY == 0){gWinPosY = 100;}
    USmode = (bool) rBool1;
    SimpleMode = (bool) rBool2;
    ComMode = (bool) rBool3;

    if (QwertySet == 0){QwertySet = 1;}

if (ComMode) {
  //  MessageBox(NULL, "Going to open COM port.","gkosw.ini...", MB_OK | MB_ICONINFORMATION);                     
                    if(hCom){CloseHandle(hCom);}
                    //CloseHandle(hCom); //QwertySet = 0;
                    //strcpy(pcCommPort, "COM1");  
                    strcpy(pcCommPort, PortNum);  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                    }

if (!ComMode) {
                    if(hCom){CloseHandle(hCom);}
                    //QwertySet = 1; // Scan SDF/JKL keys
                    //MessageBox(NULL, "Type on QWERTY SDF/JKL keys. This mode supports character\n\n  overlap, Chordons and Chordon Shotcuts (781 words).\n\n   Note: Num Lock Must be OFF", "QWERTY Full Mode", MB_OK | MB_ICONINFORMATION);
                    if ((SimpleMode == FALSE)&&(QwertySet == 1)) {AutoRptDelay = AutoRptDelayF;
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY Full");      
                          }
                    if ((SimpleMode == FALSE)&&(QwertySet == 5)) {AutoRptDelay = AutoRptDelayF;
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: NumPad 789123");      
                          }
                    if ((SimpleMode == FALSE)&&(QwertySet == 6)) {AutoRptDelay = AutoRptDelayF;
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input:Modified NumPad");      
                          }
                    if ((SimpleMode == FALSE)&&(QwertySet == 7)) {AutoRptDelay = AutoRptDelayF;
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: NumPad 014-9/");      
                          }
                    if (SimpleMode == TRUE) {AutoRptDelay = AutoRptDelayS;
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY Simple");      
                          }
                    }
    
 return 0;   
}
// -----------
int gSaveParams()
{
	char gChar1[13]; // 'Do_not_edit'
    char gChar2[6];  // PortNum[6]
    char gChar3[10] = "123456789";  // Spare
    char gChar4[10] = "123456789";  // Spare
    char gChar5[10] = "123456789";  // Spare
	unsigned int gInt1 = 0;
	unsigned int gInt2 = 0;
	unsigned int gInt3 = 0;
	unsigned int gInt4 = 0;
	unsigned int gInt5 = 0;
	unsigned int gInt6 = 0;
	unsigned int gInt7 = 0;
	unsigned int gInt8 = 0;
	unsigned int gInt9 = 0;  // Window position X
	unsigned int gInt10 = 0; // Window position Y
	long gLong1;  long Mileage;
	long gLong2  = 0;
	long gLong3 = 0;
	long gLong4 = 0;
	long gLong5 = 0;
	int gBool1 = 0;
	int gBool2 = 0;
	int gBool3 = 0;
	int gBool4 = 0;
	int gBool5 = 0;
	int gBool6 = 0;
	int gBool7 = 0;
	int gBool8 = 0;
	int gBool9 = 0;
	int gBool10 = 0;

    Mileage = 20061115; // VERSION DATE

    strcpy(gChar1, "Do_not_edit");      
    strcpy(gChar2, PortNum);      
    gInt1 = Language;
    gInt2 = QwertySet;
    gInt3 = PCkeyboard;
    gInt4 = NativeLang;
    gInt9 = gWinPosX;
    gInt10 = gWinPosY;
    gLong1 = Mileage;
    gBool1 = (int) USmode;
    gBool2 = (int) SimpleMode;
    gBool3 = (int) ComMode;

	FILE *GKOSparams = fopen("gkosw.ini", "w");

	fprintf(GKOSparams, "%s\n", gChar1);
	fprintf(GKOSparams, "%s\n", gChar2);
	fprintf(GKOSparams, "%s\n", gChar3);
	fprintf(GKOSparams, "%s\n", gChar4);
	fprintf(GKOSparams, "%s\n", gChar5);
	fprintf(GKOSparams, "%d\n", gInt1);
	fprintf(GKOSparams, "%d\n", gInt2);
	fprintf(GKOSparams, "%d\n", gInt3);
	fprintf(GKOSparams, "%d\n", gInt4);
	fprintf(GKOSparams, "%d\n", gInt5);
	fprintf(GKOSparams, "%d\n", gInt6);
	fprintf(GKOSparams, "%d\n", gInt7);
	fprintf(GKOSparams, "%d\n", gInt8);
	fprintf(GKOSparams, "%d\n", gInt9);
	fprintf(GKOSparams, "%d\n", gInt10);
	fprintf(GKOSparams, "%d\n", gLong1);
	fprintf(GKOSparams, "%d\n", gLong2);
	fprintf(GKOSparams, "%d\n", gLong3);
	fprintf(GKOSparams, "%d\n", gLong4);
	fprintf(GKOSparams, "%d\n", gLong5);
	fprintf(GKOSparams, "%d\n", gBool1);
	fprintf(GKOSparams, "%d\n", gBool2);
	fprintf(GKOSparams, "%d\n", gBool3);
	fprintf(GKOSparams, "%d\n", gBool4);
	fprintf(GKOSparams, "%d\n", gBool5);
	fprintf(GKOSparams, "%d\n", gBool6);
	fprintf(GKOSparams, "%d\n", gBool7);
	fprintf(GKOSparams, "%d\n", gBool8);
	fprintf(GKOSparams, "%d\n", gBool9);
	fprintf(GKOSparams, "%d\n", gBool10);

	fclose(GKOSparams);

 return 0;   
}
/*
c   A single character
d 	An integer
e 	A floating-point number
f 	A floating-point number
g 	A floating-point number
h 	A short integer
i 	A decimal, a hexadecimal, or an octal integer
o 	An octal integer
s 	A string followed by a white space character
u 	An unsigned decimal integer
x 	A hexadecimal integer
*/
// ---------------------------
int gPaintBmp(HDC hdc)
{
//    MessageBox(NULL, "Going to Paint.","test...", MB_OK | MB_ICONINFORMATION);                     
      //BmpSelect = 2;
        //BITMAP bm;
        HDC hdcMem;
        hdcMem = CreateCompatibleDC(hdc);
        
        if (BmpSelect == 1){
                SelectObject(hdcMem, hbmGkos);}
        if (BmpSelect == 2){
                SelectObject(hdcMem, hbmGkosA);}
        if (BmpSelect == 3){
                SelectObject(hdcMem, hbmGkosSym);}
        if (BmpSelect == 4){
                SelectObject(hdcMem, hbmGkosNum);}
        if (BmpSelect == 5){
                SelectObject(hdcMem, hbmGkosCtr);}
        if (BmpSelect == 9){
                SelectObject(hdcMem, hbmGkosInfo);}

        BitBlt(hdc, 0, 30, bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCCOPY);
        //BitBlt(hdc, 0, 30, bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCPAINT);
        //BitBlt(hdc, 0, 30, bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCAND);
        DeleteDC(hdcMem);
}

//static int Language = 1; // 1 = en, 2 = fi, 3 = se, 4 = de

void SetLanguage(int Language)
     {
     switch(Language)
          {
          case 1: // en - English
               //USmode = TRUE; // select PC layout
               SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) " Lang: English"); 
               break;
          case 2: // fi - Finnish
               //USmode = FALSE; // select PC layout
               SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) " Lang: Finnish"); 
               break;
          case 3: // se - Swedish
               //USmode = FALSE; // select PC layout
               SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) " Lang: Swedish"); 
               break;
          case 4: // de - German
               //USmode = FALSE; // select PC layout
               SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) " Lang: German"); 
               break;
          default:
               Language = 1;
               //USmode = TRUE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) " Lang: English"); 
               break;                  
          }    
                     
     } 
// ----
void SetPCkeyboard(int PCkeyboard)
     {
     switch(PCkeyboard)
          {
          case 1: // US/UK PC keyboard layout
               USmode = TRUE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " PC kbd: US/UK"); 
               break;
          case 2:  // US/UK PC keyboard layout
               USmode = FALSE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " PC kbd: FIN/SWE"); 
               break;
          case 3:  // US/UK PC keyboard layout
               USmode = FALSE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " PC kbd: FIN/SWE"); 
               break;
          case 4:  // US/UK PC keyboard layout
               USmode = FALSE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " PC kbd: GERMAN"); 
               break;
          default:
               USmode = TRUE; // select PC leyout
               SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " PC kbd: US/UK"); 
               break;                  
          }             
     }

// ------------------------------ OPEN COM ----------------------------


int OpenPort()
{
    gMouseMode = FALSE; // Forget qwerty sdf/jkl mouse
    if (!ComMode){return(0);}
    
    hCom = CreateFile( pcCommPort,
                    GENERIC_READ | GENERIC_WRITE,
                    0,    // must be opened with exclusive-access
                    NULL, // no security attributes
                    OPEN_EXISTING, // must use OPEN_EXISTING
                    0,    // not overlapped I/O
                    NULL  // hTemplate must be NULL for comm devices
                    );
/* 
    hCom = CreateFile( "COM1",
        GENERIC_READ | GENERIC_WRITE,
        0,    // exclusive access 
        NULL, // default security attributes 
        OPEN_EXISTING,
        FILE_FLAG_OVERLAPPED,
        NULL 
        );
   If hFile was not opened with FILE_FLAG_OVERLAPPED, WaitCommEvent does not return
   until one of the specified events or an error occurs.
*/
   if (hCom == INVALID_HANDLE_VALUE) 
   {
       // Handle the error.  
       MessageBox(NULL, "THIS PORT CANNOT BE OPENED !\n\nIt may be used by an other application.", pcCommPort, MB_OK | MB_ICONERROR);
       strcpy(PortNum, "No input selected!");
    //   MessageBox (NULL, "CreateFile (COM) failed with error 1", hCom, MB_OK);
       return (1);
   }

   // Build on the current configuration, and skip setting the size
   // of the input and output buffers with SetupComm.

   fSuccess = GetCommState(hCom, &dcb);

   if (!fSuccess) 
   {
      // Handle the error.
    MessageBox (NULL, "GetCommState failed with error 2", pcCommPort, MB_OK);
      return (2);
   }
      
   // Fill in DCB: 9600 bps, 8 data bits, no parity, and 1 stop bit.
   dcb.BaudRate = CBR_9600;     // set the baud rate
   dcb.ByteSize = 8;             // data size, xmit, and rcv
   dcb.Parity = NOPARITY;        // no parity bit
   dcb.StopBits = ONESTOPBIT;    // one stop bit

   fSuccess = SetCommState(hCom, &dcb);

   if (!fSuccess) 
   {
      // Handle the error.
    MessageBox (NULL, "SetCommState failed with error 3", pcCommPort, MB_OK);
    strcpy(PortNum, "No input selected!");
      return (3);
   }
   
   if (fSuccess)
   {
    strcpy (PortNum, pcCommPort);
//    MessageBox (NULL, "Serial port successfully opened.", pcCommPort, MB_OK);    
    return (0);
   }
  
} // end of COM OpenPort() function--------------

// **************************************************************
// Check for incoming characters and send corresponding kbd input
// **************************************************************
int CheckCom()
{
       int Temp; // the received GKOS Keycode        

              // The ClearCommError function retrieves information about
              // a communications error and reports the current status of
              // a communications device.  It clears the device's error flag
              // to enable additional input and output (I/O) operations. 
              // If the function succeeds, the return value is nonzero.                              
                 if (ClearCommError(hCom, &errors, &status))
                  {
                         if (status.cbInQue) // There is Data to read
                         // cbInQue:
                         // Number of bytes received by the serial provider but
                         // not yet read by a ReadFile operation.
                         {
 /*
            ReadFile:
               lpBuffer
                [out] Pointer to the buffer that receives the data read from the file.
            nNumberOfBytesToRead
                [in] Number of bytes to be read from the file. 
            lpNumberOfBytesRead
                [out] Pointer to the variable that receives the number of bytes read
            lpOverlapped
                [in] Pointer to an OVERLAPPED structure. 
 */                               
                                  if (!ReadFile(hCom, &buffer, 1, &actual, NULL))
                                   {
                                   //throw (new exception("ReadFile returned FALSE."));
                                   MessageBox (NULL, "Port read error!", "COM test", MB_OK);

                                   return (1);
                                   }
//               BOOL ReadFile(HANDLE hFile, LPVOID lpBuffer, DWORD nNumberOfBytesToRead,
//               LPDWORD lpNumberOfBytesRead, LPOVERLAPPED lpOverlapped);

                              // Bytes received              
                              // Read data from buffer & do stuff
                              // Send keys to system using e.g. SendInput()
                              // int KeyCode; // the received GKOS Keycode  
                              
       Temp = (int)*buffer;   // The latest Keycode received must be
                                 // type-casted from double to int.
       Temp = Temp & 255;  // Remove extra bits.                                    
                              gCheckComInput(Temp); // This passes parameters to gCheckInput(...)
                              return (2); // (2) indicates that data was received
                             } // end of There is data to read

                       } // ClearCommError

    return (0);  // (0) indicates that nothing was received                      
} // end of function                      

//------------------------------end of COM / CheckCom() --------------------------------------


// Without timer:
// LowLevelKeyboardProc => KeyCode => gSK1() => gCheckInput() => gSK(KeyCode) => gPressKey

//

void gSK1()
{
     // Simple mode: Timers are not used:
     if (SimpleMode){
                     gRef = KeyUpCode;
                     if (!AutoStop){gSK2(gRef);}
                     //gSK2(gRef);
                     ChordValue = 0;
                     if (gChordNow() == 0){AutoStop = FALSE;}
                     }
     
     // Timer mode: Do nothing here, Timers will tell the Chord and KeyCode
 }
 
// Selct range of GKOS Ref based on 123abc and SYMB settings 
static int gSetRef (int gRef0)
{
        // gRef0 = KeyCode;
          if (gRef0 == 0){return 0;}

          if ((g123abc == TRUE) && (gSYMB == FALSE)){
             if (gRef0 <= 30){gRef0 = gRef0 + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }

          if ((gSYMB == TRUE) && (g123abc == FALSE)){
             if (gRef0 <= 41){gRef0 = gRef0 + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }

          if ((gSYMB == TRUE) && (g123abc == TRUE)){
             if ((gRef0 <= 41) && (gRef0 >= 31)){gRef0 = gRef0 + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }
          return gRef0;
}

// With timer:
// LowLevelKeyboardProc => just control timers
void gSK2 (int gRef)
{
     gRef = gSetRef(gRef);
          if (gRef == 0){return;}

/* // Moved above
        // gRef = KeyCode;
          if (gRef == 0){return;}

          if ((g123abc == TRUE) && (gSYMB == FALSE)){
             if (gRef <= 30){gRef = gRef + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }

          if ((gSYMB == TRUE) && (g123abc == FALSE)){
             if (gRef <= 41){gRef = gRef + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }

          if ((gSYMB == TRUE) && (g123abc == TRUE)){
             if ((gRef <= 41) && (gRef >= 31)){gRef = gRef + 63;}
//      MessageBox(0, "KeyCode = gRef + 64", "Test...", MB_OK);          
          }
*/

          if (QwertySet == 1)
          {
                    KeyCode = gRef;    
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case 19: // S 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 4: // D 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 6: // F 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 10: // J 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 11: // K 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 12: // L 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
                 
                    
          } // end of Switch
          return;
          } // QwertySet = 1


// Another set of keys ZXC / Arrow keys
// - does not give 6 simultaneous characters on my PC!
   if (QwertySet == 2)
      {
                    KeyCode = gRef;
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case 90: // Z
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 88: // X 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 67: // C
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_LEFT: // LeftArrow 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_DOWN: // DownArrowl
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_RIGHT: // RightArrow
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
          } // end of Switch
        } // end of QwertySet = 2

// Another set of keys ZXC / NumPad 123
// - does not give 6 simultaneous characters on my PC!
   if (QwertySet == 3)
      {
                    KeyCode = gRef;
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case 90: // Z
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 88: // X 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 67: // C
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD1: // 61 VK_NUMPAD1
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD2: // 62 VK_NUMPAD2
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD3: // 63 VK_NUMPAD3
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
          } // end of Switch
        } // end of QwertySet = 3


// Another set of keys 45t / i90
// Special relaxed position of hands
// - 6 simultaneous characters on my laptop!
   if (QwertySet == 4)
      {
                    KeyCode = gRef;
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case 0x34: // 4 34h
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 0x35: // 5 35h
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 0x54: // T 54h
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 0x49: // I 49h
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case 0x39: // 9 39h
                    ChordValue = 9;
                    gCheckInput(KeyCode);
                    return;

                case 0x30: // 0 30h
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
          } // end of Switch
        } // end of QwertySet = 4


// NumPad 789/123
// (Mey be needed: VK_NUMLOCK (90h))
          if (QwertySet == 5)
          {
                    KeyCode = gRef;    
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case VK_NUMPAD9: // 9 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD8: // 8 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD7: // 7 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD1: // 1 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD2: // 2 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD3: // 3 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
                 
                    
          } // end of Switch
          return;
          } // QwertySet = 5



// Special NumPad ,/- / 741
// (Mey be needed: VK_NUMLOCK (90h))
          if (QwertySet == 6)
          {
                    KeyCode = gRef;    
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case VK_DECIMAL: // , 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_DIVIDE: // / 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_SUBTRACT: // - 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD7: // 7 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD4: // 4 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD1: // 1 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
                 
                    
          } // end of Switch
          return;
          } // QwertySet = 6



// Special NumPad 014 -9/
// (Mey be needed: VK_NUMLOCK (90h))
          if (QwertySet == 7)
          {
                    KeyCode = gRef;    
                    pcKeyCode = iChar[KeyCode];
          switch (KeyCode)
          {                
                case VK_NUMPAD0:
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD1:
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD4:
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_SUBTRACT: //-
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_NUMPAD9: // 9 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                case VK_DIVIDE: // 1 
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                    return;

                 default: ;
                    ChordValue = 0;
                    gCheckInput(KeyCode);
                 
                 
                    
          } // end of Switch
          return;
          } // QwertySet = 7




} // end of function         

// ---------------------------------------------------------------------

//wParam is one of these: WM_KEYDOWN, WM_KEYUP, WM_SYSKEYDOWN, or WM_SYSKEYUP
//lparam is pointer to KBDLLHOOKSTRUCT
LRESULT LowLevelKeyboardProc(int nCode, WPARAM wParam, LPARAM lParam)
{
   if(nCode == HC_ACTION) // HC_ACTION tells that the wParam lParam contain information about a keyboard message.
   {
      KBDLLHOOKSTRUCT *HookStruct = (KBDLLHOOKSTRUCT *)lParam;
//        if(HookStruct->vkCode == VK_NEXT)
//        if(HookStruct->vkCode == VK_ESCAPE){GkosON = !GkosON;}

        // Press ESC to turn GKOS mode back ON  
        if(HookStruct->vkCode == VK_ESCAPE){
                              GkosON = TRUE;
                              HQ = FALSE;
                              SetLanguage(Language); // For Correct statusbar display
                              SetPCkeyboard(PCkeyboard); // Update the status/shifts display
                              return FALSE;}


// ------------------------------
    qCode1 = HookStruct->vkCode;


   if (HQ)
   {
         qConvert = FALSE;
         GkosON = FALSE; // Turn OFF GKOS mode
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "HQ L mode - Esc!"); 

         return FALSE; //DISABLE HQ FUNCTION!!
   
    switch (qCode1)
        {
      case 0x20: // Space = Shift
         if (wParam == WM_KEYDOWN) {qSpace = TRUE; return TRUE;}
         if (wParam == WM_KEYUP) {
                    if ((qSpace == TRUE) && (qDown == FALSE))
                                  {
                    qSpace = FALSE;
                    keybd_event(0x20, 0, 0, 0);
                    keybd_event(0x20, 0, KEYEVENTF_KEYUP, 0);
                    return TRUE;
                                  }
                                 }
         return TRUE;
      case 0xA0: // Left Shift 0xA0
            if (wParam == WM_KEYDOWN) 
               {if (qShift == TRUE){
                    qShift = FALSE;
                    pcKeyUp(0xA1);
                    SetCapsLock(FALSE);
                    SetCapsLock(TRUE);
                    //gPressKey(0x41); //a TEST
                    qCAPS = TRUE;
                    return TRUE;}
               
                if (qCAPS == TRUE){
                    SetCapsLock(TRUE);
                    SetCapsLock(FALSE);
                    //gPressKey(0x42); //b TEST
                    qCAPS = FALSE;
                    qShift = FALSE;
                    return TRUE;}
               
               qShift = TRUE;
               pcKeyDown(0xA1);// Right SHIFT
                     //pcKeyDown(0x10);// SHIFT
                     //gPressKey(0x14); // Press CAPS LOCK (down & up)
                     //keybd_event(0x41, 0, 0, 0); //TEST
                     //keybd_event(0x41, 0, KEYEVENTF_KEYUP, 0);

               return TRUE;} //TEST
         return TRUE;
      case 0x11: // Ctrl
         if (wParam == WM_KEYDOWN) {qCtrl = TRUE;}
         //if (wParam == WM_KEYUP) {qCtrl = FALSE;}
         return TRUE;
      case 0x12: // Alt
         if (wParam == WM_KEYDOWN){qAlt = TRUE;}
         //if (wParam == WM_KEYUP) {qAlt = FALSE;}
         return TRUE;

      // -----   

      case 0xDC: //   > + 
            qConvert = TRUE; qCode2 = 0xBB; break;
      case 0x31: //  1 > 0 
            qConvert = TRUE; qCode2 = 0x30; break;
      case 0x32: //  2 > 9 
            qConvert = TRUE; qCode2 = 0x39; break;
      case 0x33: //  3 > 8 
            qConvert = TRUE; qCode2 = 0x38; break;
      case 0x34: //  4 > 7 
            qConvert = TRUE; qCode2 = 0x37; break;
      case 0x35: //  5 > 6
            qConvert = TRUE; qCode2 = 0x38; break;


      case 0x09: //  TAB > BackSpace 
            qConvert = TRUE; qCode2 = 0x08; break;
      case 0x14: //  CAPS >  (US: Apostrophe ') 
            qConvert = TRUE; qCode2 = 0xDE; break;


      case 0x51: //  Q > P 
            qConvert = TRUE; qCode2 = 0x50; break;
      case 0x57: //  W > O 
            qConvert = TRUE; qCode2 = 0x4F; break;
      case 0x45: //  E > I 
            qConvert = TRUE; qCode2 = 0x49; break;
      case 0x52: //  R > U 
            qConvert = TRUE; qCode2 = 0x55; break;
      case 0x54: //  T > Y
            qConvert = TRUE; qCode2 = 0x59; break;


      case 0x41: //  A >  
            qConvert = TRUE; qCode2 = 0xC0; break;
      case 0x53: //  S > L
            qConvert = TRUE; qCode2 = 0x4C; break;
      case 0x44: //  D > K 
            qConvert = TRUE; qCode2 = 0x4B; break;
      case 0x46: //  F > J
            qConvert = TRUE; qCode2 = 0x4A; break;
      case 0x47: //  G > H  
            qConvert = TRUE; qCode2 = 0x48; break;


      
      case 0xE2: //  < >  
            qConvert = TRUE; qCode2 = 0xDD; break;
      case 0x5A: //  Z > - 
            qConvert = TRUE; qCode2 = 0xBD; break;
      case 0x58: //  X > . 
            qConvert = TRUE; qCode2 = 0xBC; break;
      case 0x43: //  C > , 
            qConvert = TRUE; qCode2 = 0xBE; break;
      case 0x56: // V > M
            qConvert = TRUE; qCode2 = 0x4D; break;
      case 0x42: // B > N
            qConvert = TRUE; qCode2 = 0x4E; break;




         } // end of swirch

             
         // Rest of keys 
         if (wParam == WM_KEYDOWN)
         {
              if (qConvert == FALSE) {
              return FALSE;}
         }
         if (wParam == WM_KEYUP)
         {

               if (qConvert == FALSE) {
                  if (qShift == TRUE) {qShift = FALSE;
                     pcKeyUp(0xA1);// Right SHIFT
                       }
               return FALSE;}
         }
 
 
         // Potentially converted keys            
         if (wParam == WM_KEYDOWN) 
             {
             qDown = TRUE;
             return TRUE;
             }
         if (wParam == WM_KEYUP) 

            if (qSpace == FALSE)
             {
             if (qConvert == FALSE) {return TRUE;}
                // always convert CAPS > Enter
                if (qCode1 == 0x14){qCode1 = 0x0D;}             
             keybd_event(qCode1, 0, 0, 0);
             keybd_event(qCode1, 0, KEYEVENTF_KEYUP, 0);
             qSpace = FALSE;
             qDown = FALSE;            
               if (qShift == TRUE) {qShift = FALSE;
               pcKeyUp(0xA1);}// Right SHIFT
             return TRUE;
             }

            if (qSpace == TRUE)
             {
             if (qConvert == FALSE) {return TRUE;}
             keybd_event(qCode2, 0, 0, 0);
             keybd_event(qCode2, 0, KEYEVENTF_KEYUP, 0);
             qSpace = FALSE;
             qDown = FALSE;            
               if (qShift == TRUE) {qShift = FALSE;
               pcKeyUp(0xA1);}// Right SHIFT             
             return TRUE;
             }
         
                    



   }
                              
// ------------------------------

   if (QwertySet == 1)
   {
    if (GkosON == FALSE){return FALSE;}             
//      if(HookStruct->vkCode == 83)
        // S D F = 83 68 70 = c b a
        // J K L = 74 75 76 = d e f
    switch (HookStruct->vkCode)
        {
      case 83: // S
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 68: // D
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 70: // F
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case 74: // J
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 75: // K
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 76: // L
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
         // Mouse clicks on WERTY keys G and H:
      case 71: // G = Left Mouse Button
         if (wParam == WM_KEYDOWN){ 
             if (mLdown == FALSE){LeftClickDown();}
                 mLdown = TRUE;}
         if (wParam == WM_KEYUP) {mLdown = FALSE; LeftClickUp();}
         return TRUE;
      case 72: // H = Right Mouse Button
         if (wParam == WM_KEYDOWN){ 
             if (mRdown == FALSE){RightClickDown();}
                 mRdown = TRUE;}
         if (wParam == WM_KEYUP) {mRdown = FALSE; RightClickUp();}
         return TRUE;
                  
      default:
            //  return TRUE; //TEST ONLY

         // for unknown reason (not to turn off gkos mode in one case):
         if (qCode1 == 0x99){return FALSE;} // (0x99 = 153) <= chordon 'of_the_'

         //  if (NationalSC){return FALSE;}
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         
         
         if (qCode1 == 0x51){
                    if (Cdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();         
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
        // SendMessage(hWndStatusBar, SB_SETTEXT, 2, (LPARAM) "                  "); // This cannot be put back easily
         
         return FALSE;

        //          }
        // return TRUE;

         
         } // end of switch
         } // end of QwertySet = 1
         
   if (QwertySet == 2)
   {
//      if(HookStruct->vkCode == 83)
        // Z X C = 26 24 3
        // Arrows = VK_LEFT VK_DOWN VK_RIGHT
    switch (HookStruct->vkCode)
        {
      case 90: // Z
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 88: // X
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 67: // C
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case VK_LEFT: // LeftArrow
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_DOWN: // DownArrow
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_RIGHT: // RightArrow
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      default: ;
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         if (qCode1 == 0x51){
                    if (Fdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         return FALSE;
         }
         } // end of QwertySet = 2
         

   if (QwertySet == 3)
   {
//      if(HookStruct->vkCode == 83)
        // Z X C = 26 24 3
        // NumPad 1 2 3 =  61 62 63  - NOTE: Num Lock must be ON !!!
    switch (HookStruct->vkCode)
        {
      case 90: // Z
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 88: // X
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 67: // C
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case VK_NUMPAD1: // 61 VK_NUMPAD1
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD2: // 62 VK_NUMPAD1
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD3: // 63 VK_NUMPAD3
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      default: ;
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         if (qCode1 == 0x51){
                    if (Fdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         return FALSE;
          }
         } // end of QwertySet = 3
         


   if (QwertySet == 4)
   {
//      if(HookStruct->vkCode == 83)
        // 45T = 34h 35h 
        // I90 =     39h 30h  - NOTE: Num Lock must be OFF !!!
    switch (HookStruct->vkCode)
        {
      case 0x34: // 4
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 0x35: // 5
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 0x54: // T
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case 0x49: // I
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0; AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 0x39: // 9
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case 0x30: // 0
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; AutoRpt = FALSE; gSK1();}
         return TRUE;

      default: ;
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         if (qCode1 == 0x51){
                    if (Fdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         return FALSE;
         }
         } // end of QwertySet = 4
         

   if (QwertySet == 5)  // NumPad 9 8 7 / 1 2 3
   {
    if (GkosON == FALSE){return FALSE;}             
//      if(HookStruct->vkCode == 83)
        // 
        //
    switch (HookStruct->vkCode)
        {
      case VK_NUMPAD9: // 9
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD8: // 8
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD7: // 7
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case VK_NUMPAD1: // 1
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD2: // 2
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD3: // 3
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
         // Mouse clicks on WERTY keys G and H:
//      case 71: // G = Left Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mLdown == FALSE){LeftClickDown();}
//                 mLdown = TRUE;}
//         if (wParam == WM_KEYUP) {mLdown = FALSE; LeftClickUp();}
//         return TRUE;
//      case 72: // H = Right Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mRdown == FALSE){RightClickDown();}
//                 mRdown = TRUE;}
//         if (wParam == WM_KEYUP) {mRdown = FALSE; RightClickUp();}
//         return TRUE;

      case VK_NUMPAD0: return TRUE; // Disabled
      case VK_NUMPAD4: return TRUE; // Disabled
      case VK_NUMPAD5: return TRUE; // Disabled
      case VK_NUMPAD6: return TRUE; // Disabled
                  
      default:
            //  return TRUE; //TEST ONLY

            return FALSE;  // Always keep GKOS mode in 789/123 Input mode

         // for unknown reason (not to turn off gkos mode in one case):
         if (qCode1 == 0x99){return FALSE;} // (0x99 = 153) <= chordon 'of_the_'

         //  if (NationalSC){return FALSE;}
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         
         
         if (qCode1 == 0x51){
                    if (Cdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         
         return FALSE;

        //          }
        // return TRUE;

         
         } // end of switch
         } // end of QwertySet = 5



   if (QwertySet == 6)  // NumPad , / - / 7 4 1  (Special)
   {
    if (GkosON == FALSE){return FALSE;}             
//      if(HookStruct->vkCode == 83)
        // 
        //
    switch (HookStruct->vkCode)
        {
      case VK_DECIMAL: // ,
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_DIVIDE: // /
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_SUBTRACT: // -
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case VK_NUMPAD7: // 7
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD4: // 4
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD1: // 1
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
         // Mouse clicks on WERTY keys G and H:
//      case 71: // G = Left Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mLdown == FALSE){LeftClickDown();}
//                 mLdown = TRUE;}
//         if (wParam == WM_KEYUP) {mLdown = FALSE; LeftClickUp();}
//         return TRUE;
//      case 72: // H = Right Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mRdown == FALSE){RightClickDown();}
//                 mRdown = TRUE;}
//         if (wParam == WM_KEYUP) {mRdown = FALSE; RightClickUp();}
//         return TRUE;

      case VK_NUMPAD0: return TRUE; // Disabled
      case VK_NUMPAD8: return TRUE; // Disabled
      case VK_NUMPAD5: return TRUE; // Disabled
      case VK_NUMPAD2: return TRUE; // Disabled
                  
      default:
            //  return TRUE; //TEST ONLY

            return FALSE;  // Always keep GKOS mode in 789123 Input mode

         // for unknown reason (not to turn off gkos mode in one case):
         if (qCode1 == 0x99){return FALSE;} // (0x99 = 153) <= chordon 'of_the_'

         //  if (NationalSC){return FALSE;}
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         
         
         if (qCode1 == 0x51){
                    if (Cdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         
         return FALSE;

        //          }
        // return TRUE;

         
         } // end of switch
         } // end of QwertySet = 6


// NumPad , / - / 7 4 1  (Special)
   if (QwertySet == 7)  // NumPad 0 1 4 - 9 /  = abcdef (Standard)
   {
    if (GkosON == FALSE){return FALSE;}             
//      if(HookStruct->vkCode == 83)
        // 
        //
    switch (HookStruct->vkCode)
        {
      case VK_NUMPAD4: //c = 4
         if (wParam == WM_KEYDOWN) {Cdown = TRUE; TimerC = 0; ChordValue = ChordValue + 4; TimerRpt = 0; AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Cdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD1: //b = 1
         if (wParam == WM_KEYDOWN) {Bdown = TRUE; TimerB = 0; ChordValue = ChordValue + 2; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Bdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD0: //a = 0
         if (wParam == WM_KEYDOWN) {Adown = TRUE; TimerA = 0; ChordValue = ChordValue + 1; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Adown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;

      case VK_SUBTRACT: //d = -
         if (wParam == WM_KEYDOWN) {Ddown = TRUE; TimerD = 0; ChordValue = ChordValue + 8; TimerRpt = 0;  AutoRpt = FALSE; }
         if (wParam == WM_KEYUP) {Ddown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_NUMPAD9: //e = 9
         if (wParam == WM_KEYDOWN) {Edown = TRUE; TimerE = 0; ChordValue = ChordValue + 16; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Edown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
      case VK_DIVIDE: //f = /
         if (wParam == WM_KEYDOWN) {Fdown = TRUE; TimerF = 0; ChordValue = ChordValue + 32; TimerRpt = 0;  AutoRpt = FALSE;}
         if (wParam == WM_KEYUP) {Fdown = FALSE; AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gSK1();}
         return TRUE;
         // Mouse clicks on WERTY keys G and H:
//      case 71: // G = Left Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mLdown == FALSE){LeftClickDown();}
//                 mLdown = TRUE;}
//         if (wParam == WM_KEYUP) {mLdown = FALSE; LeftClickUp();}
//         return TRUE;
//      case 72: // H = Right Mouse Button
//         if (wParam == WM_KEYDOWN){ 
//             if (mRdown == FALSE){RightClickDown();}
//                 mRdown = TRUE;}
//         if (wParam == WM_KEYUP) {mRdown = FALSE; RightClickUp();}
//         return TRUE;

      case VK_NUMPAD7: return TRUE; // Disabled
      case VK_NUMPAD8: return TRUE; // Disabled
      case VK_NUMPAD5: return TRUE; // Disabled
      case VK_NUMPAD2: return TRUE; // Disabled
      case VK_NUMPAD3: return TRUE; // Disabled
      case VK_NUMPAD6: return TRUE; // Disabled
      case VK_MULTIPLY: return TRUE; // Disabled
      case VK_ADD: return TRUE; // Disabled
      case VK_DECIMAL: return TRUE; // Disabled
                  
      default:
            //  return TRUE; //TEST ONLY

            return FALSE;  // Always keep GKOS mode in 789/123 Input mode

         // for unknown reason (not to turn off gkos mode in one case):
         if (qCode1 == 0x99){return FALSE;} // (0x99 = 153) <= chordon 'of_the_'

         //  if (NationalSC){return FALSE;}
         GkosON = FALSE; // Turn OFF GKOS mode when other keys are pressed
         
         
         if (qCode1 == 0x51){
                    if (Cdown == TRUE){HQ = TRUE;}}
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
         
         return FALSE;

        //          }
        // return TRUE;

         
         } // end of switch
         } // end of QwertySet = 7




         
   } // end of if... HC_ACTION

   return CallNextHookEx(MouseHook, nCode, wParam, lParam);
}


void InstallHook(void)
{
   if (ComMode){return;} // COM mode, no hooks in use
   
   if(MouseHook)
      return;

   if(HookFailed) // Aready known that hook install will not succeed
      return;      

   MouseHook = SetWindowsHookEx(WH_KEYBOARD_LL, (HOOKPROC  )LowLevelKeyboardProc, GetModuleHandle(0), 0);
   if(!MouseHook)
   {
      HookFailed = TRUE;           
      MessageBox(0, "Failed to install QWERTY Keyboard Hook.\nOnly COM port input can be used.", "Windows XP preferred.", MB_OK);
   }
}

void UnInstallHook(void)
{
   if (ComMode){return;} // COM mode, no hooks in use

   if(MouseHook)
      UnhookWindowsHookEx(MouseHook);
   MouseHook = 0;
}


// ----------------------------------------------

DWORD CheckMenuItem(
    HMENU hmenu,	// handle to menu 
    UINT uIDCheckItem,	// menu item to check or uncheck
    UINT uCheck	// menu item flags 
   );

//==============================================================================
LRESULT CALLBACK WndProc(HWND    hWnd,
                         UINT    Msg,
                         WPARAM  wParam,          
                         LPARAM  lParam)
{
    switch (Msg)
    {
        case WM_CREATE:
        {
         // Small edit box
            hWndEditBox = CreateWindow(
            "EDIT", 
            "Click here and type on SDF/JKL", 
            WS_VISIBLE | WS_CHILD
            | ES_MULTILINE | ES_AUTOVSCROLL,
            38, 
            54, 
            112, 
            60, 
            hWnd, 
            (HMENU)IDC_EDITBOX_TEXT, 
            (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE), 
            NULL);

            if (!hWnd){
                MessageBox(NULL, "Small Edit Box Failed.", "Error", MB_OK | MB_ICONERROR);}
         // ---    

         // status bar    
            hWndStatusBar = CreateWindowEx(
            0,
            STATUSCLASSNAME,
            NULL,
            WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP,
            0,
            0,
            0,
            0,
            hWnd,
            (HMENU)IDC_STATUSBAR,
            (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE),
            NULL);

            if (!hWndStatusBar)
            {
                MessageBox(NULL, "Failed To Create The Status Bar", "Error", MB_OK | MB_ICONERROR);
                return 0;
            }

            int iStatusWidths[] = {100, 200, -1};
            char text[256];

            SendMessage(hWndStatusBar, SB_SETPARTS, 3, (LPARAM)iStatusWidths);

            SendMessage(hWndStatusBar, SB_SETTEXT, 0,(LPARAM)"PC Kbd: FIN/SWE");
            SendMessage(hWndStatusBar, SB_SETTEXT, 1,(LPARAM)"Lang: English");
            //sprintf(text, "%d", 3);
            //SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)text);
            if (SimpleMode){SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY  Simple");}
            if (!SimpleMode){SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY Full mode");}
            ShowWindow(hWndStatusBar, SW_SHOW);
        // end of status bar    

        // toolbar -----           
            TBADDBITMAP tbab; 
            //TBBUTTON tbb[3]; 
            TBBUTTON tbb[20]; // BUTTONS + SPACES

            hWndToolBar = CreateWindowEx(
            0, 
            TOOLBARCLASSNAME, 
            (LPSTR)NULL,
            WS_CHILD,
            0,
            0, 
            0, 
            0,
            hWnd, 
            (HMENU)IDTB_TOOLBAR, 
            (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE),
            NULL);

            if (!hWndToolBar)
            {
                MessageBox(NULL, "Tool Bar Failed.", "Error", MB_OK | MB_ICONERROR);
                return 0;
            }

            SendMessage(hWndToolBar, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);

            SendMessage(hWndToolBar, TB_SETBITMAPSIZE, (WPARAM)0, (LPARAM)MAKELONG(20, 20));

            tbab.hInst = (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE); 
            // Bmp MUST be 440x20 pixels, Bit depth 8 (!!!),
            //     Frame count 1, Vert/Hor Resolution 96/96 dpi
            tbab.nID   = IDB_TOOLBITMAP; 
            SendMessage(hWndToolBar, TB_ADDBITMAP, (WPARAM) 2, (LPARAM) &tbab);

            ZeroMemory(tbb, sizeof(tbb));

            tbb[0].iBitmap = 1; 
            tbb[0].idCommand = TB_TEST1; 
            tbb[0].fsState = TBSTATE_ENABLED; 
            tbb[0].fsStyle = TBSTYLE_BUTTON; 

            tbb[1].fsStyle = TBSTYLE_SEP;
 
            tbb[2].iBitmap = 2; 
            tbb[2].idCommand = TB_TEST2; 
            tbb[2].fsState = TBSTATE_ENABLED; 
            tbb[2].fsStyle = TBSTYLE_BUTTON; 

            tbb[3].fsStyle = TBSTYLE_SEP;
 
            tbb[4].iBitmap = 3; 
            tbb[4].idCommand = TB_TEST3; 
            tbb[4].fsState = TBSTATE_ENABLED; 
            tbb[4].fsStyle = TBSTYLE_BUTTON; 

            tbb[5].fsStyle = TBSTYLE_SEP;
 
            tbb[6].iBitmap = 4; 
            tbb[6].idCommand = TB_TEST4; 
            tbb[6].fsState = TBSTATE_ENABLED; 
            tbb[6].fsStyle = TBSTYLE_BUTTON; 

            tbb[7].fsStyle = TBSTYLE_SEP;
 
            tbb[8].iBitmap = 5; 
            tbb[8].idCommand = TB_TEST5; 
            tbb[8].fsState = TBSTATE_ENABLED; 
            tbb[8].fsStyle = TBSTYLE_BUTTON; 

            tbb[9].fsStyle = TBSTYLE_SEP;
 
            tbb[10].iBitmap = 6; 
            tbb[10].idCommand = TB_TEST6; 
            tbb[10].fsState = TBSTATE_ENABLED; 
            tbb[10].fsStyle = TBSTYLE_BUTTON; 

            tbb[11].fsStyle = TBSTYLE_SEP;
 
            tbb[12].iBitmap = 7; 
            tbb[12].idCommand = TB_TEST7; 
            tbb[12].fsState = TBSTATE_ENABLED; 
            tbb[12].fsStyle = TBSTYLE_BUTTON; 

            tbb[13].fsStyle = TBSTYLE_SEP;
 
            tbb[14].iBitmap = 8; 
            tbb[14].idCommand = TB_TEST8; 
            tbb[14].fsState = TBSTATE_ENABLED; 
            tbb[14].fsStyle = TBSTYLE_BUTTON; 

            tbb[15].fsStyle = TBSTYLE_SEP;
 
            tbb[16].iBitmap = 9; 
            tbb[16].idCommand = TB_TEST9; 
            tbb[16].fsState = TBSTATE_ENABLED; 
            tbb[16].fsStyle = TBSTYLE_BUTTON; 

            tbb[17].fsStyle = TBSTYLE_SEP;
 
            tbb[18].iBitmap = 10; 
            tbb[18].idCommand = TB_TEST10; 
            tbb[18].fsState = TBSTATE_ENABLED; 
            tbb[18].fsStyle = TBSTYLE_BUTTON; 


 
            SendMessage(hWndToolBar, TB_ADDBUTTONS, 19, (LPARAM)&tbb); // BUTTONS + SPACES
// Uncomment this when the Toolbar will be used:
            ShowWindow(hWndToolBar, SW_SHOW); 
        // end of toolbar    

        // ---bitmaps--- 
                   
        // BMP With = 347, Height = 134 when window size is: 356, 240
         hbmGkos = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSBMP));
         if(!hbmGkos){
         MessageBox(hWnd, "Load of BMP 1 resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}
         GetObject(hbmGkos, sizeof(bm), &bm);

         hbmGkosA = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSABMP));
         if(!hbmGkosA){
         MessageBox(hWnd, "Load of BMP A resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}

         hbmGkosCtr = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSCTRBMP));
         if(!hbmGkosCtr){
         MessageBox(hWnd, "Load of BMP Ctr resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}

         hbmGkosNum = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSNUMBMP));
         if(!hbmGkosNum){
         MessageBox(hWnd, "Load of BMP Num resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}

         hbmGkosSym = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSSYMBMP));
         if(!hbmGkosSym){
         MessageBox(hWnd, "Load of BMP Sym resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}

         hbmGkosInfo = LoadBitmap(hInstBmp, MAKEINTRESOURCE(GKOSINFOBMP));
         if(!hbmGkosInfo){
         MessageBox(hWnd, "Load of BMP Info resource failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return -1;
            		}
         
//-------------------------------------------

           // reset key-down timers    
           TimerA = 0; TimerB = 0; TimerC = 0; TimerD = 0; TimerE = 0; TimerF = 0;
           TimerRpt = 0;
            // Set Timer on
           SetTimer(hWnd, idTimer1, nTimerDelay, NULL);
                      
           // Set Charcater tables and set Keyboard hook               

           gReadParams(); // from gkosw.ini file
           SetTables();              
           SetLanguage(Language);
           SetPCkeyboard(PCkeyboard);
           
           // Set Process Priority to HIGH
           hProcess = GetCurrentProcess();
           gStat = SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);

           
           InstallHook();
        return 0;                                            
           }
        break;




//----------------------------------------------
//    ... Dealing with commands ...
//----------------------------------------------    

        case WM_COMMAND: 
        { 
            switch(LOWORD(wParam))
            { 

                case IDM_FILE_MONITOR:
                      gDriver = 0;
                      BmpSelectTEMP = 1;
                      if(hbmGkos && hbmGkosA)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                              // Full edit box
                     if (!ComMode){
                        hWndEditBox = CreateWindow(
                        "EDIT", 
                        "Click here and type on SDF/JKL", 
                        WS_VISIBLE | WS_CHILD
                        | ES_MULTILINE | ES_AUTOVSCROLL,
                        0, 
                        32, 
                        344, 
                        130, 
                        hWnd, 
                        (HMENU)IDC_EDITBOX_TEXT, 
                        (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE), 
                        NULL);
                        }
                     if (ComMode){  // COM port mode
                        hWndEditBox = CreateWindow(
                        "EDIT", 
                        "Click here and type on GKOS IR Keyboard ", 
                        WS_VISIBLE | WS_CHILD
                        | ES_MULTILINE | ES_AUTOVSCROLL,
                        0, 
                        32, 
                        344, 
                        130, 
                        hWnd, 
                        (HMENU)IDC_EDITBOX_TEXT, 
                        (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE), 
                        NULL);
                        }

            
                        if (!hWnd){
                            MessageBox(NULL, "Edit Box Failed.", "Error", MB_OK | MB_ICONERROR);} 
                break;

                case IDM_FILE_DRIVER:
                      gDriver = 1;
                      BmpSelectTEMP = 1;
                      if(hbmGkos)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;

                case IDM_FILE_EXIT:
                    UnInstallHook();
                    gSaveParams();
                    ClearShifts();
                    // Set Process Priority back to NORMAL
                    hProcess = GetCurrentProcess();
                    gStat = SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);
                    PostMessage(hWnd, WM_CLOSE, 0, 0);
                    Done = TRUE;
                break;
                 
                // Two modes of qwerty input:
                case IDM_INPUT_QWERTY:
                    if(hCom){CloseHandle(hCom);}
                    ComMode = FALSE; // Scan QWERTY keys
                    InstallHook(); QwertySet = 1;
                    MessageBox(NULL, "Type on QWERTY SDF/JKL keys. This mode supports character\n\n  overlap, Chordons and Chordon Shotcuts (781 words).\n\n    You can use keys G and H as Mouse Buttons!\n\n        --- Note: Num Lock must be OFF ---", "QWERTY Full Mode", MB_OK | MB_ICONINFORMATION);
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY Full mode");
                    AutoRptDelay = AutoRptDelayF;
                    SimpleMode = FALSE;
                break; 
                case IDM_INPUT_QWERTY_SIMPLE:
                    if(hCom){CloseHandle(hCom);}
                    ComMode = FALSE;// Scan QWERTY keys
                    InstallHook(); QwertySet = 1;
                    MessageBox(NULL, "Type on QWERTY SDF/JKL keys. This mode supports\n\nfast entering of single characters without overlap.\n\n You can use keys G and H as Mouse Buttons!", "QWERTY Simple Mode", MB_OK | MB_ICONINFORMATION);
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: QWERTY Simple");
                    AutoRptDelay = AutoRptDelayS;
                    SimpleMode = TRUE;
                    // ----------------------------------------------

               //     CheckMenuItem((HMENU)IDR_MENU, IDM_INPUT_QWERTY_SIMPLE, MF_CHECKED);
/*
    HMENU hmenu,	// handle to menu 
    UINT uIDCheckItem,	// menu item to check or uncheck
    UINT uCheck	// menu item flags 
   );
*/
//                    ModifyMenu((HMENU)IDR_MENU, IDM_INPUT_QWERTY_SIMPLE, MF_CHECKED, IDM_INPUT_QWERTY_SIMPLE, MF_STRING); 
/*
    HMENU hMnu,	// handle of menu 
    UINT uPosition,	// menu item to modify
    UINT uFlags,	// menu item flags 
    UINT uIDNewItem,	// menu item identifier or handle of drop-down menu or submenu
    LPCTSTR lpNewItem	// menu item content 
   );	
*/
                break; 

                case IDM_INPUT_QWERTY_NUMPAD:
                    if(hCom){CloseHandle(hCom);}
                    ComMode = FALSE;// Scan QWERTY keys
                    InstallHook(); QwertySet = 5;
                    MessageBox(NULL, "Type on NumPad 789 123 keys. This mode supports character\n\n  overlap, Chordons and Chordon Shotcuts (781 words).\n\n *** NOTE that you must have NumLock ON! ***", "NumPad Full Mode", MB_OK | MB_ICONINFORMATION);
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: NumPad 789 123");
                    AutoRptDelay = AutoRptDelayF;
                    SimpleMode = FALSE;
                break; 

                case IDM_INPUT_QWERTY_NUMPAD_SPEC:
                    if(hCom){CloseHandle(hCom);}
                    ComMode = FALSE;// Scan QWERTY keys
                    InstallHook(); QwertySet = 6;
                    MessageBox(NULL, "Type on special modified NumPad -+ENTER  741 keys. This mode supports character\n\n  overlap, Chordons and Chordon Shotcuts (781 words).\n\n *** NOTE that you must have NumLock ON! ***", "NumPad Full Mode", MB_OK | MB_ICONINFORMATION);
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: Special NumPad");
                    AutoRptDelay = AutoRptDelayF;
                    SimpleMode = FALSE;
                break; 

                case IDM_INPUT_QWERTY_NUMPAD_3:
                    if(hCom){CloseHandle(hCom);}
                    ComMode = FALSE;// Scan QWERTY keys
                    InstallHook(); QwertySet = 7;
                    MessageBox(NULL, "Type on NumPad 014 -91 keys. This mode supports character\n\n  overlap, Chordons and Chordon Shotcuts (781 words).\n\n *** NOTE that you must have NumLock ON! ***", "NumPad Full Mode", MB_OK | MB_ICONINFORMATION);
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)"Input: NumPad 014 -9/");
                    AutoRptDelay = AutoRptDelayF;
                    SimpleMode = FALSE;
                break; 


                // COM Port input selection
                case IDM_INPUT_COM1:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;
                    strcpy(pcCommPort, "COM1");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM2:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;;
                    strcpy(pcCommPort, "COM2");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM3:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;
                    strcpy(pcCommPort, "COM3");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM4:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;;
                    strcpy(pcCommPort, "COM4");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM5:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;
                    strcpy(pcCommPort, "COM5");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM6:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;;
                    strcpy(pcCommPort, "COM6");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                break; 
                case IDM_INPUT_COM7:
                    UnInstallHook();
                    CloseHandle(hCom); ComMode = TRUE;
                    strcpy(pcCommPort, "COM7");  
                    OpenPort();
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)tempstring);
                break; 


                // --------------------------


                case IDM_VIEW_PIC: 
                      gDriver = 1;
                      BmpSelectTEMP = 1;
                      BmpSelect = 1;
                      if(hbmGkos)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case IDM_VIEW_LET: 
                     BmpSelectTEMP = 2;
                     BmpSelect = 2;
                      if(hbmGkosA)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case IDM_VIEW_SYM: 
                     BmpSelectTEMP = 3;
                     BmpSelect = 3;
                      if(hbmGkosSym)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case IDM_VIEW_NUM: 
                     BmpSelectTEMP = 4;
                     BmpSelect = 4;
                      if(hbmGkosNum)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case IDM_VIEW_FUN: 
                     BmpSelectTEMP = 5;
                     BmpSelect = 5;
                      if(hbmGkosCtr)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                // Toolbar functions:
                case TB_TEST1: // Monitor
                   // MessageBox(NULL, "Not yet implemented.", "Draft...", MB_OK | MB_ICONINFORMATION);
                      gDriver = 0;
                      BmpSelectTEMP = 1;
                      BmpSelect = 1;
                      if(hbmGkos && hbmGkosA)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                              // Full edit box
                        hWndEditBox = CreateWindow(
                        "EDIT", 
                        "Click here and type on SDF/JKL", 
                        WS_VISIBLE | WS_CHILD
                        | ES_MULTILINE | ES_AUTOVSCROLL,
                        0, 
                        32, 
                        344, 
                        130, 
                        hWnd, 
                        (HMENU)IDC_EDITBOX_TEXT, 
                        (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE), 
                        NULL);
            
                        if (!hWnd){
                            MessageBox(NULL, "Edit Box Failed.", "Error", MB_OK | MB_ICONERROR);}
 
                break;
                case TB_TEST2: // Driver
                   // MessageBox(NULL, "Not yet implemented.", "Draft...", MB_OK | MB_ICONINFORMATION);
                      gDriver = 1;
                      BmpSelectTEMP = 1;
                      BmpSelect = 1;
                      if(hbmGkos)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case TB_TEST3: // Letters
                     BmpSelectTEMP = 2;
                     BmpSelect = 2;
                      if(hbmGkosA)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case TB_TEST5: // Symbols
                   // MessageBox(NULL, "Not yet implemented.", "Draft...", MB_OK | MB_ICONINFORMATION);
                     BmpSelectTEMP = 3;
                     BmpSelect = 3;
                      if(hbmGkosSym)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case TB_TEST4: // Numbers
                   // MessageBox(NULL, "Not yet implemented.", "Draft...", MB_OK | MB_ICONINFORMATION);
                     BmpSelectTEMP = 4;
                     BmpSelect = 4;
                      if(hbmGkosNum)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case TB_TEST6:  // Controls
                //    MessageBox(NULL, "Not yet implemented.", "Draft...", MB_OK | MB_ICONINFORMATION);
                     BmpSelectTEMP = 5;
                     BmpSelect = 5;
                      if(hbmGkosCtr)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                case TB_TEST7: 
                    MessageBox(NULL, "Not yet ready to show.", "National layouts", MB_OK | MB_ICONINFORMATION);
                break;
                case TB_TEST8: 
                    //MessageBox(NULL, "Not yet implemented.", "TYBOT - Typing Robot", MB_OK | MB_ICONINFORMATION);
                    gTybot(); // Typer Robot to test speed of possible PC kbd GKOS entry
                break;
                case TB_TEST9: 
                    MessageBox(NULL, "Not yet implemented.", "USER SPEED TEST", MB_OK | MB_ICONINFORMATION);
                    // Test user typing skills
                break;
                case TB_TEST10: 
                     BmpSelectTEMP = 9;
                     BmpSelect = 9;
                      if(hbmGkosInfo)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break;
                
                
                //--------- 
                case IDM_HELP_GKOS: 
                     BmpSelectTEMP = 9;
                     BmpSelect = 9;
                      if(hbmGkosInfo)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
                break; 
                case IDM_HELP_ABOUT: 
                    MessageBox(NULL, "GKOS Monitor/Driver\n\n - Version 0.61 - \n\n by Seppo Tiainen\n Veikkola, FINLAND\n\n   1 March 2007", "About...", MB_OK | 0);
                break; 

                // PC keyboard layout (PC kbd scan codes)
                case IDM_KBD_FINSWE: 
                    PCkeyboard = 2;
                    SetPCkeyboard(PCkeyboard);
                    MessageBox(NULL, "The PC keyboard shall have FIN/SWE layout.","GKOS - PC kbd layout", MB_OK | MB_ICONINFORMATION);
                break; 
                case IDM_KBD_USUK: 
                    PCkeyboard = 1;
                    SetPCkeyboard(PCkeyboard);
                    MessageBox(NULL, "The PC keyboard shall have US/UK layout.","GKOS - PC kbd layout", MB_OK | MB_ICONINFORMATION);

                    //Language (shortcuts and national characters)
                case IDM_KBD_ENG:
                    Language = 1;  
                    NativeLang = 1;
                    SetLanguage(Language);
                    //MessageBox(NULL, "GKOS Language set to English","GKOS - Language", MB_OK | MB_ICONINFORMATION);
                break; 
                case IDM_KBD_FIN:
                    Language = 2;  
                    NativeLang = 2;
                    SetLanguage(Language);
                    //MessageBox(NULL, "GKOS Language set to Finnish","GKOS - Language", MB_OK | MB_ICONINFORMATION);
                break; 
                case IDM_KBD_SWE:
                    Language = 3;  
                    NativeLang = 3;
                    SetLanguage(Language);
                    //MessageBox(NULL, "GKOS Language set to Swedish","GKOS - Language", MB_OK | MB_ICONINFORMATION);
                break; 
                case IDM_KBD_GER:
                    Language = 4;  
                    NativeLang = 4;
                    SetLanguage(Language);
                    //MessageBox(NULL, "GKOS Language set to German","GKOS - Language", MB_OK | MB_ICONINFORMATION);
                break; 


            } 
            return 0; 
        } 
        break;

        
    //---bitmap---
    case WM_PAINT:
    if (gDriver == 0);
    {
       {
        if(hbmGkos && hbmGkosA)
                   {
        PAINTSTRUCT ps;
        HDC hdc = BeginPaint(hWnd, &ps);
        gPaintBmp(hdc);
        EndPaint(hWnd, &ps);
                   }
        }
    }
    break;

//-------------------------------------------------------
        case WM_TIMER:
             
        // Change helper screen
        //---------------------
        if ((BmpSelectTEMP != BmpSelect)&&(gDriver == 1))
        //if (BmpSelectTEMP != BmpSelect)
        {  
                     BmpSelect = BmpSelectTEMP;
                      if(hbmGkosA)
                     {
                        HDC hdcWindow;
                        hdcWindow = GetDC(hWnd);
                        gPaintBmp(hdcWindow);            
                        ReleaseDC(hWnd, hdcWindow);
                     }
  
         }
             
             
             
        //----------------------     
             
             
             // Tybot delay Counter
             if (TybotCnt != 0)
             {
             TybotCnt--;
                  if (TybotCnt == 0){
                     if (TybotA == 1){keybd_event(0x46, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     if (TybotB == 1){keybd_event(0x44, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     if (TybotC == 1){keybd_event(0x53, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     if (TybotD == 1){keybd_event(0x4a, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     if (TybotE == 1){keybd_event(0x4b, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     if (TybotF == 1){keybd_event(0x4c, 0, KEYEVENTF_KEYUP, 0);} // key up                   
                     }
             }
             // Autorepeat:
//             if (AutoShiftsCleared == FALSE)
//                              {
//                              gShiftsOFF();
//                              AutoShiftsCleared = TRUE;                              
//MessageBox(NULL, "Shifts Cleared after Autorepeat.","Test point...", MB_OK | MB_ICONINFORMATION);
//                              }
                                                
             //if (AutoRpt){AutoShiftsCleared = FALSE;}              
                           
             if (!AllUp){if (AutoRpt == TRUE){AutoRpt = FALSE; //gShiftsOFF();
                                             }//default
             AutoRptCnt--;
                  if (AutoRptCnt == 0){ AutoStop = TRUE; // stop extra chars after key release
                  AutoShiftsCleared = FALSE; // Prepare for clearing shifts after autorepeat
                  AutoRpt = TRUE; AutoRptCnt = 5; // Repeat prompt speed delay
                          //gSK2(1); //TEST
                          // chord may change at key release:
                          if (RptCharRef != gCharRef[gChordNow()]){
                                         AutoRpt = FALSE; //gShiftsOFF();
                                         RptCharRef = gCharRef[gChordNow()];
                                         AutoRptCnt = AutoRptDelay;
                                                                   }
                                     }
                        }
              
             if (AutoRpt){ // repeat safe characters 1...52
                          RptCharRef = gCharRef[gChordNow()];
                          if (RptCharRef < 53){
                          // ADD HERE: Check that he present chord is non-zero!               
                          //gSK2(1); //TEST
                          gSK2(RptCharRef);
                          AutoRpt = FALSE;//default
                                             }
                          }
              
                           
             // -----------
             // Timers used, chord detection based on key-down periods:
             AllUp = TRUE; //default

             if (Adown){AllUp = FALSE; if (TimerA < 256){++TimerA;}}
             if (Bdown){AllUp = FALSE; if (TimerB < 256){++TimerB;}}
             if (Cdown){AllUp = FALSE; if (TimerC < 256){++TimerC;}}
             if (Ddown){AllUp = FALSE; if (TimerD < 256){++TimerD;}}
             if (Edown){AllUp = FALSE; if (TimerE < 256){++TimerE;}}
             if (Fdown){AllUp = FALSE; if (TimerF < 256){++TimerF;}}
             if (AllUp == FALSE){
                       AKUsent = FALSE; if (TimerRpt < 256){++TimerRpt;}
                       if (gMaximumTimer < 256){++gMaximumTimer;}
                       }

             // -----------
             if (AnyUp) {AutoRptCnt = AutoRptDelay;

                             if (AutoShiftsCleared == FALSE)
                              {
                              gShiftsOFF();
                              AutoShiftsCleared = TRUE;                              
//MessageBox(NULL, "Shifts Cleared after Autorepeat.","Test point...", MB_OK | MB_ICONINFORMATION);
                              }

                              if (AutoRpt == TRUE){
                              AutoRpt = FALSE;
                                     }
                         }
             if (SimpleMode){
                          AnyUp = FALSE; // default (due to autorepeat)
                          break;} // Timers not used,
             // just detect chord when any key goes up, done in gSK1(), 
             // and check autorepeat


             // values:
                      // 5...10 take as a single letter
                      // 50... start autorepeat

              // AnyUp: Any key goes up
              // AllUp: All keys are up        

              // Latest Chord = ChordValue2
              // Possible previous Chord = ChordValue1 (from gCheckBack())

              if (AnyUp) { //ChordValue = 0;
                                                                              
                          paramC = gMinimumTime();
                          //TimerMinTmp = 4 + ((int)(paramC/16));
                          ChordValue = gChord(); // Get Chord C/2 ago
                          if (ChordValue == 0){ChordValue = gCheckFastHit();}
                          //ChordValue = gChordNow(); // TEST
                          ChordValue2 = ChordValue;
 
                          gCheckBack();
                          ClearTimers(); 
                                            
                          AnyUp = FALSE; // default
                          paramC = 16;   // default
// ------------ OUTPUT OF CHRACTERS --------------------------------------
                          if (ChordValue1 > 0){
                          // ***** Output previous chord character ****          
                                gRef = gCharRef[ChordValue1];
                                if (!AutoStop){gSK2(gRef);}
                                         }
                          // ***** Output latest chord character ****          
                          gRef = gCharRef[ChordValue2];
// if (ChordValue2 == 63){MessageBox(NULL, "123-ABC pressed.","Test point...", MB_OK | MB_ICONINFORMATION);}
                          if (!AutoStop){gSK2(gRef);}
                          }
                          
                          // <<AKU>> 
                          //  AKUsent = FALSE;

              if ((AllUp == TRUE) && (AKUsent == FALSE)){
                AKUsent = TRUE;
                gCheckInput(253);
                AutoStop = FALSE; // default
// MessageBox(NULL, "Returned from gSK2(gRef)","Test point...", MB_OK | MB_ICONINFORMATION);
                  }
               if (AllUp){gMaximumTimer = 0;}  
               ChordValue0 = gChordNow(); // Save this for the next round            
//      if (TimerRpt>45){gSK(Chord());}//MessageBox(0, "TimerRpt > 45", "Test...", MB_ICONEXCLAMATION | MB_OK);}
              break;
              //return 0;
                          
//-------------------------------------------------------

        //----------
        case WM_SIZE:
            SendMessage(hWndStatusBar, WM_SIZE, 0, 0);
            SendMessage(hWndToolBar, TB_AUTOSIZE, 0, 0);

        break;
/*
Window position:
BOOL GetWindowRect(HWND hWnd, LPRECT lpRect);
- hWnd Identifies the window. 
- lpRect Points to a RECT structure that receives the screen
  coordinates of the upper-left and lower-right corners of the window. 

RECT Structure:
typedef struct _RECT {    // rc 
    LONG left; 
    LONG top; 
    LONG right; 
    LONG bottom; 
} RECT; 
*/

        case WM_CLOSE:
            UnInstallHook();
            // Get WINDOWPOS here gWinPosX & gWinPosY
            gSaveParams();
            ClearShifts();
            // Set Process Priority back to NORMAL
            hProcess = GetCurrentProcess();
            gStat = SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);            
            DeleteObject(hbmGkos);
            DestroyWindow(hWnd);
        break;

        case WM_DESTROY:
            UnInstallHook();
            // Get WINDOWPOS here gWinPosX & gWinPosY
            gSaveParams();
            ClearShifts();
            // Set Process Priority back to NORMAL
            hProcess = GetCurrentProcess();
            gStat = SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);            
            DeleteObject(hbmGkos);
            PostQuitMessage(0);
            Done = TRUE;
        break;

        default:
            return (DefWindowProc(hWnd, Msg, wParam, lParam));
    }

    return 0;
}

INT WINAPI WinMain( HINSTANCE  hInstance,
                    HINSTANCE  hPrevInstance,
                    LPSTR      lpCmdLine,
                    INT        nCmdShow )
{
    InitCommonControls();

    WNDCLASSEX    wc;
    //---bitmap---
    hInstBmp = hInstance;
    //------------
    wc.cbSize           = sizeof(WNDCLASSEX);
    wc.style            = 0;
    wc.lpfnWndProc      = (WNDPROC)WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
    wc.hIconSm          = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    //wc.hbrBackground    = (HBRUSH)(COLOR_WINDOW + 1);
    //wc.hbrBackground = (HBRUSH) (COLOR_BTNFACE+1);
    //wc.hbrBackground = (HBRUSH) (7); // 0=transparant, 6=white, 7=black
    //wc.hbrBackground = (HBRUSH) (g_rgbBackground);
    wc.hbrBackground   = (HBRUSH) GetStockObject(LTGRAY_BRUSH);

    //wc.lpszMenuName     = NULL;
    wc.lpszMenuName     = MAKEINTRESOURCE(IDR_MENU);    
    wc.lpszClassName    = ClassName;

    if (!RegisterClassEx(&wc))
    {
        MessageBox(NULL, "Failed to Register the Window Class.", "Error", MB_OK | MB_ICONERROR);
        return 0;
    }
    HWND    hWnd;

    hWnd = CreateWindowEx(
    WS_EX_CLIENTEDGE,
    ClassName,
    "GKOS Monitor/Driver v.0.61",
    WS_OVERLAPPEDWINDOW,
    gWinPosX, //CW_USEDEFAULT, // position x
    gWinPosY, //CW_USEDEFAULT, // position y
    356, // Window width
    240, // Window height
    NULL,
    NULL,
    hInstance,
    NULL);

    if (!hWnd)
    {
        MessageBox(NULL, "Main Window Creation Failed.", "Error", MB_OK | MB_ICONERROR);
        return 0;
    }

    ShowWindow(hWnd, SW_SHOW);
    UpdateWindow(hWnd);

    MSG    Msg;
  /*
    while (GetMessage(&Msg, NULL, 0, 0))
    {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }
 */

// /*
 // This is an option to the while loop above for COM port polling
              
              // The GetMessage function retrieves a message from the calling thread's
              // message queue. The function dispatches incoming sent messages until a
              // posted message is available for retrieval. - Unlike GetMessage, the
              // PeekMessage function does not wait for a message to be posted before
              // returning:       
              // bRet = PeekMessage(&Msg,NULL,0,0,PM_REMOVE) != 0 
              // If a message is available, the return value is nonzero.
              // If no messages are available, the return value is zero.                     
        //while( (bRet = GetMessage(&Msg,NULL,0,0)) != 0)
        
        BOOL bRet;
        Done = FALSE;
        do
        {
             // ------ While using COM port input: -----
             if (ComMode)
             {
                // Poll COM port:
                bRet = CheckCom();
                if (bRet == 0) // There was nothing at COM port to receive
                {
                SleepEx(10,FALSE); //  DWORD dwMilliseconds, BOOL bAlertable 
                // Wait 30 ms only if the COM buffer was empty
                }
                
                // Poll Windows Messages:
                bRet = PeekMessage(&Msg,NULL,0,0,PM_REMOVE) != 0 ;
                if (bRet == 0)
                {
                //break;
                }
                if (bRet == -1)
                {
                 // handle the error and possibly exit
                }
                if (bRet != 0)
                {
                // Message available.

                HWND hActiveWindow = GetActiveWindow();
                // The IsWindow function determines whether the specified 
                // window handle identifies an existing window.
                // The IsDialogMessage function determines whether a message is intended
                // for the specified dialog box and, if it is, processes the message.
                   if (!IsWindow(hActiveWindow) || !IsDialogMessage(hActiveWindow,&Msg))
                   {
                // The TranslateMessage function translates virtual-key messages into 
                // character messages. The character messages are posted to the calling
                // thread's message queue, to be read the next time the thread calls the
                // GetMessage or PeekMessage function.
                        TranslateMessage(&Msg);
                // The DispatchMessage function dispatches a message to a window procedure.
                // It is typically used to dispatch a message retrieved by the GetMessage
                // function.        
                        DispatchMessage(&Msg);                       
                    }
                } // end of bRet != 0
                
             }  // end of (QwertySet == 0)
             
             // ------ While using QWERTY input:
             if (!ComMode)             
                {
                GetMessage(&Msg, NULL, 0, 0);
                        TranslateMessage(&Msg);
                        DispatchMessage(&Msg);
                }  // end of (!ComMode)
             // --------------------------------
                        
        } while (!Done);
// */
//---------------end of option---------------------
    return Msg.wParam;
}
//----------------------End of WinMain()-----------------------------------

// Get Chord value paramC/2 clock periods ago
static int gChord()
{
       int ChordTmp;
         ChordTmp = 1*(TimerA > paramC/2) +
                    2*(TimerB > paramC/2) +       
                    4*(TimerC > paramC/2) +       
                    8*(TimerD > paramC/2) +       
                   16*(TimerE > paramC/2) +       
                   32*(TimerF > paramC/2);
         return ChordTmp;
}

// Get Chord value now
static int gChordNow() 
{
       int ChordTmp1;
         ChordTmp1 = 1*(Adown) +
                    2*(Bdown) +       
                    4*(Cdown) +       
                    8*(Ddown) +       
                   16*(Edown) +       
                   32*(Fdown);
         return ChordTmp1;
}

//--- Check if this was a single fast stroke of key(s)
// In that case, it needs to be detected simply by chord value, i.e.
// what keys have been down since all keys were up 
static int gCheckFastHit()
{
       int ChordTmp2 = 0;
       
//       if (gMaximumTimer <= 8){
//       if ((gMaximumTimer <= TimerMin) && (gMaximumTimer >= 2))
       if (gMaximumTimer <= TimerMin)
       {
       gMaximumTimer = 100; // Stop extra characters
       ChordTmp2 = ChordValue0;
       }
       
       return ChordTmp2;       
}

// ------------
static void ClearTimers()
{TimerA = 0; TimerB = 0; TimerC = 0; TimerD = 0; TimerE = 0; TimerF = 0;
 TimerRpt = 0;
}

static void gCheckBack()
{
 // Latest Chord = ChordValue2 <= ChordValue;
 // Possible previous Chord = ChordValue1:
 ChordValue1 = 0;
 if (TimerA > TimerMin){TimerA = TimerA - paramC;
    if (TimerA > 0.75*paramC){ChordValue1 = ChordValue1 + 1;}
    } 
 if (TimerB > TimerMin){TimerB = TimerB - paramC;
    if (TimerB > 0.75*paramC){ChordValue1 = ChordValue1 + 2;}
    } 
 if (TimerC > TimerMin){TimerC = TimerC - paramC;
    if (TimerC > 0.75*paramC){ChordValue1 = ChordValue1 + 4;}
    }
 if (TimerD > TimerMin){TimerD = TimerD - paramC;
    if (TimerD > 0.75*paramC){ChordValue1 = ChordValue1 + 8;}
    } 
 if (TimerE > TimerMin){TimerE = TimerE - paramC;
    if (TimerE > 0.75*paramC){ChordValue1 = ChordValue1 + 16;}
    } 
 if (TimerF > TimerMin){TimerF = TimerF - paramC;
    if (TimerF > 0.75*paramC){ChordValue1 = ChordValue1 + 32;}
    } 
    //ChordValue1 = 1; 
}

// Get maximum timer value before key release
static int gMaximumTime()
{
       int gMaxTimer = 1;

        if (TimerA > gMaxTimer) {gMaxTimer = TimerA;}
        if (TimerB > gMaxTimer) {gMaxTimer = TimerB;}
        if (TimerC > gMaxTimer) {gMaxTimer = TimerC;}
        if (TimerD > gMaxTimer) {gMaxTimer = TimerD;}
        if (TimerE > gMaxTimer) {gMaxTimer = TimerE;}
        if (TimerF > gMaxTimer) {gMaxTimer = TimerF;}

       return gMaxTimer;
}

// Find paramC
static int gMinimumTime()
{
int gMinTimer = 255; // 100 is minimum that works ok
//int gMaxTimer = 1;
//// Option 1
//if (TimerA > gMaxTimer) {gMaxTimer = TimerA;}
//if (TimerB > gMaxTimer) {gMaxTimer = TimerB;}
//if (TimerC > gMaxTimer) {gMaxTimer = TimerC;}
//if (TimerD > gMaxTimer) {gMaxTimer = TimerD;}
//if (TimerE > gMaxTimer) {gMaxTimer = TimerE;}
//if (TimerF > gMaxTimer) {gMaxTimer = TimerF;}
//
//TimerMin = 4 + (int)(gMaxTimer/16);
//TimerMin = 8;
// Option 2
if ((TimerA > TimerMin)&&(TimerA < gMinTimer)){gMinTimer = TimerA;}
if ((TimerB > TimerMin)&&(TimerB < gMinTimer)){gMinTimer = TimerB;}
if ((TimerC > TimerMin)&&(TimerC < gMinTimer)){gMinTimer = TimerC;}
if ((TimerD > TimerMin)&&(TimerD < gMinTimer)){gMinTimer = TimerD;}
if ((TimerE > TimerMin)&&(TimerE < gMinTimer)){gMinTimer = TimerE;}
if ((TimerF > TimerMin)&&(TimerF < gMinTimer)){gMinTimer = TimerF;}

//if (gMinTimer == 0){gMinTimer = 2;}
return gMinTimer;
//return gMaxTimer;
}
void DrawGkos(HDC hdc)
{
   HDC hdcMemory;
   hdcMemory = CreateCompatibleDC(hdc);

//   SelectObject(hdcMemory, hbmMask);
//   BitBlt(hdc, ballX, ballY, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCAND);

   SelectObject(hdcMemory, hbmGkos);
   BitBlt(hdc, 0, 0, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCPAINT);
//   BitBlt(hdc, 0, 0, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCPAINT);

   DeleteDC(hdcMemory);
}
//==============================================================================
// This Scans the GKOS kbd state when Chord values+192 are received through the COM port
// So, same routines as for QWERTY SDF/JKL entry can be used. WM_TIMER checks timers etc.
bool static gUpdateGKOSstatus(int KeyCodeCOM)
{
   //return FALSE;
   
   bool gRTS;
// Check which keys are up and which are down
// Is any key going up right now?
// What is the Chord value right now?

   Adown = ((1 & KeyCodeCOM)  == 1);
   Bdown = ((2 & KeyCodeCOM)  == 2);
   Cdown = ((4 & KeyCodeCOM)  == 4);
   Ddown = ((8 & KeyCodeCOM)  == 8);
   Edown = ((16 & KeyCodeCOM) == 16);
   Fdown = ((32 & KeyCodeCOM) == 32);

         ChordValue = 1*(Adown) +
                      2*(Bdown) +       
                      4*(Cdown) +       
                      8*(Ddown) +       
                     16*(Edown) +       
                     32*(Fdown);    
         
   if (KeyCodeCOM == 0){AnyUp = FALSE;} // added for v.0.51 to avoid being stuck
                     
       //   ChordValue = 14; // 't' = TEST              
    gRTS = FALSE; // TRUE if any key goes up => send a char in Simple mode
                     
    //if (GkosON == FALSE){return FALSE;}             
                                          
   if (Adown != AdownPrev){
       if (Adown) {TimerA = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Adown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
   if (Bdown != BdownPrev){
       if (Bdown) {TimerB = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Bdown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
   if (Cdown != CdownPrev){
       if (Cdown) {TimerC = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Cdown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
   if (Ddown != DdownPrev){
       if (Ddown) {TimerD = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Ddown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
   if (Edown != EdownPrev){
       if (Edown) {TimerE = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Edown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
   if (Fdown != FdownPrev){
       if (Fdown) {TimerF = 0; TimerRpt = 0;  AutoRpt = FALSE;}
       if (!Fdown){AnyUp = TRUE; KeyUpCode = gCharRef[ChordValue]; TimerRpt = 0; AutoRpt = FALSE; gRTS = TRUE;}
             }
      
      // TEMPORARY SOLUTION: Always disable SimpleMode in Chords2COM mode
         if (SimpleMode == TRUE){      // Something fishy with this: SimpleMode will not be disabled this way....    
                    SimpleMode = FALSE; // TEMP setting
                    strcpy(tempstring, "Input: ");
                    strcat(tempstring, PortNum);
                    if (SimpleMode){strcat(tempstring, " Simple");}
                    if (!SimpleMode){strcat(tempstring, " Full mode");}
                    SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM)tempstring);
                    }
       // end of TEMP SOLUTION
      
       
      if (gRTS){ // Simple mode. BUT THIS DOES NOT WORK YET FOR SOME REASON!!!!!
               // MessageBox (NULL, "We are here.", " y ", MB_OK);
                gSK1();} // In simple mode, send Character if any key goes up.
      // Timer mode: Do nothing here, Timers will tell the Chord and KeyCode

      // 
      AdownPrev = Adown;
      BdownPrev = Bdown;
      CdownPrev = Cdown;
      DdownPrev = Ddown;
      EdownPrev = Edown;
      FdownPrev = Fdown;
                /*
         gMouseShiftOFF();
         gMouseCtrlOFF();
         gMouseAltOFF();         
         AutoRpt = FALSE;
         AllUp = TRUE; //default
         Adown = FALSE;Bdown = FALSE;Cdown = FALSE;Ddown = FALSE;Edown = FALSE;Fdown = FALSE;
         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "   GKOS is OFF   "); 
         SendMessage(hWndStatusBar, SB_SETTEXT, 1, (LPARAM) "  Press Esc key! "); 
        // SendMessage(hWndStatusBar, SB_SETTEXT, 2, (LPARAM) "                  "); // This cannot be put back easily
                 */
         return FALSE;

} // end of function

//==============================================================================
void gCheckComInput(int KeyCodeC)
{
//if (KeyCodeC == 188){MessageBox(0, "KeyCode = 188", "Test...", MB_OK);}
static int KeyCodeK = 0;

       KeyCodeK = KeyCodeC;
        // Leave Mouse Data as it is. Start of Mouse Data indication:
       if (KeyCodeK == 128){gCheckInput(KeyCodeK);
                            return;}

       // Leave Mouse Data as it is. Skip Mouse Data bytes.
       if (MouseData == 0) // Only KeyCodes are dealt with:
       {
       
         // COM Rx MODE: Receiving plain Chords+192 via COM?
         if (KeyCodeK == 192){ // Switch to receiving plain Chords+192 via COM
         // Kind of TEMP solution. Should be several (64...127) consecutive 192s
             //MessageBox (NULL, "KeyCode 192 received", " x ", MB_OK);

                      if (Chords2COM == FALSE){
                          strcpy(tempstring, "Input: ");
                          strcat(tempstring, PortNum);
      // TEMP disable SimpleMode
                          if (SimpleMode){strcat(tempstring, " (Simple N/A)");}
                          if (!SimpleMode){strcat(tempstring, " Full mode");}
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                          }                       
                          Chords2COM = TRUE;
                          }


         // COM Rx MODE: Receiving processed (by PIC) KeyCodes?
         if (KeyCodeK <= 191){  // Switch to receiving KeyCodes via COM
                      if (KeyCodeK == 0){return;}
                      if (Chords2COM == TRUE){
                          strcpy(tempstring, "Input: ");
                          strcat(tempstring, PortNum);
                          strcat(tempstring, " KeyCodes");
                          SendMessage(hWndStatusBar, SB_SETTEXT, 2,(LPARAM) tempstring);
                          Chords2COM = FALSE;
                          }
                      }
         // Receive KeyCodes             
         if (Chords2COM == FALSE){
                      if (g123abc == TRUE){
                          if (KeyCodeK <= 29){KeyCodeK = KeyCodeK + 63;}
                                  }
                      if (gSYMB == TRUE){
                          if (KeyCodeK <= 41){KeyCodeK = KeyCodeK + 63;}
                                  }
                      gCheckInput(KeyCodeK); return;}

         // Receive Chords+192
         if (Chords2COM == TRUE){ // Yes. Chord values+192 are received.
            // This will send characters when keys are released etc.
             KeyCodeK = KeyCodeK & 191; // Clear the two MSBs even though
             gUpdateGKOSstatus(KeyCodeK); // this function may not mind those bits
                  return;
                  }
         
       }  // end of MouseData == 0
}

// ------------------------------------------
void gCheckInput(int KeyCodeX)
{
//if (KeyCode == 188){MessageBox(0, "KeyCode = 188 (SHIFT Key)", "Test...", MB_OK);}
//if (KeyCode == 54){MessageBox(0, "KeyCode = 54 (ENTER)", "Test...", MB_OK);}
//if (KeyCode == 1){MessageBox(0, "KeyCode = 1 (a)", "Test...", MB_OK);}
                  
       if (MouseData == 2)
       {
//MessageBox(NULL, "Mouse Data MouseByte 1.","Test point...", MB_OK | MB_ICONINFORMATION);
          MouseByte[1]=KeyCodeX;
         // gDisplay(2); //'b' BUTTON
       }
       if (MouseData == 3)
       {
//MessageBox(NULL, "Mouse Data MouseByte 2.","Test point...", MB_OK | MB_ICONINFORMATION);
          MouseByte[2]=KeyCodeX;
         // gDisplay(24); // 'x' Mouse X
       }
       if (MouseData == 4)
       {
//MessageBox(NULL, "Mouse Data MouseByte 3.","Test point...", MB_OK | MB_ICONINFORMATION);
          MouseByte[3]=KeyCodeX;
         // gDisplay(25); // 'y' Mouse Y
       }
       if (MouseData == 5)
       {
//MessageBox(NULL, "Mouse Data MouseByte 4.","Test point...", MB_OK | MB_ICONINFORMATION);
          MouseByte[4]=KeyCodeX;
         // gDisplay(26); // 'z' Mouse Z (Scrollwheel)
       }
       if (MouseData >= 6)
       {
//MessageBox(NULL, "Mouse Data MouseByte 5.","Test point...", MB_OK | MB_ICONINFORMATION);
          if (KeyCodeX == 255) // Mouse Data End indication
          {
//MessageBox(NULL, "Mouse Data End.","Test point...", MB_OK | MB_ICONINFORMATION);
         // gDisplay(255);
          gSendMouse();
          }
          MouseData=0;
       }
       if (KeyCodeX == 128) // Mouse Data Satrt indication
       {
          MouseData = 1;
//MessageBox(NULL, "Mouse Data Start.","Test point...", MB_OK | MB_ICONINFORMATION);
        //  gDisplay(128);
       }
       if (MouseData == 0) // just send keycode
       {                     
          // --------That was mouse----------


          // --------This is keyboard: -------

           // ------------------
          UnInstallHook(); 

          if (ChordonData == 2)
          {
             ChordonByte[1]=KeyCodeX;
            // gDisplay(KeyCode); //'1'
          }

          if (ChordonData == 3)
          {
             ChordonByte[2]=KeyCodeX;
            // gDisplay(KeyCode); //'2'

             if (KeyCodeX == 253) // 53=end, 52 = Word Right (to test), 253 = AKU
                {
                    //gDisplay(ChordonByte[1]);
                    //gDisplay(KeyCodeX);
                    if (Monitor == FALSE)
                    {
             //          gSK(24); //'x' test
// if (ChordonByte[1] == 61){MessageBox(NULL, "123-ABC pressed.","Test point...", MB_OK | MB_ICONINFORMATION);}
             
                       gSK(ChordonByte[1]);
                       //gSK(KeyCodeX);
                    }
                ChordonData = 1;
                }

          }



          //           [AKU] ! h [AKU]  = 'had '  for example
          if (KeyCodeX == 253) // 53=end, 52 = Word Right =52 (to test), 253 = AKU
          {
            if (ChordonData == 0)
              {
              ChordonData = 1; // after each AKU
    //          gSK(25); //'y' test
              //gDisplay(253);
              }
          }


          if (ChordonData == 4)
          {
             ChordonData = 0;
             ChordonByte[3]=KeyCodeX;
            // gDisplay(KeyCodeX); // Monitor only
             KeyCodeX = 0; // No output of this any more

                 if (ChordonByte[3] == 253) // 53=end, second [AKU]
                    {                   
                    ChordonData = 1; // after each AKU
      //                 gSK(26); //'z'test
                    //gDisplay(253); //'AKU'
                    // SEND SHORTCUTS HERE
                    gSendShortcut();
                    }  
                 if (ChordonByte[3] != 253)  // 253 = AKU
                    {
                    gSendRest();                   
                    }  

          }


          
          // ----------------------------------------------------
          if (ChordonData == 0) // just send keycode
          {
                 //   gDisplay(KeyCode);
                    if (Monitor == FALSE)
                    {
                       gSK(KeyCodeX);
                    }
           }


           if (ChordonData != 0)
           {
               ChordonData++;
           }


           InstallHook();
          // ----------------------------------------------------          // ----------------------------------------------------
       }

       if (MouseData != 0)
       {
        MouseData++;
       }



}  // end of function

//-------------------------------------------------------------------
/*
void gPrint(char *TempChar)
     {
        if (Monitor == TRUE)
        {
        strcpy(A,(char*)BCX_Get_Text(Edit1)); // get existing display data
        sprintf(A,"%s%s",A, TempChar);
        BCX_Set_Text(Edit1,A);
        return;
        }
     }


void gDisplay(int KeyCode)
     {
        //if (KeyCode == 0) {
         //          return;}          
        if ((Monitor == TRUE) && (USmode == TRUE))        
        {
        strcpy(A,(char*)BCX_Get_Text(Edit1)); // get existing display data
        sprintf(A,"%s%s",A, gCharUS[KeyCode]);
        BCX_Set_Text(Edit1,A);
        return;
        }

        if (Monitor == TRUE)
        {
        strcpy(A,(char*)BCX_Get_Text(Edit1)); // get existing display data
        sprintf(A,"%s%s",A, gChar[KeyCode]);
        BCX_Set_Text(Edit1,A);
        return;
        }
     }
                
        // char A[2048]; Defined at the beginning now
        // ADD something to avoid display buffer overflow
//        strcpy(A,(char*)BCX_Get_Text(Edit1)); // get existing display data
*/
               
// ------------ Generate PC key presses ---------------------------
// Note: applications should not use the scan code because it is 
// OEM-dependent, so use virtual key value

void gTybotSub() // 
{               
        TybotCnt = 2; // Must be at least 2 (=20ms)
        //bVk=pcKeyCode; bScan=0; dwFlags=0; dwExtraInfo=0;
        if (TybotA == 1){keybd_event(0x46, 0, 0, 0);} // GKOS key A down
        if (TybotB == 1){keybd_event(0x44, 0, 0, 0);} // GKOS key B down
        if (TybotC == 1){keybd_event(0x53, 0, 0, 0);} // GKOS key C down
        if (TybotD == 1){keybd_event(0x4a, 0, 0, 0);} // GKOS key D down
        if (TybotE == 1){keybd_event(0x4b, 0, 0, 0);} // GKOS key E down
        if (TybotF == 1){keybd_event(0x4c, 0, 0, 0);} // GKOS key F down
        
        // Delay
        // KEYUP is in WM_TIMER:
        // keybd_event(0x46, 0, KEYEVENTF_KEYUP, 0); // key up                   

}

void gTybot() // -gTybot
{
  // Machine generated key entries to test and trim this software
     //MessageBox(NULL, "Not yet implemented.", "TYBOT - Typing Robot", MB_OK | MB_ICONINFORMATION);
     //retrun;

     // QWERTY keys S=0x53 D=0x44 F=0x46   J=0x4a K=0x4b L=0x4c
                // pcKeyCode = iChar[gKeyCode];
                //  pcShifts = iShifts[gKeyCode];
                //  gPressKey1();
                
                TybotA = 1; TybotD = 1;
                TybotB = 1; TybotE = 0;
                TybotC = 0; TybotF = 0;
                gTybotSub();

// Must be done under WM_TIMER, not here
//                TybotA = 1; TybotD = 0;
//                TybotB = 1; TybotE = 0;
//                TybotC = 0; TybotF = 0;
//                gTybotSub();

}


void gPressKey(int bVkValue)
     {
        // if pcShifts = 0, then use shifted characters when needed                   
        // press the key
        bVk=bVkValue; bScan=0; dwFlags=0; dwExtraInfo=0;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key down
        dwFlags=KEYEVENTF_KEYUP;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key up                   
     }                        

void gPressKey1() // Press pcKeyCode
     {
        // if pcShifts = 0, then use shifted characters when needed                   
        // press the key
        bVk=pcKeyCode; bScan=0; dwFlags=0; dwExtraInfo=0;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key down
        dwFlags=KEYEVENTF_KEYUP;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key up                   
     }                        


void pcKeyDown(int bVkValue) // Key down any value
     {
        bVk=bVkValue; bScan=0; dwFlags=0; dwExtraInfo=0;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key down
     }                        

void pcKeyDown1() // Key Down pcKeyCode
     {
        bVk=pcKeyCode; bScan=0; dwFlags=0; dwExtraInfo=0;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key down
     }                        

void pcKeyUp(int bVkValue) // Key up any value
     {
        bVk=bVkValue; bScan=0; dwFlags=0; dwExtraInfo=0;
        dwFlags=KEYEVENTF_KEYUP;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key up                   
     }                        

void pcKeyUp1() // Key up pcKeyCode
     {
        bVk=pcKeyCode; bScan=0; dwFlags=0; dwExtraInfo=0;
        dwFlags=KEYEVENTF_KEYUP;
        keybd_event(bVk, bScan, dwFlags, dwExtraInfo); // key up                   
     }                        

// -------------------------------------------------------------
void gSendWindowsKeys(int gWKey)  // Not used
{
// MessageBox (NULL, "Selecting Fn key", "Test...", MB_OK);
    // gSK(122); return; // TEST !!!!
               	  switch (gWKey)
                    { 
                    case 19:// s -> 'Win Start key' (left)       
                      gSK(122);
 // MessageBox (NULL, "That should have been the F1 key", "Test...", MB_OK);
                       break;
                    case 13:// m -> 'Win Menu key'       
                      gSK(121);
 // MessageBox (NULL, "That should have been the F1 key", "Test...", MB_OK);
                       break;
                       
                       }
     
}     


     // Check if Keycode is a shift. If Shift, Ctrl or Alt is ON already,
     // do not allow sending of certain characters (special symbols keys). 

int gCheckShifts(int KeyCodeS)
     {

//if (KeyCode == 188){MessageBox(0, "KeyCode = 188 (SHIFT Key)", "Test...", MB_OK);}
//if (KeyCode == 54){MessageBox(0, "KeyCode = 54 (ENTER)", "Test...", MB_OK);}
//if (KeyCode == 1){MessageBox(0, "KeyCode = 1 (a)", "Test...", MB_OK);}
//
/*
               	  switch (gKeyCode)
                    { 
                    case 242: gFUNC = TRUE; return; // Shift TBD
                    case 243: return; // Shift TBD
                    case 244: return; // Shift TBD
                    case 245: gWIN = TRUE; return; // Shift to WinKeys (S = start, m = Menu etc)
                    default:
                      break;      
                    }
*/
//    MessageBox (NULL, "Checkin shifts!", "Test", MB_OK);

//   if (KeyCode == 253){return 1;} //AKU
// if (KeyCode == 61){MessageBox(NULL, "123-ABC pressed.","Test point...", MB_OK | MB_ICONINFORMATION);}
      
                  pcKeyCode = iChar[KeyCodeS];
                  
                  // return 1 means that the KeyCode shall not be sent to PC keyboard
                  // Some symbols do not use Shift for GKOS symbols on PC keyboard
                  if ((gShift) && (XShifts == 0) && (gAltGr == FALSE)) {  // Allow Shift for AltGr+Shift (special symbols)
                  if ((KeyCodeS > 75) && (KeyCodeS < 106)) {return 1;}
                  if ((KeyCodeS > 30) && (KeyCodeS < 42)) {return 1;}}
                  
                  // Some symbols do not allow Ctrl ****CHECK!!!!
                  if (gCtrl) {
                  if ((KeyCodeS > 75) && (KeyCodeS < 106)) {return 1;}
                  if ((KeyCodeS > 30) && (KeyCodeS < 42)) {return 1;}}

                  // Some symbols do not allow Alt ****CHECK!!!!
                  if (gCtrl) {
                  if ((KeyCodeS > 75) && (KeyCodeS < 106)) {return 1;}
                  if ((KeyCodeS > 30) && (KeyCodeS < 42)) {return 1;}}

                  // Scroll = SYMB Up/Down Arrow
                  if (gSYMB) 
                  {
               	  switch (KeyCodeS)
                    { 
                    case 42:// 'Up Arrow'
                         ScrollDown();
                         if (!AutoRpt){gSYMB = FALSE;}
                         return 1; // don't send the key
                    case 43:// 'Down Arrow'
                         ScrollUp();
                         if (!AutoRpt){gSYMB = FALSE;}
                         return 1;
                    }    
                  }  
                                              
                  //gSendFunctionKeys
                  if (gFUNC)
                  {
               	  switch (KeyCodeS)
                     { 
                    case 1:// 'F1' a
                         gSK(106); return 1;
                    case 2:// 'F2' b
                         gSK(107); return 1;
                    case 3:// 'F3' c
                         gSK(108); return 1;
                    case 4:// 'F4' d
                         gSK(109); return 1;
                    case 5:// 'F5' e
                         gSK(110); return 1;
                    case 6:// 'F6' f
                         gSK(111); return 1;
                    case 8:// 'F7' h
                         gSK(112); return 1;
                    case 9:// 'F8' i
                         gSK(113); return 1;
                    case 10:// 'F9' j
                         gSK(114); return 1;
                    case 7:// 'F10' g
                         gSK(115); return 1;
                    case 12:// 'F11' l
                         gSK(116); return 1;
                    case 13:// 'F12' m
                         gSK(117); return 1;
                     }
                   }        

                  // Not a valid key
                  if (pcKeyCode == 0x00){
                              return 1;}
                              
                  // Autorepeat?            
                  if (AutoRpt == TRUE){   // If Autorepeat, there is no
                                      // need to check shifts.
                              return 0;} // Send anyhow

                  // Function keys F1...                
                  if (KeyCodeS == 242){
                              if (gFUNC == FALSE){gFUNC = TRUE;
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: F1-F12"); 
                                  return 1;}
                              gFUNC = FALSE;
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}
                  // Special Shift, TBD (s = Start, m = Menu)                
                  if (KeyCodeS == 243){
                              if (gSPE == FALSE){gSPE = TRUE;
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: SPECIAL"); 
                                  return 1;}
                              gSPE = FALSE;
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}

                  // Window key m (s = Start, m = Menu)                
                  if (KeyCodeS == 245){
                              if (gWIN == FALSE){gWIN = TRUE;
                                  pcKeyDown(0x5d);
                                  pcKeyUp(0x5d);
                                  return 1;
                                   // to be finalised:                                 
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Shift: Win (s or m)"); 
                                  return 1;}
                              gWIN = FALSE;
                              pcKeyUp(0x5d);
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}
                  // Window key s (s = Start, m = Menu)    //NOT USED            
                  if (KeyCodeS == 244){
                              if (gWIN == FALSE){gWIN = TRUE;
                                  pcKeyDown(0x5b);
                                  pcKeyUp(0x5b);
                                  return 1;
                                   // to finalised:                                 
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Shift: Win (s or m)"); 
                                  return 1;}
                              gWIN = FALSE;
                              pcKeyUp(0x5d);
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}




                  // SYMB           
                  if (KeyCodeS == 60){
 //   MessageBox (NULL, "Checking SYMB!", "Test", MB_OK);
                              if (gSYMB == FALSE){
                                  gSYMB = TRUE;
                                  BmpSelectTEMP = 3;
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: SYMB  "); 
                                  return 1;}
                              gSYMB = FALSE;
                              BmpSelectTEMP = 2;
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}

                  // 123-ABC           
                  if (KeyCodeS == 61){
  //  MessageBox (NULL, "Checkin 123-ABC!", "Test", MB_OK);
                              if (gESC == TRUE){gESC = FALSE;} // Clear F1...F12
                                 // Mouse activation Shift + 123-ABC
                              if (gMouseMode == TRUE){
                                     gMouseMode = FALSE;
                                     SetPCkeyboard(PCkeyboard);  // Update the status display
                                     return 1;}
                              if (gShift == TRUE){
                                     g123abc = FALSE;
                                     //BmpSelectTEMP = 2;
                                     gShiftOFF();
                                     gMouseMode = TRUE;
                                     SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse ");
                                     //gShift = FALSE; pcKeyUp(0x10);
                                     return 1;} 
                                     
                                  // Alt + 123-ABC (= Change language)  
                              if (gAlt == TRUE){gAlt = FALSE;

                                 if (Language == 1) // Change Language
                                     {// USmode = FALSE;
                                     Language = NativeLang;
                                     SetLanguage(Language);
                                     pcKeyDown(0x12);
                                     pcKeyUp(0x12);
                                     SetPCkeyboard(PCkeyboard); // Update the status display
                                     return 1;} 
                                 if (Language == NativeLang)
                                     {// USmode = TRUE;
                                     Language = 1;
                                     SetLanguage(Language);
                                     pcKeyDown(0x12);
                                     pcKeyUp(0x12);
                                     SetPCkeyboard(PCkeyboard); // Update the status display
                                     return 1;} 
                                 // Any other language => English (TEMPORARY before more languages are introduced)
                                     Language = 1;
                                     SetLanguage(Language);
                                     pcKeyDown(0x12);
                                     pcKeyUp(0x12);
                                     SetPCkeyboard(PCkeyboard); // Update the status display
                                     return 1; 
                                  }
                              if (g123abc == FALSE){
                                  g123abc = TRUE;
                                  BmpSelectTEMP = 4; // Show number help
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mode: 123  "); 
                                      return 1;}
                              g123abc = FALSE;
                              BmpSelectTEMP = 2; // Show letters help
                              Space123 = 0;
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}




                  // Shift KEY
                  if (KeyCodeS == 188){
//                  if (KeyCode == 59){
//    MessageBox (NULL, "Checkin shifts!", "Test", MB_OK);
                              if (gCAPS == TRUE){
                                  gCAPS = FALSE;
                                  SetCapsLock(TRUE);
                                  SetCapsLock(FALSE);
                                  //gShiftOFF();
                                  //gShift = FALSE; pcKeyUp(0x10);
                                 SetPCkeyboard(PCkeyboard); // Update the status display
                                  return 1;}
                              if (gShift == TRUE){
                                  gShiftOFF();
                                  //gShift = FALSE; pcKeyUp(0x10);
                                     if (Painting == TRUE) {
                                                      Painting = FALSE;
                                                      //}
                                                      return 1;}
                                  gCAPS = TRUE;
                                  SetCapsLock(TRUE);
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: CAPS  ");
                                  if (SpaceShift == 1){SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: CAPS>Space  ");} 
                                  return 1;}
                          //    pcKeyDown(0x10);
                              gShiftON();
                              //gShift = TRUE; pcKeyDown(0x10);
                              return 1;}
                              
                  // Shift ON                
                  if (KeyCodeS == 59){
                              //gShift = TRUE; pcKeyDown(0x10);
                              gShiftON();
                              return 1;}
                  // Shift OFF                
                  if (KeyCodeS == 187){
                              gShiftOFF();
                              //gShift = FALSE; pcKeyUp(0x10);
                              return 1;}

                  // Space (Clear 123/CAPS sometimes)               
                  if (KeyCodeS == 50){
                      if ((SpaceShift == 1)&&(gCAPS == TRUE)){                               
                                  gCAPS = FALSE;
                                  SetCapsLock(TRUE);
                                  SetCapsLock(FALSE);
                                  gShiftOFF();
  //   MessageBox (NULL, "Clear Shift", "Test", MB_OK);
                                  //gShift = FALSE; pcKeyUp(0x10);
                                  SpaceShift = 0;            }
                      if ((Space123 == 1)&&(g123abc == TRUE)){                               
                                  g123abc = FALSE;
                                  Space123 = 0;            
  //   MessageBox (NULL, "Clear 123", "Test", MB_OK);
                                  SetPCkeyboard(PCkeyboard); // Update the status display                              
                                  }
                           //   pcKeyUp(pcKeyCode);
                              //pcKeyUp(0x10);}
                                   }

                  // Word Right => Ctrl ON                
                  if (KeyCodeS == 52){
                              gCtrl = TRUE;
                              return 0;} // Send anyhow
                  // Word Left => Ctrl ON                
                  if (KeyCodeS == 48){
                              gCtrl = TRUE;
                              return 0;} // Send anyhow
                  // Ctrl ON/OFF
                  if (KeyCodeS == 62){
   // MessageBox (NULL, "Checkin Ctrl!", "Test", MB_OK);
                           if (gCtrl == FALSE){   
                              gCtrl = TRUE;
                             // pcKeyDown(0xa2);
// This harms delete word left function (Chordon: Shift Backspace):
//                              if (Painting == TRUE) {
//                                 Painting = FALSE;
//                                 gShift = FALSE; pcKeyUp(0x10);}
                              if (!Painting){SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift:  Ctrl  ");}                                    
                              return 1;}
                            gCtrl = FALSE;
                            if (!Painting){SetPCkeyboard(PCkeyboard);} // Update the status display                              
                          //  pcKeyUp(0xa2);
                            return 1;  
                              }
                  // Ctrl OFF           
                  if (KeyCodeS == 190){
                              gCtrl = FALSE;
                          //    pcKeyUp(pcKeyCode);
                              pcKeyUp(0xa2);
                              if (!Painting){SetPCkeyboard(PCkeyboard);} // Update the status display                              
                              return 1;}
                  // Alt ON/OFF (63/191)          
                  if (KeyCodeS == 63){
  //  MessageBox (NULL, "Checkin Alt!", "Test", MB_OK);
                              if (gAlt == FALSE){
                                  gAlt = TRUE;
                                  pcKeyDown(0x12);
                                  pcKeyUp(0x12);
                                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift:  Alt  ");                                    
                                  return 1;}
                              gAlt = FALSE;
                              pcKeyDown(0x12);
                              pcKeyUp(0x12);
                              SetPCkeyboard(PCkeyboard); // Update the status display                              
                              return 1;}
                  // Alt OFF           
                  if (KeyCodeS == 191){
                              gAlt = FALSE;
                           //   pcKeyUp(pcKeyCode);
                              pcKeyDown(0x12);
                              pcKeyUp(0x12);
                              SetPCkeyboard(PCkeyboard); // Update the status display                              
                              return 1;}
                  // AltGr ON/OFF           
                  if (KeyCodeS == 118){
    //MessageBox (NULL, "Checkin AltGr!", "Test", MB_OK);
                              if (gAltGr == FALSE){
                                  gAltGr = TRUE;
                                  //pcKeyDown(0x41);
                                  //pcKeyUp(0x41);
                              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift:  AltGr  ");                                    
                                  return 1;}
                              gAltGr = FALSE;
                              //pcKeyDown(0x41);
                              //pcKeyUp(0x41);
                              SetPCkeyboard(PCkeyboard); // Update the status display                              
                              return 1;}
                              
                             
                  // AltGr OFF           
                  if (KeyCodeS == 246){
                              gAltGr = FALSE;
                          //    pcKeyUp(pcKeyCode);
                              pcKeyUp(0x41);
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}
                  // CAPS ON           
                  if (KeyCodeS == 119){
                              gCAPS = TRUE;
                              SetCapsLock(TRUE);
                              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift:  CAPS  ");                                    
                              return 1;}
                  // CAPS OFF           
                  if (KeyCodeS == 246){
                              gCAPS = FALSE;
                              SetCapsLock(TRUE);
                              SetCapsLock(FALSE);
                              SetPCkeyboard(PCkeyboard); // Update the status display
                              return 1;}
                  // Num Lock ON           
                  if (KeyCodeS == 120){
                              gNumLock = TRUE;
                              SetNumLock(TRUE);
                              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Num Lock ON ");                                    
                              return 1;}
                  // Num Lock OFF           
                  if (KeyCodeS == 248){
                              gNumLock = FALSE;
                              SetNumLock(FALSE);
                              SetPCkeyboard(PCkeyboard); // Update the status display                              
                              return 1;}
                  // Scroll Lock ON           
                  if (KeyCodeS == 124){
                              gScroll = TRUE;
                              SetScroll(TRUE);
                              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Scroll Lock ON");                                    
                              return 1;}
                  // Scroll Lock OFF           
                  if (KeyCodeS == 252){
                              gScroll = FALSE;
                              SetScroll(FALSE);
                              SetPCkeyboard(PCkeyboard); // Update the status display                              
                              return 1;}
                              
                               
                              
                  // Esc - for Function keys F1...F12
                  // Here at the end because Shifts may be used          
                  if (KeyCodeS == 56){   //56 = Esc, 55 = TAB to test
//   MessageBox (NULL, "Checkin ESC! (1)", "Test", MB_OK);
                              if (gESC == TRUE){
                                  gESC = FALSE;  // Clear F1...F12
                                  return 1;}
                              if (gSYMB == TRUE){
                                  gSYMB = FALSE;
                                  BmpSelectTEMP = 2;
                                  SetPCkeyboard(PCkeyboard); // Update the status display                                 
                                  gESC = TRUE;
//   MessageBox (NULL, "Checkin ESC! (2)", "Test", MB_OK);
                              return 1;}
                                     }
//                  // TEST: ENTER           
//                  if (KeyCodeS == 54){
//    MessageBox (NULL, "Checkin ENTER!", "Test", MB_OK);
//                            return 0;}
//                  // TEST: TAB           
//                  if (KeyCodeS == 55){
//    MessageBox (NULL, "Checkin TAB!", "Test", MB_OK);
//                            return 0;}

                              
  //  MessageBox (NULL, "Send KeyCode", "Test", MB_OK);

     return 0; // indicates that nothing was done here and KeyCode will be sent
     }

void gShiftON()
{
     gShift = TRUE; // pcKeyDown(0x10);
   //  MessageBox (NULL, "Shift ON!", "Test", MB_OK);                          
   SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: SHIFT   "); 

}     

void gShiftOFF()
{
     gShift = FALSE; //pcKeyUp(0x10);
     //MessageBox (NULL, "Shift OFF!", "Test", MB_OK);                          
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift:     "); 
     SetPCkeyboard(PCkeyboard); // Update the status display
}     
     

// Turn all Shifts OFF
void gShiftsOFF()
{                             
                     if (XShifts != 0){
                                 XShifts = 0;
                                 SetPCkeyboard(PCkeyboard); // Update the status display 
                                 };
                     if ((gShift == TRUE) && (AutoRpt == FALSE)) 
                     {
                         if (!Painting){gShiftOFF();}
                     }
                     
                     if (gCtrl == TRUE)  {
                                       gCtrl = FALSE;
                                       pcKeyUp(0xa2);
                                       if (!Painting){SetPCkeyboard(PCkeyboard);} // Update the status display
                                       }
                     if (gAlt == TRUE)   {
                                       gAlt = FALSE;
                                       pcKeyUp(0x12);          
                                       SetPCkeyboard(PCkeyboard); // Update the status display
                                       }
                     if (gSYMB == TRUE){gSYMB = FALSE;
                          BmpSelectTEMP = 2;
                          SetPCkeyboard(PCkeyboard); // Update the status display
                          }
                     if (gESC == TRUE){gESC = FALSE;} // For function keys F1...F12
                     if (gAltGr == TRUE){
                          SetPCkeyboard(PCkeyboard); // Update the status display                              
                          gAltGr = FALSE;} // 

                     if (gFUNC == TRUE){gFUNC = FALSE;
                          BmpSelectTEMP = 2;
                          SetPCkeyboard(PCkeyboard); // Update the status display
                          }
                     if (gWIN == TRUE){gWIN = FALSE;
                          BmpSelectTEMP = 2;
                          SetPCkeyboard(PCkeyboard); // Update the status display
                          }
                     if (gSPE == TRUE){gSPE = FALSE;
                          BmpSelectTEMP = 2;
                          SetPCkeyboard(PCkeyboard); // Update the status display
                          }

                     return;
}

/////////////////////////////
// --- mouse functions --- //
/////////////////////////////

void static gMouseDisplay(void)
{
     if (gMouseSHIFT){
     SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: SHIFT "); 
            if (gMouseCtrl){
            SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mouse: SHIFT+Ctrl");                             
                  if (gMouseAlt){
                  SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mouse:SH+Ctr+Alt");                                 
                  return;
                  }
            }                    
            if (gMouseAlt){
            SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mouse: SHIFT+Alt");                             
            }
     return;
     }
     if (gMouseCtrl){
     SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: Ctrl "); 
            if (gMouseAlt){
            SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mouse: Alt+Ctrl");                             
            }
     return;
     }
     if (gMouseAlt){
     SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: Alt "); 
     return;
     }
     SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse "); 
     return;
}     



void static gMouseCtrlOFF(void)
{
if (!gMouseCtrl){return;}
     gMouseCtrl = FALSE;
     pcKeyUp(0xa2); 
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse "); 
}

void static gMouseCtrlON(void)
{
     if (gMouseCtrl){return;}
     gMouseCtrl = TRUE;
     pcKeyDown(0xa2); 
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: Ctrl "); 
}


void static gMouseAltOFF(void)
{
if (!gMouseAlt){return;}
     gMouseAlt = FALSE;
     pcKeyUp(0x12); 
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse "); 
}

void static gMouseAltON(void)
{
     if (gMouseAlt){return;}
     gMouseAlt = TRUE;
     pcKeyDown(0x12); 
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: Alt"); 
}



void static gMouseShiftOFF(void)
{
if (!gMouseSHIFT){return;}
     gMouseSHIFT = FALSE;
     pcKeyUp(0x10); 
    // update status display
     gMouseDisplay();
    //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse "); 
}

void static gMouseShiftON(void)
{
     if (gMouseSHIFT){return;}
     gMouseSHIFT = TRUE;
     pcKeyDown(0x10);
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: SHIFT"); 
}


void static gMouseScrollOFF(void)
{
if (!gMouseScroll){return;}
    gMouseScroll = FALSE;
    // update status display
    gMouseDisplay();
    //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) "Mode: GKOS Mouse "); 
}

void static gMouseScrollON(void)
{
     if (gMouseScroll){return;}
     gMouseScroll = TRUE;
     // update status display
     gMouseDisplay();
     //SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mouse: SCROLL"); 
}

// GKOS keyboard is operating as a Mouse (pointer control), gMouseMode = TRUE
// - activated by Shift 123abc or Shift_123abc
// gMouseSHIFT = FALSE; // Shifts can be used with mouse too
// gMouseCtrl = FALSE; 
// gMouseAlt = FALSE; 
// gMouseScroll = FALSE; 
/*
 
 SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: SPECIAL"); 
 SetPCkeyboard(PCkeyboard); // Update the status display

*/
void GKOSmouse(int gMouseCode)
{
     AutoRptDelay = 26;
     switch (gMouseCode)
            {
              case 61: // 123abc          
                   //gMouseMode = FALSE; done by:
                   gCheckShifts(gMouseCode);
                   gMouseShiftOFF();
                   gMouseCtrlOFF();
                   gMouseAltOFF();
                   AutoRptDelay = AutoRptDelayF;                             
                   return;
              case 60: // SYMB          
                   if (gMouseScroll){
                             gMouseScrollOFF();        
                             return;}
                   //AutoRptDelay = AutoRptDelayF; // May be needed                             
                   gMouseScrollON(); return;
              case 42: // Up
                   if (gMouseScroll){
                             ScrollDown(); // view up        
                             return;}
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 0x04;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 44: // Up Fast
                   if (gMouseScroll){
                             ScrollDownFast();        
                             return;}
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 16; // 
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 54: // Enter = Up Big Step
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 132; // 164
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 43: // Dn
                   if (gMouseScroll){
                             ScrollUp(); // view down       
                             return;}
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 4;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 45: // Dn Fast
                   if (gMouseScroll){
                             ScrollUpFast();        
                             return;}
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 16;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 57: // Del = Dn Big step
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x00;
                   MouseByte[3] = 132;   // 164
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
                                 
              case 1: // a
                   gMouseScrollOFF();
                   MouseByte[1] = 0x38; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x03; // dx
                   MouseByte[3] = 0x03; // dy
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 15: // o
                   gMouseScrollOFF();
                   MouseByte[1] = 0x38; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 11;  // dx
                   MouseByte[3] = 11;  // dy
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 16: // p
                   gMouseScrollOFF();
                   MouseByte[1] = 0x38; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 93;  // dx
                   MouseByte[3] = 93;  // dy
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 2: // b
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 4;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 46: // backspace
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 16;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 48: // word left = Big step
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 132;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 3: // c
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 3;
                   MouseByte[3] = 3;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 19: // s
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 11;
                   MouseByte[3] = 11;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 22: // v
                   gMouseScrollOFF();
                   MouseByte[1] = 0x18; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 93;
                   MouseByte[3] = 93;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 4: // d
                   gMouseScrollOFF();
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x03;
                   MouseByte[3] = 0x03;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 7: // g
                   gMouseScrollOFF();
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 11;
                   MouseByte[3] = 11;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 8: // h
                   gMouseScrollOFF();
                   MouseByte[1] = 0x28; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 93;
                   MouseByte[3] = 93;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 5: // e
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x04;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 50: // space
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 16;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 52: // word right = Big Step
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 132;
                   MouseByte[3] = 0x00;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 6: // f
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 0x03;
                   MouseByte[3] = 0x03;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 11: // k
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 11;
                   MouseByte[3] = 11;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              case 14: // n
                   gMouseScrollOFF();
                   MouseByte[1] = 0x08; // y ovr, x ovr, y sign, x sign;  1, MB, RB, LB
                   MouseByte[2] = 93;
                   MouseByte[3] = 93;
                   MouseByte[4] = 0x00;
                   gSendMouse();
                   return;
              // Shifts    
              case 188: // Shift Key = Mouse SHIFT
                   if (gMouseSHIFT){
                             gMouseShiftOFF();
                             return;}
                   //AutoRptDelay = AutoRptDelayF; // May be needed                             
                   gMouseShiftON(); return;
              case 62: //  Ctrl = Mouse Ctrl
                   if (gMouseCtrl){
                             gMouseCtrlOFF();        
                             return;}
                   //AutoRptDelay = AutoRptDelayF; // May be needed                             
                   gMouseCtrlON(); return;
              case 63: // Alt = Mouse Alt
                   if (gMouseAlt){
                             gMouseAltOFF();        
                             return;}
                   //AutoRptDelay = AutoRptDelayF; // May be needed                             
                   gMouseAltON(); return;
              // Clicks     
              case 27: // TH = Left Click
                   gMouseScrollOFF();
                   AutoRptDelay = AutoRptDelayF;                             
                   LeftClickDown();
                   LeftClickUp();
                   gMouseShiftOFF();
                   gMouseCtrlOFF();
                   gMouseAltOFF();
                   return;                                      
              case 23: // w = Right Click
                   gMouseScrollOFF();
                   AutoRptDelay = AutoRptDelayF;                             
                   RightClickDown();
                   RightClickUp();
                   gMouseShiftOFF();
                   gMouseCtrlOFF();
                   gMouseAltOFF();
                   return;
              // Drags
              case 34: // ? = Left Button down
                   gMouseScrollOFF();
                   LeftClickDown();
                   return;                                      
              case 33: // ! = Left Button Up
                   gMouseScrollOFF();
                   LeftClickUp();
                   return;                                      
              // Scrolls 
              case 28: // 
                   gMouseScrollOFF();
                   ScrollDownFast();
                   return;                  
              case 30: // 
                   gMouseScrollOFF();
                   ScrollUpFast();
                   return;                  
              case 24: // x Zoom in
                   gMouseScrollOFF();
                   gMouseCtrlON();
                   ScrollDownSlow(); // view up
                   gMouseCtrlOFF();
                   return;                  
              case 26: // z Zoom out
                   gMouseScrollOFF();
                   gMouseCtrlON();
                   ScrollUpSlow(); // view down
                   gMouseCtrlOFF();
                   return;                  
              default:
                   return;
              } // end of switch
}

// Use this function to send the key press referred to by the KeyCode
void gSK(int gKeyCode)
     {
             KeyCode = gKeyCode;
//if (gKeyCode == 61){MessageBox(NULL, "123-ABC pressed.","Test point...", MB_OK | MB_ICONINFORMATION);}
          
          if (gKeyCode != 0)        
          {
     // IS IT OK THAT THIS gSK() ROUTINE CALLS ITSELF ?
     // - it is OK, seems                      
                  // GKOS as a Mouse?
                  if (gMouseMode){GKOSmouse(gKeyCode); return;}
                  // If this is pressing a shift key, press it and return
                  if (gCheckShifts(gKeyCode) == 1){return;}

                  if (Language == 2){ // Fin grammar: accented or not
                    if (AccentedVovelsSPACE){
                                             AccentedVovelsSPACE = FALSE;
                                             AccentedVovelsTEMP = TRUE;
                                            }                 
                    switch (gKeyCode)
                               { 
                    case 1:// 'a'
                               AccentedVovelsTEMP = FALSE;
                               break;
                    case 21:// 'u'
                               AccentedVovelsTEMP = FALSE;
                               break;
                    case 15:// 'o'
                               AccentedVovelsTEMP = FALSE;
                               break;
                    case 50:// ' '
                               AccentedVovels = AccentedVovelsTEMP;                  
                               AccentedVovelsSPACE = TRUE;
                               //AccentedVovelsTEMP = FALSE; // TEST
                               break;
                               }
                    }
                  // National Shortcuts iShiftsUS = 9 
                  if ( (iShiftsUS[gKeyCode] == 9) && (Language == 1) ) {
                                         
               	  switch (gKeyCode)
                    { 
                    case 27:// 'TH'
                      //if (gESC == TRUE){break;}       
                      UnInstallHook(); //NationalSC = TRUE;       
                      gSK(20); gSK(8);
                      InstallHook(); //NationalSC = FALSE;       
                      //TempChar = "th"; gPrint(TempChar)
                      ; break;
                    case 28:// 'a '       
                      UnInstallHook(); //NationalSC = TRUE;
                      gSK(1); gSK(50);
                      InstallHook(); //NationalSC = FALSE;       
                      //TempChar = "a "; gPrint(TempChar)
                      ; break;
                    case 29:// 'the'       
                      UnInstallHook(); //NationalSC = TRUE;       
                      gSK(20); gSK(8); gSK(5); gSK(50);
                      InstallHook(); //NationalSC = FALSE;
                      //TempChar = "the "; gPrint(TempChar)
                      ; break;
                    case 30:// 'of '
                      UnInstallHook(); //NationalSC = TRUE;
                      gSK(15); gSK(6); gSK(50);
                      InstallHook(); //NationalSC = FALSE;
                      //TempChar = "of "; gPrint(TempChar)
                      ; break;
                    case 41:// 'to_'       
                      UnInstallHook(); //NationalSC = TRUE;
                      gSK(20); gSK(15); gSK(50);
                      InstallHook(); //NationalSC = FALSE;
                      //TempChar = "to "; gPrint(TempChar)
                      ; break;
                    case 39:// 'and '       
                      UnInstallHook(); //NationalSC = TRUE;
                      gSK(1); gSK(14); gSK(4); gSK(50);
                      InstallHook(); //NationalSC = FALSE;
                      //TempChar = "and "; gPrint(TempChar)
                      ; break;
                    } // end of switch
                  return;
                  }
//                  // TEST: ENTER           
//                  if (gKeyCode == 54){
//    MessageBox (NULL, "Checking ENTER!", "Test", MB_OK);
//                                        }

//if (gKeyCode == 54){MessageBox(0, "gKeyCode = 54 (ENTER)", "Test...", MB_OK);}
                   
                  // for symbols, Shift or AltGr may be needed here
                  // iShifts[i]=1 for SHIFT,  and =2 for AltGr
                  // press key with the basic code
                  if (USmode == TRUE)
                  {
                      if (iShiftsUS[gKeyCode] == 8){gShift = TRUE; gAltGr = TRUE;}
                             if (gAlt == TRUE){pcKeyDown(0x12);}
                             if (gCtrl == TRUE) {
                                         pcKeyDown(0xa2); } //Ctrl down
                             if (iShiftsUS[gKeyCode] == 1) {
                                         pcKeyDown(0x10); } //SHIFT down to select character
                             if ((gShift == TRUE)&& (iShiftsUS[gKeyCode] != 1)){
                                         pcKeyDown(0x10); } //SHIFT down
                             if ((iShiftsUS[gKeyCode] == 2)||(XShifts == 2)||(gAltGr == TRUE)) {
                                         pcKeyDown(0x11);  //AltGr down VALUE???
                                         pcKeyDown(0x12);  //0x11=Ctrl 0x12=Alt 
                                                          }
                  pcKeyCode = iCharUS[gKeyCode];
                  //pcShifts = iShiftsUS[gKeyCode];
                  gPressKey1();
                             if ((iShiftsUS[gKeyCode] == 2)||(XShifts == 2)||(gAltGr == TRUE)) {
                                         pcKeyUp(0x11);   //AltGr up VALUE???
                                         pcKeyUp(0x12);   //0x11=Ctrl 0x12=Alt 
                                                          }
                             if (iShiftsUS[gKeyCode] == 1) {
                                         pcKeyUp(0x10);   }  //SHIFT up to deselect character                                                         
                             if ((gShift == TRUE)&& (iShiftsUS[gKeyCode] != 1)){
                                         pcKeyUp(0x10); }   //SHIFT up
                             if (gAlt == TRUE){pcKeyUp(0x12);}

                  }
                  // FIN/SWE mode
                  else {
                      if (iShifts[gKeyCode] == 8){gShift = TRUE; gAltGr = TRUE;}
                             if (gAlt == TRUE){pcKeyDown(0x12);}
                             if  (gCtrl == TRUE) {
                                         pcKeyDown(0xa2); } //Ctrl down
                             if (iShifts[gKeyCode] == 1) {
                                         pcKeyDown(0x10); } //SHIFT down to select character
                             if ((gShift == TRUE)&& (iShifts[gKeyCode] != 1)){
                                         pcKeyDown(0x10); } //SHIFT down
                             if ((iShifts[gKeyCode] == 2)||(XShifts == 2)||(gAltGr == TRUE)) {
                                         pcKeyDown(0x11);  //AltGr down VALUE???
                                         pcKeyDown(0x12);  //0x11=Ctrl 0x12=Alt 
                                                          }
                  pcKeyCode = iChar[gKeyCode];
                  //pcShifts = iShifts[gKeyCode];
                  gPressKey1();

                             if ((iShifts[gKeyCode] == 2)||(XShifts == 2)||(gAltGr == TRUE)) {
                                         pcKeyUp(0x11);   //AltGr up VALUE???
                                         pcKeyUp(0x12);   //0x11=Ctrl 0x12=Alt 
                                                        }
                             if (iShifts[gKeyCode] == 1) {
                                         pcKeyUp(0x10);   }  //SHIFT up to deselect character
                             if ((gShift == TRUE)&& (iShifts[gKeyCode] != 1)){
                                         pcKeyUp(0x10); }   //SHIFT up
                                         // 

                  }
                  

                 // To make sure:
                 // Always clear these shifts after key press (even if
                 // also Keycodes are received to release these keys):
                 
                 // But do not release SHIFT if Paint or Autorepeat is on etc.

                 if ((gShift == TRUE) && (AutoRpt == FALSE)) {
                         	switch (gKeyCode) 
	                               {
                                   case 44:  Painting = TRUE; break; //PgUp etc     
                                   case 45:  Painting = TRUE; break;      
                                   case 42:  Painting = TRUE; break;      
                                   case 43:  Painting = TRUE; break;      
                                   case 49:  Painting = TRUE; break;      
                                   case 53:  Painting = TRUE; break;      
                                   case 48:  Painting = TRUE; break; 
                                   case 52:  Painting = TRUE; break;      
                                   case 47:  Painting = TRUE; break; // Left Arrow                              
                                   case 51:  Painting = TRUE; break; // RightArrow
                                   default:  gShiftOFF(); //gShift = FALSE; pcKeyUp(0x10);
                                   }
                             }
                 // Clear shifts if not autorepeating            
                 if (AutoRpt == FALSE){   // Check also this at CheckShifts()
                 gShiftsOFF();
                                      }

            } // end of if KeyCode != 0

     } // end of function




void gSendMouse()
     {
     static int LTemp = 0; 
//  MessageBox(NULL, "Mouse data.","Test point 2...", MB_OK | MB_ICONINFORMATION);               
        //Send Mouse move and status *change* of buttons
        dwMFlags=MOUSEEVENTF_MOVE;
        dwMExtraInfo=0;
        // Left mouse button pressed?
        if (MouseByte[1] & 0x01){
                                 if (!btLDownDelta)
                                          {
                                 dwMFlags=dwMFlags | MOUSEEVENTF_LEFTDOWN;
                                 btLDownDelta = TRUE;
                                 btLUpDelta = FALSE;
        //mouse_event(dwMFlags, 0, 0, 0, dwMExtraInfo); return;
                                           }      
                                 }
        // Left mouse button released?                                 
        if (!(MouseByte[1] & 0x01)){
                                 if (!btLUpDelta)
                                           {
                                 dwMFlags=dwMFlags | MOUSEEVENTF_LEFTUP;
                                 btLUpDelta = TRUE;      
                                 btLDownDelta = FALSE;
        //mouse_event(dwMFlags, 0, 0, 0, dwMExtraInfo); return;
                                           }
                                 }
        // Right mouse button pressed?                                                                                                   
        if (MouseByte[1] & 0x02){
                                 if (!btRDownDelta)
                                          {
                                 dwMFlags=dwMFlags | MOUSEEVENTF_RIGHTDOWN;
                                 btRDownDelta = TRUE;
                                 btRUpDelta = FALSE;
        //mouse_event(dwMFlags, 0, 0, 0, dwMExtraInfo); return;
                                           }      
                                 }
        // Right mouse button released?                                                                  
        if (!(MouseByte[1] & 0x02)){
                                 if (!btRUpDelta)
                                           {
                                 dwMFlags=dwMFlags | MOUSEEVENTF_RIGHTUP;
                                 btRUpDelta = TRUE;      
                                 btRDownDelta = FALSE;
        //mouse_event(dwMFlags, 0, 0, 0, dwMExtraInfo); return;
                                           }
                                 }

        // Mouse movements etc
        // dx is LONG = 32-bit signed value: 0 x ffff ffff
        //dwMFlags=MOUSEEVENTF_MOVE;
        dx = (DWORD)MouseByte[2]; dy = (DWORD)MouseByte[3];
        //dx=MouseByte[2] & 0x000000ff; dy=MouseByte[3] & 0x000000ff;
        dwData=(DWORD)MouseByte[4];
        //dwData=MouseByte[4] & 0x000000ff;
        dwMExtraInfo=0;
        //dy = -dy;
        
        // Mouse movement directions (dx,dy), and y,x two sign bit value 0 to 3:
        //              
        // (fff0,fff0)            (y,x)              (000f,fff0)
        //      (ffff,ffff) 3       2        2  (0001,ffff)
        //                   \      |       /
        //                 1 - (0000,0000) - 0
        //                   /      |       \
        //      (ffff,0001) 1       0        0  (0001,0001)
        // (fff0,000f)                               (000f,000f)
        
        
        // Correct the opposite y direction used for GKOS IR Mouse: 
        //if (!gMouseMode) {
                      //   MouseByte[1] = MouseByte[1] ^ 0x20;  // Bitwise XOR operation
                      //   }
                         
        // Set correct x and y directions by checking the sign bits
        // No correction if GKOS as mouse, only IR reception must be corrected
        // dx = 1; dy = 1;

        // dx
        //if (MouseByte[1] == MouseByte[1]) {  // dx < 0 (left = negative)        
        if (((WORD)MouseByte[1] & 0x10) == 0x10) {  // dx < 0 (left = negative)
                                 if (!gMouseMode) {dx = 0x000000ff - dx;}  // OK
                                 dx = 0xffffffff - dx + 1; // OK
                                 //dx = 0; // TEST
                                 } 

        // Turn dy values around (Up is positive in GKOS IR!)
        if (!gMouseMode){
                         MouseByte[1] = (MouseByte[1] ^ 0x20);
                         dy = 0x000000ff - dy;
                         } //                                         
        // dy
        if (((WORD)MouseByte[1] & 0x20) == 0x20) {  // dy < 0  (up = negative)
                                 if (!gMouseMode) {dy = 0x000000ff - dy;} 
                                 //if (gMouseMode) { dy = 0xffffffff - dy + 1;}
                                 dy = 0xffffffff - dy + 1;
                                 //dy = 0; // TEST
                                 } 
//dy = 0;
        // dx = 0xffffffff - 5; dy = 0xffffffff - 5; // TEST up left
        // dx = 5; dy = 5; // TEST down right
        // dx = 0; // TEST Stop
         // dy = 0; // TEST Stop
        mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
        
     }

// --- Mouse Clicks on the QWERTY keyboard G and H keys
void RightClickDown()
{
     dx=0; dy=0;
     dwData=0; dwMExtraInfo=0;
     dwMFlags = MOUSEEVENTF_RIGHTDOWN;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}
void RightClickUp()
{
     dx=0; dy=0;
     dwData=0; dwMExtraInfo=0;
     dwMFlags = MOUSEEVENTF_RIGHTUP;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}

void LeftClickDown()
{
//   if  (gCtrl == TRUE) {
//                     pcKeyDown(0xa2); } //Ctrl down
//   if (gShift == TRUE) {
//                     pcKeyDown(0x10); } //SHIFT down

     dx=0; dy=0;
     dwData=0; dwMExtraInfo=0;
     dwMFlags = MOUSEEVENTF_LEFTDOWN;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}
void LeftClickUp()
{
     dx=0; dy=0;
     dwData=0; dwMExtraInfo=0;
     dwMFlags = MOUSEEVENTF_LEFTUP;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);

//   if (gShift == TRUE) {
//   					 gShift = FALSE;
//                     pcKeyUp(0x10);   }  //SHIFT up
//   if  (gCtrl == TRUE) {
//                     gCtrl = FALSE;
//                     pcKeyUp(0xa2); } //Ctrl up

}


// --- Scroll (using SYMB Arrows) ---
void ScrollUp() // Wheel rotated towards the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=(DWORD)-120; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}

void ScrollUpFast() // Wheel rotated towards the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=(DWORD)-240; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}
void ScrollUpSlow() // Wheel rotated towards the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=(DWORD)-20; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}

void ScrollDown() // Wheel rotated away from the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=120; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}

void ScrollDownFast() // Wheel rotated away from the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=240; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}
void ScrollDownSlow() // Wheel rotated away from the user
{
     dwMFlags = MOUSEEVENTF_WHEEL;
     dx=0; dy=0;
     dwData=20; dwMExtraInfo=0;
     mouse_event(dwMFlags, dx, dy, dwData, dwMExtraInfo);
}

void ScrollRight()
{
// must be implemented with arrow keys?
}

void ScrollLeft()
{
// must be implemented with arrow keys?
}
//-----

void SetScroll(BOOL bState)
   {
      BYTE keyState[256];

      GetKeyboardState((LPBYTE)&keyState);
      if( (bState && !(keyState[VK_SCROLL] & 1)) ||
          (!bState && (keyState[VK_SCROLL] & 1)) )
      {
      // Simulate a key press
         keybd_event( VK_SCROLL,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | 0,
                      0 );

      // Simulate a key release
         keybd_event( VK_SCROLL,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,
                      0);
      }
   }




void SetNumLock(BOOL bState)
   {
      BYTE keyState[256];

      GetKeyboardState((LPBYTE)&keyState);
      if( (bState && !(keyState[VK_NUMLOCK] & 1)) ||
          (!bState && (keyState[VK_NUMLOCK] & 1)) )
      {
      // Simulate a key press
         keybd_event( VK_NUMLOCK,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | 0,
                      0 );

      // Simulate a key release
         keybd_event( VK_NUMLOCK,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,
                      0);
      }
   }

void SetCapsLock(BOOL bState)
   {
      BYTE keyState[256];

      GetKeyboardState((LPBYTE)&keyState);
      if( (bState && !(keyState[VK_CAPITAL] & 1)) ||
          (!bState && (keyState[VK_CAPITAL] & 1)) )
      {
      // Simulate a key press
         keybd_event( VK_CAPITAL,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | 0,
                      0 );

      // Simulate a key release
         keybd_event( VK_CAPITAL,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,
                      0);
      }
   }


// Clear all shift (before exiting the program)
void ClearShifts()
                                  {
                 if (gShift == TRUE) {
                                   gShiftOFF();}//gShift = FALSE; pcKeyUp(0x10);}
                 if (gCtrl == TRUE)  {
                                   gCtrl = FALSE;
                                   SetPCkeyboard(PCkeyboard); // Update the status display
                                   pcKeyUp(0xa2);}
                 if (gAlt == TRUE)   {
                                   gAlt = FALSE;
                                   SetPCkeyboard(PCkeyboard); // Update the status display
                                   pcKeyUp(0x12);}
                                   }

//==============================================================================

//--------------------------------

// RUN-TIME FUNCTIONS

//char*   BCX_TmpStr(size_t);
char*   chr(int,int=0,int=0,int=0,int=0,int=0,int=0,int=0,int=0,int=0);


// is this really ASC ????
//extern int asc(char *z,int index)
//{
//  return (unsigned char) z[index];
//}

     
char *BCX_TmpStr (size_t Bites)
{
  static int   StrCnt;
  static char *StrFunc[2048];
  StrCnt=(StrCnt + 1) & 2047;
  if(StrFunc[StrCnt]) free (StrFunc[StrCnt]);
  return StrFunc[StrCnt]=(char*)calloc(Bites+128,1);
}

// tx_char$ = CHR$(35) <=> strcpy(tx_char,(char*)chr(35));
char *chr (int a,int b,int c,int d,int e,int f,int g,int h,int i,int j)
{
  register char *strtmp = BCX_TmpStr(11);
  strtmp[0]  = a;
  strtmp[1]  = b;
  strtmp[2]  = c;
  strtmp[3]  = d;
  strtmp[4]  = e;
  strtmp[5]  = f;
  strtmp[6]  = g;
  strtmp[7]  = h;
  strtmp[8]  = i;
  strtmp[9]  = j;
  strtmp[10] = 0;
  return strtmp;
}

// LEFT$() <=> left()
//    temp$ = LEFT$(tx_str$, 5)  <=>  strcpy(temp,(char*)left(tx_str,5));
char *left (char *S, int length)
{
  register int tmplen = strlen(S);
  char *strtmp = BCX_TmpStr(tmplen);
  strcpy (strtmp,S);
  if(length>tmplen)
    strtmp[tmplen] = 0;
  else
    strtmp[length] = 0;
  return strtmp;
}

// RIGHT$ <=> right()
//      tempA$ = RIGHT$(tx_str$, 4)  <=>  strcpy(tempA,(char*)right(tx_str,4));
char *right (char *S, int length)
{
  int tmplen = strlen(S);
  char *BCX_RetStr = BCX_TmpStr(tmplen);
  tmplen -= length;
  if (tmplen<0) tmplen = 0;
  return strcpy(BCX_RetStr, S + tmplen);
}


char *mid (char *S, int start, int length)
{
  register int tmplen = strlen(S);
  char *strtmp;
  if(start>tmplen||start<1) return BCX_TmpStr(1);
  if (length < 0) length = tmplen - start + 1;
  strtmp = BCX_TmpStr(length);
  strncpy(strtmp,&S[start-1],length);
  strtmp[length] = 0;
  return strtmp;
}


//---------------------------------

// -------------------


void gSendShortcutX() // not used any more here
{
    // MessageBox(0, "Shortcut detected", "Test point...", MB_OK);          
//                    gSK(74); // '#' TEST!!!!!  
//                    TEST:
//                    strcpy(A, (char*)SC[sc1 + sc2]); SENDSTR(A);
     strcpy(A, SCen[251]); SENDSTR(A);  // &&&&&&&


}


void gSendRest() // Send single characters if no shortcut
                {
                    //gDisplay(ChordonByte[1]);
                    //gDisplay(ChordonByte[2]);
                    //gDisplay(ChordonByte[3]);
                    if (Monitor == FALSE)
                       {
   //   MessageBox(0, "gSendRest()", "Test...", MB_OK);          
                  //  gSK(75); // '' TEST!!!!!            
                    gSK(ChordonByte[1]);
                    gSK(ChordonByte[2]);
                    gSK(ChordonByte[3]);
                       }
                }
// --------------




//----------------------------------

// This is only for sending characters in GKOS shortcut strings.
// Single characters must be sent directly: gSK(74) = # etc.
void SENDCHAR (char *tx_char)
{
    int KeyCodeSnd;
    KeyCodeSnd = AscToGKOS[(int) *tx_char];
    // Some special charcters coded as standard characters
               	  switch (KeyCodeSnd)
                  { 
                    case 127:// 'Delete Word Left (or Right)'
                   //MessageBox(NULL, "Delete Word Left","Test point...", MB_OK | MB_ICONINFORMATION);       
                       gSK(188); // Shift
                       gSK(62); // Ctrl
                       if (SpaceShift == 0){gSK(47); // LeftArrow
                                        }  
                       if (SpaceShift == 4){gSK(51); // RightArrow
                                        }  
                       SpaceShift = 0; // Set Default every time
                       gSK(57); // Delete
                       //gSK(187); // Shift OFF (not needed?)
                       Painting = FALSE;
                       return;
                    case 48:// 'Word Left'       
                       gSK(62);
                       gSK(47);
                       return;
                    case 52:// 'Word Right'       
                       gSK(62);
                       gSK(62);
                       return;
                    case 74:// 'Shift'   =  #  
                       gSK(188);
                       return;
                    case 76:// 'Ctrl'    =  &
                       gSK(62);
                       return;
                    case 88:// 'LeftArrow'= <      
                       gSK(47);
                       return;
                    case 92:// 'RightArrow'= >     
                       gSK(51);
                       return;
                    case 98:// 'BackSpace' = _    
                       gSK(46);
                       return;
                    case 82:// 'Delete'    = *     
                       gSK(57);
                       return;
                    case 86:// 'Scroll up'    = (     
                       //gSK(1); // test
                       ScrollUp();
                       return;
                    case 90:// 'Scroll down'    = )     
                       //gSK(1); // test
                       ScrollDown();
                       return;
                    case 38:// 'Send nothing'    = /
                         g123abc = TRUE;   
                         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Mode: 123>Space  ");
                         return;
            /*        case 1: // a => endings not accented
                         AccentedVovelsTEMP = FALSE; gSK(1);
                         return;     
                    case 21: // u => endings not accented
                         AccentedVovelsTEMP = FALSE; gSK(21);
                         return;     
                    case 15: // o => endings not accented
                         AccentedVovelsTEMP = FALSE; gSK(15);
                         return;  */   
                    case 87: // [ o/ Fin grammar
                         gSK(30); // or gSK(15);
                         if (!AccentedVovels) {gSK(15);} // o 
                         if (AccentedVovels) {gSK(30);}  // 
                         return;
                    case 91: // ] a/ Fin grammar
                         if (!AccentedVovels) {gSK(1);} // a 
                         if (AccentedVovels) {gSK(29);} // 
                         return;           
                    case 94:// 'Send nothing'    = :
                         XShifts = 2;  // AltGr activated by user   
                         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: AltGr  "); 
                         return;
                    case 95:// 'Send nothing'    = ;
                         //XShifts = 8;  // Shift+AltGr activared by user   
                         SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Shift: SHIFT+AltGr "); 
                         gShift = TRUE; XShifts = 2;
                         return;
                    case 80:// 'Send nothing'    = =  
                         return;
                        //gSK(1); // test
                       return;
                                          // Shift etc here...
                      
                  } // end of switch

   gSK(AscToGKOS[(int) *tx_char]);  
}

// ------------------------------------------------------

// For sending GKOS shortcut strings
void SENDSTR (char *tx_str)
{
  static int Nchar;
         // for Finnish grammar only:
    AccentedVovels = AccentedVovelsTEMP;              
    AccentedVovelsTEMP = TRUE; // Default = accented word (Fin) -> accented endings
                               // Any a, o, u included => word not accented
                               // e, i, y do not matter
                               // ,  make a word accented
  for(Nchar=1; Nchar<=strlen(tx_str); Nchar+=1)
    {
      //strcpy(tx_char,(char*)mid(tx_str,Nchar,1)); // <-ERROR
      tx_char = (char*) mid(tx_str,Nchar,1); // <-OK
      SENDCHAR(tx_char);
//      A = (char*)mid(tx_str,Nchar,1); // ei
//      SENDCHAR(A);

    }
    strcpy(tx_char, ""); sc1 = 1; sc2 = 2;

    AccentedVovels = AccentedVovelsTEMP; // Update for possible next ending 
}

// ==========================================================
//                  Selecting Shortcut Strings
//                   and creating the tables
// ==========================================================
//
// =================FIRST CHARACTER is Special===============

void gSendSC_Period()
{    sc1 = 0;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  . \ => grave  
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  .  / => acute (100)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(100);  break; // Send nothing (= =)
    //  . Ins => TBD
    case 58:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)


    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ------------------ 

void gSendSC_Comma()
{    sc1 = 41;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  , / => 
    case 38:          sc1 = 781; sc2 = 9; gSK(100);  break; // Send nothing (= =)
    //  , \  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  , ~ => TBD
    case 41:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  'x'  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// -----------------

void gSendSC_Exclamation()
{    sc1 = 82;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  ! / => comma above (p-key)
    case 38:          sc1 = 781; sc2 = 9; gSK(240);  break; // Send nothing (= =)
    //  \ Up  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    
  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    
    // Shift Alt Gr:
    case 60:  sc1 = 781;  sc2 = 11; break;  //  ! \ => Shift AltGr  (XShifts = 8)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       
                   gSendRest(); 
                   break;
    }
}
// ------
 
void gSendSC_Question()
{    sc1 = 123;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  ? \ => 'question mark' above (p-key)
    case 37:          sc1 = 781; sc2 = 9; gSK(241); break; // Send nothing (= =)
    //  ? /  => acute (100)  TEMPLATE ONLY
    //case 38:          sc1 = 781; sc2 = 9; gSK(100); break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 60:  sc1 = 781;  sc2 = 10; break;  //  ? \ => AltGr  (XShifts = 2)
    case 188:         sc2 = 41; break; // S (Shift)
    
    // Activate AltGr by Chordon ? \
    default:       gSendRest(); break;
    }
}

// ------------

void gSendSC_Hyphen()
{    sc1 = 164;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  - \  => grave ` (101) 
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  - /  => acute (100)  TEMPLATE ONLY
    //case 38:          sc1 = 781; sc2 = 9; gSK(100);  break; // Send nothing (= =)
    //  -  => u above ('-key)
    case 39:          sc1 = 781; sc2 = 9; gSK(226);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 


    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)

    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}
// ------


void gSendSC_Apostrophe()
{
    sc1 = 205;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  ' / => acute 
    case 38:          sc1 = 781; sc2 = 9; gSK(100); break; // Send nothing (= =)
    //  ' ~ => tilde (-key)
    case 40:          sc1 = 781; sc2 = 9; gSK(223); break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_UpArrow()
{    sc1 = 246;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''
    
    // Accents and special:
    //  Up \ => grave ` (101)
    //case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  Up / => grave ` (101) TEMPLATE ONLY
    //case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    // Up ^ => ring above
    case 41:          sc1 = 781; sc2 = 9; gSK(230); break; // Send nothing (= =)
    // Up Ins => line above (-key)
    case 58:          sc1 = 781; sc2 = 9; gSK(224); break; // Send nothing (= =)
    //  Up SYMB => F1...Fn
    case 60:          sc1 = 781; sc2 = 9; 
                      gSK(242);  break; // Send nothing (= =)
                      //gFUNC = TRUE; break;

  // Ins |  and   give  x  if not defined
    //case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_DownArrow()
{    sc1 = 287;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''
    
    // Accents and special:
    //  Dn \ => grave ` (101) TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  Dn / => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    // Dn  => cedilla = inv c below
    case 39:          sc1 = 781; sc2 = 9; gSK(118); gSK(100);  break; // Send nothing (= =)
    // Dn ~ => c below
    case 40:          sc1 = 781; sc2 = 9; gSK(118); gSK(59); gSK(100);  break; // Send nothing (= =)

    //  Dn SYMB => TBD TEMPLATE ONLY
    case 60:          sc1 = 781; sc2 = 9; gSK(243);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    //case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 


    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ---------
void gSendSC_BackSpace()
{    sc1 = 328;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''
    
    // Accents and special:
    //  Bs \ => grave ` (101) TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  Bs / => grave ` (101) TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 


    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_Space()
{    sc1 = 369;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''
    
    // Accents and special:
    //  Sp \ => grave ` (101) TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  Sp / => grave ` (101) TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 50:          sc2 = 40; break; // F (Space)
    case 188:          sc2 = 41; break; // S (Shift)           
    case 52: sc1 = 781; sc2 = 5; SpaceShift = 1; Space123 = 0; break; // WR (Space WordRight)           
    case 61: sc1 = 781; sc2 = 6; Space123 = 1; break; // 123abc (Space 123abc)           
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_Shift()
{    sc1 = 410;
	switch (ChordonByte[2])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    case 31:          sc2 = 31; break; // '.
    case 32:          sc2 = 32; break; // ',
    case 33:          sc2 = 33; break; // '!
    case 34:          sc2 = 34; break; // '?
    case 35:          sc2 = 35; break; // '-
    case 36:          sc2 = 36; break; // ''

    // Accents and special:
    //  ! / => two acutes above 
    case 38:          sc1 = 781; sc2 = 9; gSK(118); gSK(28);  break; // Send nothing (= =)
    //  ? \ => two dots above
    case 37:          sc1 = 781; sc2 = 9; gSK(39);  break; // Send nothing (= =)

    //TEST: Shift PgUp
   // case 44:  sc1 = 781; sc2 = 11; SpaceShift = 0; Space123 = 0; break;  //  Sh \ => Shift+AltGr  (XShifts = 8)       

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 


    case 42:          sc2 = 37; break; // U (UpArrow)
    case 43:          sc2 = 38; break; // D (DnArrow)
    case 46:          sc2 = 39; break; // B (BackSpace)
    case 48: sc1 = 781; sc2 = 7; SpaceShift = 0; break; // WL (Shift WordLeft)           
    case 50:          sc2 = 40; break; // F (Space)
    case 52: sc1 = 781; sc2 = 1; SpaceShift = 1; Space123 = 0; break; // WR (Shift WordRight)           
    case 57: sc1 = 781; sc2 = 8; SpaceShift = 4; break; // WR (Shift Delete) 
    case 61: // GKOS as Mouse (both SHIFT 123-abc and Sh_123abc do it!
                     sc1 = 781; sc2 = 9; gShift = TRUE; gSK(61);
                     break;
    case 188:         sc2 = 41; break; // S (Shift)           
    default:       gSendRest(); break;
    }
}

// ===================SECOND CHARCTER is special:=============================

// --------------

void gSendSC_Period2()
{    sc1 = 451;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ .  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / .  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}

// ------------------ 

void gSendSC_Comma2()
{    sc1 = 481;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ ,  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / ,  => grave ` (101)
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}

// -----------------

void gSendSC_Exclamation2()
{    sc1 = 511;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ !  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / !  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}
// ------
 
void gSendSC_Question2()
{    sc1 = 541;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ ?  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / ?  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}

// ------------

void gSendSC_Hyphen2()
{    sc1 = 571;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ -  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / -  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}
// ------


void gSendSC_Apostrophe2()
{
    sc1 = 601;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ '  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / '  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_UpArrow2()
{    sc1 = 631;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    
    // Accents and special:
    //  \ Up  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / Up  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_DownArrow2()
{    sc1 = 661;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ Dn  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / Dn  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 

    default:       gSendRest(); break;
    }
}

// ---------
void gSendSC_BackSpace2()
{    sc1 = 691;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

    // Accents and special:
    //  \ Bs  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / Bs  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)    

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_Space2()
{    sc1 = 721;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '
    
    // Accents and special:
    //  \ Sp  => grave ` (101)  TEMPLATE ONLY
    case 37:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)
    //  / Sp  => grave ` (101)  TEMPLATE ONLY
    case 38:          sc1 = 781; sc2 = 9; gSK(101);  break; // Send nothing (= =)    

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    
    default:       gSendRest(); break;
    }
}

// ---------

void gSendSC_Shift2()
{    sc1 = 751;
	switch (ChordonByte[1])
    { 
    case 1:          sc2 = 1; break; // 'a
    case 2:          sc2 = 2; break; // 'b
    case 3:          sc2 = 3; break; // 'c
    case 4:          sc2 = 4; break; // 'd
    case 5:          sc2 = 5; break; // 'e
    case 6:          sc2 = 6; break; // 'f
    case 7:          sc2 = 7; break; // 'g
    case 8:          sc2 = 8; break; // 'h
    case 9:          sc2 = 9; break; // 'i
    case 10:          sc2 = 10; break; // j'
    case 11:          sc2 = 11; break; // 'k
    case 12:          sc2 = 12; break; // 'l
    case 13:          sc2 = 13; break; // 'm
    case 14:          sc2 = 14; break; // 'n
    case 15:          sc2 = 15; break; // 'o
    case 16:          sc2 = 16; break; // 'p
    case 17:          sc2 = 17; break; // 'q
    case 18:          sc2 = 18; break; // 'r
    case 19:          sc2 = 19; break; // 's
    case 20:          sc2 = 20; break; // 't
    case 21:          sc2 = 21; break; // 'u
    case 22:          sc2 = 22; break; // 'v
    case 23:          sc2 = 23; break; // 'w
    case 24:          sc2 = 24; break; // 'x
    case 25:          sc2 = 25; break; // 'y
    case 26:          sc2 = 26; break; // 'z
    case 27:          sc2 = 27; break; // 'u
    case 28:          sc2 = 28; break; // '
    case 29:          sc2 = 29; break; // '
    case 30:          sc2 = 30; break; // '

  // Ins |  and   give  x  if not defined
    case 58:          sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 102:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 103:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 104:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 39:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 40:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
    case 41:         sc1 = 781; sc2 = 9; gSK(24); break; // Send nothing 
 
    default:       gSendRest(); break;

    }
}
// 781 chordon shortcuts total !!!

// ----------- Additional functions -----------


// Symb + x:
void gSendSC_Symb()
{    sc1 = 781;
     sc2 = 9; // Default = send no shorcut
     if (ChordonByte[2] <= 59){  // Less than any char before shifts
                         sc1 = 781; sc2 = 9;
                         gSYMB = TRUE; // SYMB ON
        // gSK() does not take into account g123abc and gSYMB status
                         gSK(gSetRef(ChordonByte[2])); //SYMB
                         return;
                         }
	switch (ChordonByte[2])
    { 
    // SYMB_Up  = SYMB Up = Scroll up  (SYMB_Up/Dn not implemented)     
    case 1:   sc1 = 781; sc2 = 9; gSK(24); break; // Just test SYMB_a
    default:
       //gSK(60);// Send SYMB     
       gSendRest(); break;
    }
}

// x + Symb:
void gSendSC_Symb2()
{    sc1 = 781;
     sc2 = 9; // Default = send no shorcut
	switch (ChordonByte[1])
    { 
    // Activate WinKeys etc
    // w SYMB => WinKeys
    //case 23:   sc1 = 781; sc2 = 9; gSK(245); break;
    // TH SYMB => TBD
    //case 27:   sc1 = 781; sc2 = 9; gSK(244); break;
    // Up SYMB => TBD This must be at 'gSendSC_Up'
    //case 42:   gFunc = TRUE; break;
    // Dn SYMB => TBD  This must be at 'gSendSC_Dn'
    //case 43:   sc1 = 781; sc2 = 9; gSK(235); break;

    default:
       gSendRest(); break;
    }
}

// BackSlash (\):
void gSendSC_BackSlash2()
{    sc1 = 781;
     sc2 = 9; // Defult = send nothing
	switch (ChordonByte[1])
    { 
    // Activate Shift+AltGr by Chordon Sh \
    // a \ => dot above (.-key)
    case 1:   sc1 = 781; sc2 = 9; gSK(59); gSK(118); gSK(31); break;
    // m \ =>  overstrike (.-key)
    case 13:   sc1 = 781; sc2 = 9; gSK(239);  break;
    // o \ => TEMP: inv ^ ('-key)
    case 15:   sc1 = 781; sc2 = 9; gSK(225); break;
    //case 34:   sc1 = 781; sc2 = 10; break;  //   ? \ => AltGr  (XShifts = 2)       
    //case 188:  sc1 = 781; sc2 = 11; break;  //  Sh \ => Shift+AltGr  (XShifts = 8)       

    default:
      gSendRest(); 
      break;
    }
}

// Slash (/):
void gSendSC_Slash2()
{    sc1 = 781;
     sc2 = 9; // Default = send nothing
	switch (ChordonByte[1])
    { 
    // Activate Shift+AltGr by Chordon Sh \
    // c / => dot below (.-key)
    case 3:   sc1 = 781; sc2 = 9; gSK(118); gSK(31); break;
    // s / => TEMP: ^ (-key)
    case 19:   sc1 = 781; sc2 = 9; gSK(41); break;
    //  u / => u above ('-key)
    case 21:          sc1 = 781; sc2 = 9; gSK(226);  break; // Send nothing (= =)
    //case 34:   sc1 = 781; sc2 = 10; break;  //   ? \ => AltGr  (XShifts = 2)       
    //case 188:  sc1 = 781; sc2 = 11; break;  //  Sh \ => Shift+AltGr  (XShifts = 8)       

    default:
       gSendRest(); break;
    }
}


// PgUp :  
void gSendSC_PgUp2()
{    sc1 = 781;
     sc2 = 9; // Default (Send Nothing, no word shortcut)
	switch (ChordonByte[1])
    { 
    // a PgUp => (Alt) 
    case 1:  gSK(63);  break;
    // b PgUp => Break/Pause
    case 2:  gPressKey(0x13);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Break/Pause Key"); 
             break;
    //  d PgUp => (delete)
    case 4:  gSK(57);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Del Key pressed "); 
             break;
    //  e PgUp => (esc)
    case 5:  gSK(56);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Esc pressed "); 
             break;
    //  g PgUp => (Alt Gr)
    case 7:  gSK(118);  break;
    //  o PgUp =>
    case 15: gSK(62); gSK(2); gSK(190); // Ctrl ON, b, Ctrl OFF  
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Bold ON/OFF "); 
             break;
    //  p PgUp => Print Screen
    case 16: gPressKey(0x2c);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Print Screen "); 
             break;
    //  q PgUp =>
    case 17: gPressKey(0x58);  break;
    //  h PgUp => (home)
    case 8: gPressKey(0x70); // F1 key
            SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Help key pressed "); 
            break;

    //  i PgUp => (italics)
    case 9: gSK(62); gSK(9); gSK(190); // Ctrl ON, i, Ctrl OFF 
           SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Italics ON/OFF "); 
           break;

    default:
       gSendRest(); break;
    }
}

// PgDn :  
void gSendSC_PgDn2()
{    sc1 = 781;
     sc2 = 9; // Default (Send Nothing, no word shortcut)
	switch (ChordonByte[1])
    { 
    // b PgDn => Bold
    case 2:  gSK(62); gSK(2); gSK(190); // Ctrl ON, b, Ctrl OFF  
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Bold ON/OFF "); 
             break;
    // c PgDn => SetScroll(TRUE/FALSE)
    case 3:  gPressKey(0x58); gPressKey(0x08); // x + Backspace make scroll lock more reliable! 
             if (gScroll == FALSE){gSK(124); break;}
             gSK(252);
             break; 
    //  e PgDn => (end)
    case 5:  gSK(53);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " End key pressed "); 
             break;
    //  f PgDn => Font ?
    case 6:  gPressKey(0x58);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Font selection "); 
             break;
    //  k PgDn => Right Windows Key
    case 11:  gPressKey(0x5c);
              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Right WinKey"); 
              break;
    //  s PgDn => Left Windows Key
    case 19: gPressKey(0x5b);
              SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " WinKey Start"); 
              break;
    //  u PgDn =>
    case 21: gSK(62); gSK(21); gSK(190); // Ctrl ON, u, Ctrl OFF  
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " Underline ON/OFF "); 
             break;
    //  v PgDn => TBD
    case 22: gPressKey(0x58);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " TBD key pressed "); 
             break;
    //  m PgDn => Windows Applications/Menu Key
    case 13: gPressKey(0x5d);
             SendMessage(hWndStatusBar, SB_SETTEXT, 0, (LPARAM) " WinKey Menu "); 
             break;
    //  n PgDn => SetNumLock(TRUE/FALSE)
    case 14: // NumLock should not be used on laptops because
             // keys J,K,L (= GKOS D,E,F) change to 1,2,3
         if (gNumLock == FALSE){gSK(120); break;}
         gSK(248);
          break; 

    default:
       gSendRest();
       break;
    }
}




void gSendSC_WordRight()
{    sc1 = 781;
     sc2 = 9; // Defult
	switch (ChordonByte[2])
    { 
    case 188:         sc2 = 1; SpaceShift = 1; Space123 = 0; break; // WR Sh => Next word in capitals
                      // WR Sh is not reliable. Use Sh WR instead!!! (in Shift table)
    case 60:          sc2 = 2; SpaceShift = 2; break; // WR SYMB => 
    case 61:          sc2 = 3; Space123 = 1; break; // WR 123-ABC =>
    case 57:          sc2 = 4; SpaceShift = 4; break; // WR Delete => Delete next word
    default:
       SpaceShift = 0;
       Space123 = 0;
       gSendRest(); break;
    }
}
// Not in use yet:
void gSendSC_WordLeft()
{    sc1 = 781;
     sc2 = 9; // Defult
	switch (ChordonByte[2])
    { 
    case 188:         sc2 = 1; break; // WL Sh => 
    case 60:          sc2 = 2; break; // WL SYMB => 
    case 61:          sc2 = 3; break; // WL 123-ABC =>
    default:
       SpaceShift = 0;
       Space123 = 0;
       gSendRest(); break;
    }
}

// Select language for shortcut string
void gSendShortcutSelect()
{ // Language: 1 = en, 2 = fi, 3 = se, 4 = de
   if (sc1 == 781){ // Special characters, common to all languages
                   strcpy(A, SCen[sc1 + sc2]);
                   SENDSTR(A); return; //gPrint(A);
                   }
   switch (Language)
   {
   case 1:
        strcpy(A, SCen[sc1 + sc2]);  // &&&&&&&
        SENDSTR(A); return; //gPrint(A);     
   case 2:
        strcpy(A, SCfi[sc1 + sc2]); // &&&&&&&
        SENDSTR(A); return; //gPrint(A);     
   case 3:
        strcpy(A, SCxx[sc1 + sc2]); // &&&&&&&
        SENDSTR(A); return; //gPrint(A);     
   case 4:
        strcpy(A, SCyy[sc1 + sc2]); // &&&&&&&
        SENDSTR(A); return; //gPrint(A);     
   default:
        strcpy(A, SCen[sc1 + sc2]); // &&&&&&&
        SENDSTR(A); return; //gPrint(A);                
   }
}     

// After having found a sequence like [AKU] ! a [AKU]  this is entered:
void gSendShortcut()
     {
     if ((g123abc) | (gSYMB) | (gESC) | (gAlt)){return;}               
                    
                    
     // TEST:
            /*        char *TempChar = gShortcut[286];
                    gPrint(TempChar); // includes Monitor == true
                    return;
                    */
// Depending on the first special character of the chord, choose
// what to do with the second byte (letter or special as second char)
	switch (ChordonByte[1]) 
            { 
                case 36: // '
                   // TempChar = "['] "; gPrint(TempChar);
                    gSendSC_Apostrophe(); gSendShortcutSelect();
                    return; //break;  
                case 35: // -
                   // TempChar = "[-] "; gPrint(TempChar);
                    gSendSC_Hyphen(); gSendShortcutSelect();
                    return; //break; 
                case 31: // .
                   // TempChar = "[.] "; gPrint(TempChar);
                    gSendSC_Period(); gSendShortcutSelect();
                    return; //break; 
                case 32: // ,
                   // TempChar = "[,] "; gPrint(TempChar);
                    gSendSC_Comma(); gSendShortcutSelect();
                    return; //break; 
                case 34: // ?
                   // TempChar = "[?] "; gPrint(TempChar);
                    gSendSC_Question(); gSendShortcutSelect();
                    return; //break; 
                case 33: // !
                   // TempChar = "[!] "; gPrint(TempChar);
                    gSendSC_Exclamation(); gSendShortcutSelect();
                    //strcpy(A, (char*)SC[sc1 + sc2]);
                    //SENDSTR(A); //gPrint(A);
                    return; //break; 
                case 42: // UpArr
                   // TempChar = "[UpArrow] "; gPrint(TempChar);
                    gSendSC_UpArrow(); gSendShortcutSelect();
                    return; //break; 
                case 43: // DnArr
                   // TempChar = "[DnArrow] "; gPrint(TempChar);
                    gSendSC_DownArrow(); gSendShortcutSelect();
                    return; //break;
                case 46: // BackSpace (B)
                   // TempChar = "[BackSpace] "; gPrint(TempChar);
                    gSendSC_BackSpace(); gSendShortcutSelect();
                    return; //break; 
                case 50: // Space (F)
                   // TempChar = "[Space] "; gPrint(TempChar);
                    gSendSC_Space(); gSendShortcutSelect();
                    return; //break;
                case 188: // Shift (S)
                   // TempChar = "[Shift] "; gPrint(TempChar);
                    gSendSC_Shift(); gSendShortcutSelect();
                    return; //break; 
                case 52: // WordRight
                   // TempChar = "[Shift] "; gPrint(TempChar);
                    gSendSC_WordRight(); gSendShortcutSelect();
                    return; //break; 
                case 48: // WordLeft
                   // TempChar = "[Shift] "; gPrint(TempChar);
                    gSendSC_WordLeft(); gSendShortcutSelect();
                    return; //break; 
                case 60: // SYMB (quick selection fo numbers etc: SYMB_a = 1)
                    // TempChar = "[Shift] "; gPrint(TempChar);
                    gSendSC_Symb(); gSendShortcutSelect();
                    return; //break; 
                default: 
                    break;
                }
// Depending on the second special character of the chord, choose
// what to do with the first byte (only letter is possible as first one)

            	switch (ChordonByte[2]) 
            	    { 
                            case 36: // '
                               // TempChar = "['] "; gPrint(TempChar);
                                gSendSC_Apostrophe2(); gSendShortcutSelect();
                                return; //break;  
                            case 35: // -
                               // TempChar = "[-] "; gPrint(TempChar);
                                gSendSC_Hyphen2(); gSendShortcutSelect();
                                return; //break; 
                            case 31: // .
                               // TempChar = "[.] "; gPrint(TempChar);
                                gSendSC_Period2(); gSendShortcutSelect();
                                return; //break; 
                            case 32: // ,
                               // TempChar = "[,] "; gPrint(TempChar);
                                gSendSC_Comma2(); gSendShortcutSelect();
                                return; //break; 
                            case 34: // ?
                               // TempChar = "[?] "; gPrint(TempChar);
                                gSendSC_Question2(); gSendShortcutSelect();
                                return; //break; 
                            case 33: // !
                               // TempChar = "[!] "; gPrint(TempChar);
                                gSendSC_Exclamation2(); gSendShortcutSelect();
                                return; //break; 
                            case 42: // UpArr
                               // TempChar = "[UpArrow] "; gPrint(TempChar);
                                gSendSC_UpArrow2(); gSendShortcutSelect();
                                return; //break; 
                            case 43: // DnArr
                               // TempChar = "[DnArrow] "; gPrint(TempChar);
                                gSendSC_DownArrow2(); gSendShortcutSelect();
                                return; //break; 
                            case 46: // BackSpace
                               // TempChar = "[BackSpace] "; gPrint(TempChar);
                                gSendSC_BackSpace2(); gSendShortcutSelect();
                                return; //break; 
                            case 50: // Space
                               // TempChar = "[Space] "; gPrint(TempChar);
                                gSendSC_Space2(); gSendShortcutSelect();
                                return; //break;
                            case 188: // Shift
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_Shift2(); gSendShortcutSelect();
                                return; //break; 
                            case 37: // BackSlash
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_BackSlash2(); gSendShortcutSelect();
                                return; //break; 
                            case 38: // Slash
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_Slash2(); gSendShortcutSelect();
                                return; //break; 
                            case 60: // SYMB
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_Symb2(); gSendShortcutSelect();
                                return; //break; 
                            case 44: // PgUp
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_PgUp2(); gSendShortcutSelect();
                                return; //break; 
                            case 45: // PgDn
                               // TempChar = "[Shift] "; gPrint(TempChar);
                                gSendSC_PgDn2(); gSendShortcutSelect();
                                return; //break; 
                            default: ;
                       }             
              gSendRest();
  
     } // end of function




//=====================================================================================


//--------------------------------

static void SetTables(void)
        {  
     // This table is needed for QWERTY input only

     // ChordValue  - GKOS Ref 

        gCharRef[1] = 1; 
        gCharRef[2] = 2; 
        gCharRef[4] = 3; 
        gCharRef[8] = 4; 
        gCharRef[16] = 5; 
        gCharRef[32] = 6; 
        gCharRef[24] = 7; 
        gCharRef[25] = 8; 
        gCharRef[26] = 9; 
        gCharRef[28] = 10; 
        gCharRef[48] = 11; 
        gCharRef[49] = 12; 
        gCharRef[50] = 13; 
        gCharRef[52] = 14; 
        gCharRef[3] = 15; 
        gCharRef[11] = 16; 
        gCharRef[19] = 17; 
        gCharRef[35] = 18; 
        gCharRef[6] = 19;
        gCharRef[14] = 20; 
        gCharRef[22] = 21; 
        gCharRef[38] = 22; 
        gCharRef[40] = 23; 
        gCharRef[41] = 24; 
        gCharRef[42] = 25; 
        gCharRef[44] = 26; // z
        gCharRef[5] = 27; 
        gCharRef[13] = 28; 
        gCharRef[21] = 29; 
        gCharRef[37] = 30; // 
        gCharRef[34] = 31; 
        gCharRef[20] = 32; 
        gCharRef[12] = 33; 
        gCharRef[33] = 34; 
        gCharRef[17] = 35; 
        gCharRef[10] = 36; 
        gCharRef[51] = 37; 
        gCharRef[30] = 38; 
        gCharRef[53] = 39; 
        gCharRef[46] = 40; 
        gCharRef[29] = 41; 
        gCharRef[9] = 42; 
        gCharRef[36] = 43; 
        gCharRef[27] = 44; 
        gCharRef[54] = 45; 
        gCharRef[7] = 46; 
        gCharRef[15] = 47; 
        gCharRef[23] = 48; 
        gCharRef[39] = 49; 
        gCharRef[56] = 50; 
        gCharRef[57] = 51;
        gCharRef[58] = 52; // Word Right
        gCharRef[60] = 53; // End
        gCharRef[59] = 54; 
        gCharRef[61] = 55; 
        gCharRef[31] = 56; 
        gCharRef[62] = 57; 
        gCharRef[43] = 58; // Insert
        //gCharRef[53] = 102;// | 
        //gCharRef[46] = 103;//  
        //gCharRef[29] = 104;//  
        
        gCharRef[18] = 188; //59; // Shift 59 or 188
        gCharRef[45] = 60; //126; // not 60 this time, <SYMB ON> 
        gCharRef[63] = 61; // 123-ABC 
        gCharRef[47] = 62; // Ctrl (must be 47 but was 46!!!)
        gCharRef[55] = 63; // Alt
       
        gCharRef[64] = 64; 
        gCharRef[65] = 65; 
         


// --- Using this might be wiser than manually selecting PC kbd layout: ----
// The GetOEMCP function retrieves the current original equipment
// manufacturer (OEM) code-page identifier for the system.
// UINT GetOEMCP(void);
// 1140 	IBM EBCDIC - U.S./Canada (037 + Euro symbol)
//  037 	IBM EBCDIC - U.S./Canada
// 1143 	IBM EBCDIC - Finland/Sweden (20278 + Euro symbol)
//20278 	IBM EBCDIC - Finland/Sweden


                    
        // FIN/SWE table
        // PC scancode; 1 = Shift; 2 = AtGr; 3 = Ctrl; 5 = Alt[xxx] (NumPad);
        // 8 = Shift+AltGr; 9 = Shift KEY
        //                     
     // PC scancode ; Shifts needed  ; Monitor display

                       		    gChar[0]="[NULL]";
        iChar[1]=0x41; iShifts[1]=0; //gChar[1]="a";
        iChar[2]=0x42; iShifts[2]=0; //gChar[2]="b";
        iChar[3]=0x43; iShifts[3]=0; //gChar[3]="c";
        iChar[4]=0x44; iShifts[4]=0; //gChar[4]="d";
        iChar[5]=0x45; iShifts[5]=0; //gChar[5]="e";
        iChar[6]=0x46; iShifts[6]=0; //gChar[6]="f";
        iChar[7]=0x47; iShifts[7]=0; //gChar[7]="g";
        iChar[8]=0x48; iShifts[8]=0; //gChar[8]="h";
        iChar[9]=0x49; iShifts[9]=0; //gChar[9]="i";
        iChar[10]=0x4a; iShifts[10]=0; //gChar[10]="j";
        iChar[11]=0x4b; iShifts[11]=0; //gChar[11]="k";
        iChar[12]=0x4c; iShifts[12]=0; //gChar[12]="l";
        iChar[13]=0x4d; iShifts[13]=0; //gChar[13]="m";
        iChar[14]=0x4e; iShifts[14]=0; //Char[14]="n";
        iChar[15]=0x4f; iShifts[15]=0; //gChar[15]="o";
        iChar[16]=0x50; iShifts[16]=0; //gChar[16]="p";
        iChar[17]=0x51; iShifts[17]=0; //gChar[17]="q";
        iChar[18]=0x52; iShifts[18]=0; //gChar[18]="r";
        iChar[19]=0x53; iShifts[19]=0; //gChar[19]="s";
        iChar[20]=0x54; iShifts[20]=0; //gChar[20]="t";
        iChar[21]=0x55; iShifts[21]=0; //gChar[21]="u";
        iChar[22]=0x56; iShifts[22]=0; //gChar[22]="v";
        iChar[23]=0x57; iShifts[23]=0; //gChar[23]="w";
        iChar[24]=0x58; iShifts[24]=0; //gChar[24]="x";
        iChar[25]=0x59; iShifts[25]=0; //gChar[25]="y";
        iChar[26]=0x5a; iShifts[26]=0; //gChar[26]="z";
        iChar[27]=0x55; iShifts[27]=0; //gChar[27]="";   //  / th
        iChar[28]=0xdd; iShifts[28]=0; //gChar[28]="";   //  / and
        iChar[29]=0xde; iShifts[29]=0; //gChar[29]="";   //  / the
        iChar[30]=0xc0; iShifts[30]=0; //gChar[30]="";   //  / of
        iChar[31]=0xbe; iShifts[31]=0; //gChar[31]=".";
        iChar[32]=0xbc; iShifts[32]=0; //gChar[32]=",";
        iChar[33]=0x31; iShifts[33]=1; //gChar[33]="!"; // shift 1
        iChar[34]=0xbb; iShifts[34]=1; //gChar[34]="?"; // 
        iChar[35]=0xbd; iShifts[35]=0; //gChar[35]="-"; // 
        iChar[36]=0xbf; iShifts[36]=0; //gChar[36]="'"; //
        iChar[37]=0xbb; iShifts[37]=2; //gChar[37]="<backslash>";  //
        iChar[38]=0x37; iShifts[38]=1; //gChar[38]="/"; // 
        iChar[39]=0xba; iShifts[39]=0; //gChar[39]=""; // ()
        iChar[40]=0xba; iShifts[40]=2; //gChar[40]="~"; // tilde, 'worm'
        iChar[41]=0xba; iShifts[41]=1; //gChar[41]="^";  //  ^
        iChar[42]=0x26; iShifts[42]=0; //gChar[42]="<UpArrow>"; // arrow up
        iChar[43]=0x28; iShifts[43]=0; //gChar[43]="<DownArrow>"; // arrow down
        iChar[44]=0x21; iShifts[44]=0; //gChar[44]="<PageUp>"; // page up
        iChar[45]=0x22; iShifts[45]=0; //gChar[45]="<PageDown>"; // page down
        iChar[46]=0x08; iShifts[46]=0; //gChar[46]="<BkSp>"; //&cBackSpace; //<Backspace>; // backspace 8
        iChar[47]=0x25; iShifts[47]=0; //gChar[47]="<LeftArrow>"; // arrow left
        iChar[48]=0x25; iShifts[48]=3; //gChar[48]="<Word Left>"; // word left
        iChar[49]=0x24; iShifts[49]=0; //gChar[49]="<Home>"; // home
        iChar[50]=0x20; iShifts[50]=0; //gChar[50]=" "; // &cSpace; // space
        iChar[51]=0x27; iShifts[51]=0; //gChar[51]="<RightArrow>"; // arrow right
        iChar[52]=0x27; iShifts[52]=3; //gChar[52]="<Word Right>"; // word right
        iChar[53]=0x23; iShifts[53]=0; //gChar[53]="<End>"; // end
        iChar[54]=0x0d; iShifts[54]=0; //gChar[54]="<ENTER>"; // &cEnter ; // ENTER d'13'
        iChar[55]=0x09; iShifts[55]=0; //gChar[55]="<Tab>"; // tab
        iChar[56]=0x1b; iShifts[56]=0; //gChar[56]="<Esc>"; // esc
        iChar[57]=0x2e; iShifts[57]=0; //gChar[57]="<Del>"; // delete
        iChar[58]=0x2d; iShifts[58]=0; //gChar[58]="<Ins>"; // indsert
        iChar[59]=0x10; iShifts[59]=0; //gChar[59]="<SHIFT ON>"; // Shift ON (0x10, 0xa0, 0xa1)
        iChar[60]=0x99; iShifts[60]=0; //gChar[60]="<SYMB>"; //<IR Header>"; // IR link header
        iChar[61]=0x99; iShifts[61]=0; //gChar[61]="<123-ABC>"; // 123-ABC
        iChar[62]=0xa2; iShifts[62]=0; //gChar[62]="<Ctrl ON>"; // Ctrl ON
        iChar[63]=0x12; iShifts[63]=0; //gChar[63]="<Alt ON>"; // Alt ON
        iChar[64]=0x31; iShifts[64]=0; //gChar[64]="1";        
        iChar[65]=0x32; iShifts[65]=0; //gChar[65]="2";        
        iChar[66]=0x33; iShifts[66]=0; //gChar[66]="3";
        iChar[67]=0x34; iShifts[67]=0; //gChar[67]="4";
        iChar[68]=0x35; iShifts[68]=0; //gChar[68]="5";
        iChar[69]=0x36; iShifts[69]=0; //gChar[69]="6";
        iChar[70]=0x30; iShifts[70]=0; //gChar[70]="0";
        iChar[71]=0x37; iShifts[71]=0; //gChar[71]="7";
        iChar[72]=0x38; iShifts[72]=0; //gChar[72]="8";
        iChar[73]=0x39; iShifts[73]=0; //gChar[73]="9";
        iChar[74]=0x33; iShifts[74]=1; //gChar[74]="#";        
        iChar[75]=0xdc; iShifts[75]=1; //gChar[75]="";        
        iChar[76]=0x36; iShifts[76]=1; //gChar[76]="&";
        iChar[77]=0x32; iShifts[77]=2; //gChar[77]="@";
        iChar[78]=0xbb; iShifts[78]=0; //gChar[78]="+";
        iChar[79]=0x35; iShifts[79]=1; //gChar[79]="%";
        iChar[80]=0x30; iShifts[80]=1; //gChar[80]="=";  //
        iChar[81]=0x59; iShifts[81]=0; //gChar[81]="<yen>";
        iChar[82]=0xbf; iShifts[82]=1; //gChar[82]="*";  //
        iChar[83]=0x35; iShifts[83]=2; //gChar[83]="";
        iChar[84]=0x34; iShifts[84]=2; //gChar[84]="$";        
        iChar[85]=0x33; iShifts[85]=2; //gChar[85]="";        
        iChar[86]=0x38; iShifts[86]=1; //gChar[86]="(";  // used as scroll up in shortcuts
        iChar[87]=0x38; iShifts[87]=2; //gChar[87]="[";        // US: [ FIN: 
        iChar[88]=0xe2; iShifts[88]=0; //gChar[88]="<"; 
        iChar[89]=0x37; iShifts[89]=2; //gChar[89]="{";
        iChar[90]=0x39; iShifts[90]=1; //gChar[90]=")";  // used as scroll down in shortcuts
        iChar[91]=0x39; iShifts[91]=2; //gChar[91]="]";        // US: ] FIN: 
        iChar[92]=0xe2; iShifts[92]=1; //gChar[92]=">";
        iChar[93]=0x30; iShifts[93]=2; //gChar[93]="}";
        iChar[94]=0xbe; iShifts[94]=1; //gChar[94]=":";        
        iChar[95]=0xbc; iShifts[95]=1; //gChar[95]=";";  //US: ;: FIN: 
        iChar[96]=0x31; iShifts[96]=1; //gChar[96]="<inv.!>";
        iChar[97]=0xbb; iShifts[97]=1; //gChar[97]="<inv.?>";
        iChar[98]=0xbd; iShifts[98]=1; //gChar[98]="_";
        iChar[99]=0x32; iShifts[99]=1; //gChar[99]="<quote>"; // " (quote)
        iChar[100]=0xdb; iShifts[100]=0; //gChar[100]="";
        iChar[101]=0xdb; iShifts[101]=1; //gChar[101]="`";
        iChar[102]=0xe2; iShifts[102]=2; //gChar[102]="|";
        iChar[103]=0xdc; iShifts[103]=0; //gChar[103]="";
        iChar[104]=0x34; iShifts[104]=1; //gChar[104]="";        
        iChar[105]=0xdd; iShifts[105]=1; //gChar[105]="<inv.^>";        
        iChar[106]=0x70; iShifts[106]=0; //gChar[106]="<F1>";
        iChar[107]=0x71; iShifts[107]=0; //gChar[107]="<F2>";
        iChar[108]=0x72; iShifts[108]=0; //gChar[108]="<F3>";
        iChar[109]=0x73; iShifts[109]=0; //gChar[109]="<F4>";
        iChar[110]=0x74; iShifts[110]=0; //gChar[110]="<F5>";
        iChar[111]=0x75; iShifts[111]=0; //gChar[111]="<F6>";
        iChar[112]=0x76; iShifts[112]=0; //gChar[112]="<F7>";
        iChar[113]=0x77; iShifts[113]=0; //gChar[113]="<F8>";
        iChar[114]=0x78; iShifts[114]=0; //gChar[114]="<F9>";        
        iChar[115]=0x79; iShifts[115]=0; //gChar[115]="<F10>";        
        iChar[116]=0x7a; iShifts[116]=0; //gChar[116]="<F11>";
        iChar[117]=0x7b; iShifts[117]=0; //gChar[117]="<F12>";
        iChar[118]=0x41; iShifts[118]=0; //gChar[118]="<AltGr ON>"; // !!!
        iChar[119]=0x14; iShifts[119]=0; //gChar[119]="<CAPS ON>";  // !!!
        iChar[120]=0x90; iShifts[120]=0; //gChar[120]="<Num Lock ON>"; // !!!
        iChar[121]=0x5d; iShifts[121]=0; //gChar[121]="<Windows menu>";
        iChar[122]=0x5b; iShifts[122]=0; //gChar[122]="<Windows start>";
        iChar[123]=0x2c; iShifts[123]=0; //gChar[123]="<Print Screen>";
        iChar[124]=0x91; iShifts[124]=0; //gChar[124]="<Scroll Lock (ON/OFF)>";        
        iChar[125]=0x13; iShifts[125]=0; //gChar[125]="<Pause/Break>";        
        iChar[126]=0x00; iShifts[126]=0; //gChar[126]="<SYMB>"; // !!!
        iChar[127]=0x00; iShifts[127]=0; //gChar[127]="<reserved>"; // Delete word left
        
        iChar[128]=0x00; iShifts[128]=0; //gChar[128]=" <Mouse data Start> ";
        iChar[129]=0xaf; iShifts[129]=0; //gChar[129]="Volume +";
        iChar[130]=0xb1; iShifts[130]=0; //gChar[130]="Navi Left / Skip-";
        iChar[131]=0xae; iShifts[131]=0; //gChar[131]="Volume -";
        iChar[132]=0x00; iShifts[132]=0; //gChar[132]="Channel +";
        iChar[133]=0xb0; iShifts[133]=0; //gChar[133]="Navi Right / Skip+";
        iChar[134]=0x00; iShifts[134]=0; //gChar[134]="Channel -";
        
        iChar[141]=0xad; iShifts[141]=0; //gChar[141]="Mute/(Menu)"; // M


        iChar[143]=0xb3; iShifts[143]=0; //gChar[143]="Play/Pause"; // O
        
        iChar[174]=0xb2; iShifts[174]=0; //gChar[174]="Stop"; //BackSpace

        iChar[178]=0xb3; iShifts[178]=0; //gChar[178]="Play/Pause"; // Space
                        
        iChar[187]=0x10; iShifts[187]=0; //gChar[187]="<Shift OFF>";
        iChar[188]=0x99; iShifts[188]=9; //gChar[188]="<Shift KEY>";

        iChar[189]=0; iShifts[189]=0; //gChar[189]="<123-ABC OFF>";

        iChar[190]=0xa2; iShifts[190]=0; //gChar[190]="<Ctrl OFF>"; // Key up
        iChar[191]=0x12; iShifts[191]=0; //gChar[191]="<Alt OFF>";  // Key up

        // Additional characters and functions on extended qwerty kbd (tytyty)
        // To selcet a char on a QWERTY key: 2 = AltGr, 8 = Shitf + AltGr
        iChar[221]=0xdb; iShifts[221]=2; // -key       'cedilla left'
        iChar[222]=0xdb; iShifts[222]=8; //             'cedilla right'
        iChar[223]=0xba; iShifts[223]=2; // -key       tilde (worm above)
        iChar[224]=0xba; iShifts[224]=8; //             'line above'
        iChar[225]=0xbf; iShifts[225]=2; // '-key       caron (v above)
        iChar[226]=0xbf; iShifts[226]=8; //             'U above'
        iChar[227]=0xbd; iShifts[227]=2; // --key       longer line 
        iChar[228]=0xbd; iShifts[228]=8; //             dot above
        iChar[229]=0xdd; iShifts[229]=2; // -key       double acute
        iChar[230]=0xdd; iShifts[230]=8; //             ring above
        iChar[231]=0xde; iShifts[231]=2; // -key       ae
        iChar[232]=0xde; iShifts[232]=8; //             AE
        iChar[233]=0xc0; iShifts[233]=2; // -key       Danish 
        iChar[234]=0xc0; iShifts[234]=8; //             Danish 
        iChar[235]=0xdc; iShifts[235]=2; // -key       overstrike
        iChar[236]=0xdc; iShifts[236]=8; //
        iChar[237]=0xbe; iShifts[237]=2; // .-key       dot below
        iChar[238]=0xbe; iShifts[238]=8; //             dot above
        iChar[239]=0x4c; iShifts[239]=2; // l-key       overstrike
        iChar[240]=0x00; iShifts[240]=2; // p-key       comma above right 
        iChar[241]=0x00; iShifts[241]=8; //             question mark above
        // Additional Shifts
        iChar[242]=0x99; iShifts[242]=0; // Up_SYMB => F1...F12
        iChar[243]=0x99; iShifts[243]=0; // Dn_SYMB
        iChar[244]=0x99; iShifts[244]=0; // Shift TBD
        iChar[245]=0x99; iShifts[245]=0; // WinKeys
                        
        iChar[246]=0x41; iShifts[246]=0; //gChar[246]="<AltGr OFF>"; // Key up
        iChar[247]=0x14; iShifts[247]=0; //gChar[247]="<CAPS OFF>";  // Key down & up
        iChar[248]=0x90; iShifts[248]=0; //gChar[248]="<Num Lock OFF>"; // Key down&up
        iChar[251]=0x99; iShifts[253]=9; //gChar[251]="<AKU2>"; // All keys up in UART Chord Mode       
        iChar[252]=0x91; iShifts[252]=0; //gChar[252]="<Scroll Lock OFF>"; // Key down&up       
        iChar[253]=0x99; iShifts[253]=9; //gChar[253]="<AKU>"; // All keys up in UART KeyCode Mode     


                        // gChar[254]="<SYMB OFF>"; // (SYMB OFF not used)
        iChar[254]=0x00; iShifts[254]=0; //gChar[254]="<SYMB OFF>"; // virtual
        iChar[255]=0x00; iShifts[255]=0; //gChar[255]=" <Mouse data End> ";



        // create US version of the table
        // ==============================
        
        // fist make a copy of SWE/FIN layout
        int i; i = 1;
        while (i < 256)
        {
        iShiftsUS[i]=0;		// clear all shifts first (1=SHIFT, 2=AltGr)
	iCharUS[i] = iChar[i];  // copy character codes from FIN/SWE kbd first
    gCharUS[i] = gChar[i];
        ++i;
        }
        
        // then define differencies:

        // 1 = Shift, 2 = AtGr(=Ctrl+Alt), 3 = Ctrl, 5 = Alt[xxx] (NumPad),
        // 8 = Shift+AltGr;  9 = National Shortcut
        // PC code      ; Shifts needed  ; Monitor display
        iCharUS[27]=0x00; iShiftsUS[27]=9; //gCharUS[27]="th";     //  / th
        iCharUS[28]=0x00; iShiftsUS[28]=9; //gCharUS[28]="a ";   //  / and_ > a_
        iCharUS[29]=0x00; iShiftsUS[29]=9; //gCharUS[29]="the ";   //  / the
        iCharUS[30]=0x00; iShiftsUS[30]=9; //gCharUS[30]="of ";    //  / of
        // Each character sequence above is always sent as series of letters a-z
        
        iCharUS[31]=0xbe; iShiftsUS[31]=0; //gCharUS[31]=".";
        iCharUS[32]=0xbc; iShiftsUS[32]=0; //gCharUS[32]=",";
        iCharUS[33]=0x31; iShiftsUS[33]=1; //gCharUS[33]="!"; // shift 1
        iCharUS[34]=0xbf; iShiftsUS[34]=1; //gCharUS[34]="?"; // 
        iCharUS[35]=0xbd; iShiftsUS[35]=0; //gCharUS[35]="-"; // 
        iCharUS[36]=0xde; iShiftsUS[36]=0; //gCharUS[36]="'"; // ????
        iCharUS[37]=0xe2; iShiftsUS[37]=0; //gCharUS[37]="<backslash>";  //
        iCharUS[38]=0xbf; iShiftsUS[38]=0; //gCharUS[38]="/"; // 
        iCharUS[39]=0x00; iShiftsUS[39]=9; //gCharUS[39]="and "; // ()
        iCharUS[40]=0xc0; iShiftsUS[40]=1; //gCharUS[40]="~"; // tilde, 'worm'
        iCharUS[41]=0x00; iShiftsUS[41]=9; //gCharUS[41]="to ";  //  ^
        
        iCharUS[74]=0x33; iShiftsUS[74]=1; //gCharUS[74]="#";        
        iCharUS[75]=0x00; iShiftsUS[75]=0; //gCharUS[75]="";        
        iCharUS[76]=0x37; iShiftsUS[76]=1; //gCharUS[76]="&";
        iCharUS[77]=0x32; iShiftsUS[77]=1; //gCharUS[77]="@";
        iCharUS[78]=0xbb; iShiftsUS[78]=1; //gCharUS[78]="+";
        iCharUS[79]=0x35; iShiftsUS[79]=1; //gCharUS[79]="%";
        iCharUS[80]=0xbb; iShiftsUS[80]=0; //gCharUS[80]="=";  //
        iCharUS[81]=0x59; iShiftsUS[81]=0; //gCharUS[81]="<yen>";
        iCharUS[82]=0x38; iShiftsUS[82]=1; //gCharUS[82]="*";  //
        iCharUS[83]=0x45; iShiftsUS[83]=2; //gCharUS[83]="";
        iCharUS[84]=0x34; iShiftsUS[84]=1; //gCharUS[84]="$";        
        iCharUS[85]=0x00; iShiftsUS[85]=0; //gCharUS[85]="";        
        iCharUS[86]=0x39; iShiftsUS[86]=1; //gCharUS[86]="(";
        iCharUS[87]=0xdb; iShiftsUS[87]=0; //gCharUS[87]="[";        // US: [ FIN: 
        iCharUS[88]=0xbc; iShiftsUS[88]=1; //gCharUS[88]="<"; 
        iCharUS[89]=0xdb; iShiftsUS[89]=1; //gCharUS[89]="{";
        iCharUS[90]=0x30; iShiftsUS[90]=1; //gCharUS[90]=")";
        iCharUS[91]=0xdd; iShiftsUS[91]=0; //gCharUS[91]="]";        // US: ] FIN: 
        iCharUS[92]=0xbe; iShiftsUS[92]=1; //gCharUS[92]=">";
        iCharUS[93]=0xdd; iShiftsUS[93]=1; //gCharUS[93]="}";
        iCharUS[94]=0xba; iShiftsUS[94]=1; //gCharUS[94]=":";        
        iCharUS[95]=0xba; iShiftsUS[95]=0; //gCharUS[95]=";";  //US: ;: FIN: 
        iCharUS[96]=0x00; iShiftsUS[96]=0; //gCharUS[96]="<inv.!>";
        iCharUS[97]=0x00; iShiftsUS[97]=0; //gCharUS[97]="<inv.?>";
        iCharUS[98]=0xbd; iShiftsUS[98]=1; //gCharUS[98]="_";
        iCharUS[99]=0xde; iShiftsUS[99]=1; //gCharUS[99]="<quote>"; // " (quote)
        iCharUS[100]=0xc0; iShiftsUS[100]=0; //gCharUS[100]="`";
        iCharUS[101]=0x00; iShiftsUS[101]=0; //gCharUS[101]="";
        iCharUS[102]=0xdc; iShiftsUS[102]=1; //gCharUS[102]="|";
        iCharUS[103]=0x00; iShiftsUS[103]=0; //gCharUS[103]="";
        iCharUS[104]=0x36; iShiftsUS[104]=1; //gCharUS[104]="";  // US ^ (difference)      
        iCharUS[105]=0x00; iShiftsUS[105]=0; //gCharUS[105]="<inv.^>";        



        // Additional characters and functions in Extended US qwerty 'multinational'
        // To selcet a char on a QWERTY key: 2 = AltGr, 8 = Shitf + AltGr
        iCharUS[221]=0x00; iShiftsUS[221]=0; //
        iCharUS[222]=0x00; iShiftsUS[222]=0; //
        iCharUS[223]=0x00; iShiftsUS[223]=0; //
        iCharUS[224]=0x00; iShiftsUS[224]=0; //
        iCharUS[225]=0x00; iShiftsUS[225]=0; //
        iCharUS[226]=0x00; iShiftsUS[226]=0; //
        iCharUS[227]=0x00; iShiftsUS[227]=0; //
        iCharUS[228]=0x00; iShiftsUS[228]=0; //
        iCharUS[229]=0x00; iShiftsUS[229]=0; //
        iCharUS[230]=0x00; iShiftsUS[230]=0; //
        iCharUS[231]=0x00; iShiftsUS[231]=0; //
        iCharUS[232]=0x00; iShiftsUS[232]=0; //
        iCharUS[233]=0x00; iShiftsUS[233]=0; //

        iCharUS[234]=0x00; iShiftsUS[234]=0; //
        iCharUS[235]=0x00; iShiftsUS[235]=0; //
        iCharUS[236]=0x00; iShiftsUS[236]=0; //
        iCharUS[237]=0x00; iShiftsUS[237]=0; //
        iCharUS[238]=0x00; iShiftsUS[238]=0; //
        iCharUS[239]=0x00; iShiftsUS[239]=0; //
        iCharUS[240]=0x00; iShiftsUS[240]=0; //
        iCharUS[241]=0x00; iShiftsUS[241]=0; //
        // Shifts
        iCharUS[242]=0x00; iShiftsUS[242]=0; // Up_SYMB (F1...F16 etc)
        iCharUS[243]=0x00; iShiftsUS[243]=0; // Dn_SYMB (TBD)
        iCharUS[244]=0x00; iShiftsUS[244]=0; // Shift TBD
        iCharUS[245]=0x00; iShiftsUS[245]=0; // WinKeys


        // Create Shortcut table (See GKOS Shortcuts defined at http://gkos.net)
        // <AKU> xx <AKU>, where xx = !j  for example, and <AKU> = All Keys Up condition

//---------------------------------------------------------
        
        // This table is used in chordon shortcuts:
        
        AscToGKOS[8] = 46;// backspace        
        AscToGKOS[32] = 50;// space        

        AscToGKOS[35] = 74;  // # used as a Shift in shortcuts         
        AscToGKOS[38] = 76;  // & used as a Ctrl in shortcuts (Word Right/Left)         
        AscToGKOS[42] = 82;  // * used as a Del in shortcuts         
        AscToGKOS[60] = 88;  // < used as a LeftArrow in shortcuts         
        AscToGKOS[62] = 92;  // > used as a RghtArrow in shortcuts         
        AscToGKOS[95] = 98;  // _ used as a backspace in shortcuts         
        AscToGKOS[43] = 127;  // + used as a Del Word Right in shortcuts (SpaceShift = 4)         
        AscToGKOS[37] = 127; // % used as a Del Word Left in shortcuts (SpaceShift = 0)         
        AscToGKOS[40] = 86; // ( used as a Scroll Up in shortcuts         
        AscToGKOS[41] = 90; // ) used as a Scroll Down in shortcuts         
        AscToGKOS[47] = 38;//  / Used in SC for: Send nothing, Set g123abc TRUE         

        AscToGKOS[58] = 94;// : Used in SC for: Send Nothing, set XShifts = 2 (AltGr)
        AscToGKOS[59] = 95;// ; Used in SC for: Send Nothing, set XShifts = 8 (Shift+AltGr)
        AscToGKOS[61] = 80;// =  Just send nohing.      

        AscToGKOS[33] = 33;// !         
        AscToGKOS[39] = 36;//'         
        AscToGKOS[44] = 32;// ,         
        AscToGKOS[45] = 35;//-         
        AscToGKOS[46] = 31;// .         
        AscToGKOS[63] = 34;// ?         

        AscToGKOS[48] = 70;// 0       
        AscToGKOS[49] = 64;// 1       
        AscToGKOS[50] = 65;// 2        
        AscToGKOS[51] = 66;// 3        
        AscToGKOS[52] = 67;// 4        
        AscToGKOS[53] = 68;// 5        
        AscToGKOS[54] = 69;// 6        
        AscToGKOS[55] = 71;// 7        
        AscToGKOS[56] = 72;// 8        
        AscToGKOS[57] = 73;// 9        

        AscToGKOS[91] = 87; //  [  o or   depends on Fin grammar 
        AscToGKOS[93] = 91; //  ]  a or   depends on Fin grammar

        AscToGKOS[97] = 1;//a         
        AscToGKOS[98] = 2;//         
        AscToGKOS[99] = 3;//         
        AscToGKOS[100] = 4;//         
        AscToGKOS[101] = 5;//         
        AscToGKOS[102] = 6;//         
        AscToGKOS[103] = 7;//         
        AscToGKOS[104] = 8;//         
        AscToGKOS[105] = 9;//         
        AscToGKOS[106] = 10;//         
        AscToGKOS[107] = 11;//         
        AscToGKOS[108] = 12;//         
        AscToGKOS[109] = 13;//         
        AscToGKOS[110] = 14;//         
        AscToGKOS[111] = 15;//         
        AscToGKOS[112] = 16;//         
        AscToGKOS[113] = 17;//         
        AscToGKOS[114] = 18;//         
        AscToGKOS[115] = 19;//         
        AscToGKOS[116] = 20;//         
        AscToGKOS[117] = 21;//         
        AscToGKOS[118] = 22;//         
        AscToGKOS[119] = 23;//         
        AscToGKOS[120] = 24;//         
        AscToGKOS[121] = 25;//         
        AscToGKOS[122] = 26;//         
        AscToGKOS[96] = 27; // ` => u (umlaut)         
        AscToGKOS[123] = 28;// { =>     
        AscToGKOS[124] = 28;// ('|' cannot be used, it is a separator in .ini file)        
        AscToGKOS[125] = 29;// } =>         
        AscToGKOS[126] = 30;// ~ =>       
        AscToGKOS[127] = 57;//         
        //AscToGKOS[] = ;//         
        
        
//; ASCII CODES:
//;000  (nul)  016  (dle)  032 sp  048 0  064 @  080 P  096 `  112 p
//;001  (soh)  017  (dc1)  033 !   049 1  065 A  081 Q  097 a  113 q
//;002  (stx)  018  (dc2)  034 "   050 2  066 B  082 R  098 b  114 r
//;003  (etx)  019  (dc3)  035 #   051 3  067 C  083 S  099 c  115 s
//;004  (eot)  020  (dc4)  036 $   052 4  068 D  084 T  100 d  116 t
//;005  (enq)  021  (nak)  037 %   053 5  069 E  085 U  101 e  117 u
//;006  (ack)  022  (syn)  038 &   054 6  070 F  086 V  102 f  118 v
//;007  (bel)  023  (etb)  039 '   055 7  071 G  087 W  103 g  119 w
//;008  (bs)   024  (can)  040 (   056 8  072 H  088 X  104 h  120 x
//;009  (tab)  025  (em)   041 )   057 9  073 I  089 Y  105 i  121 y
//;010  (lf)   026  (eof)  042 *   058 :  074 J  090 Z  106 j  122 z
//;011  (vt)   027  (esc)  043 +   059 ;  075 K  091 [  107 k  123 {
//;012  (np)   028  (fs)   044 ,   060 <  076 L  092    108 l  124 |
//;013  (cr)   029  (gs)   045 -   061 =  077 M  093 ]  109 m  125 }
//;014  (so)   030  (rs)   046 .   062 >  078 N  094 ^  110 n  126 ~
//;015  (si)   031  (us)   047 /   063 ?  079 O  095 _  111 o  127 (del)


// ================CHORDON SHORTCUTS:===========================

// --- These special characters can be usen in shortcut strings: ---
//        AscToGKOS[35] = 74;//  # used as a Shift in shortcuts         
//        AscToGKOS[38] = 76;//  & used as a Ctrl in shortcuts (Word Right/Left)         
//        AscToGKOS[60] = 88;//  < used as a LeftArrow in shortcuts         
//        AscToGKOS[62] = 92;//  > used as a RghtArrow in shortcuts         
//        AscToGKOS[95] = 98;//  _ used as a backspace in shortcuts         
//        AscToGKOS[42] = 82;//  * used as a Del in shortcuts         
//        AscToGKOS[37] = 127;// % used as a Del Word in shortcuts         

// All 781 GKOS Chordon Shortcuts are listed here
// - first those with a special character first (or both)
// - then those with the special character last (only)
//
//
// They have been carefully selected by a skilful GKOS typist
// in order to maintain these priciples:
//
// 1) Shortcuts for the most frequent and shortest words are the easiest ones;
//    otherwise there would be no benefit
//
// 2) Shortcuts must feel as much as possible the same as typing the beginning
//    of the word it represents - so they can even be guessed right!
//
// 3) As a consequence, special characters replace certain letters:
//     Example 1: ! can replace T because they have 2 common keys
//     Example 2: - can replace H for the same reason
//     Example 3: - can also replace A as the only key for A is common
//
// 4) The table should be kept the same in all applications but a certain
//    range could be reserved for user-defined words 
//
// 5) There can be a table of 781 words for each language

        sc1 = 0; Lc = 2; Le = 32; Lf = 32; // Period         .a
        //strncpy(Chrd[tempint], ".a", Lc); strncpy(SCen[tempint], "qwertyui", Le); strncpy(SCen[tempint], "asdfghjk", Lf);

        strncpy(Chrd[1+sc1], ".a", Lc); strncpy(SCen[1+sc1], "--1--", Le); strncpy(SCfi[1+sc1], "mihin aikaan ", Lf); // 'a
        strncpy(Chrd[2+sc1], ".b", Lc); strncpy(SCen[2+sc1], "yourself ", Le); strncpy(SCfi[2+sc1], "eiv}t ole ", Lf); // 'b
        strncpy(Chrd[3+sc1], ".c", Lc); strncpy(SCen[3+sc1], "--3--- ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(Chrd[4+sc1], ".d", Lc); strncpy(SCen[4+sc1], "--4--", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'de
        strncpy(Chrd[5+sc1], ".e", Lc); strncpy(SCen[5+sc1], "maybe ", Le); strncpy(SCfi[5+sc1], "melko ", Lf); // 'e
        strncpy(Chrd[6+sc1], ".f", Lc); strncpy(SCen[6+sc1], "sure ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(Chrd[7+sc1], ".g", Lc); strncpy(SCen[7+sc1], "--7--", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(Chrd[8+sc1], ".h", Lc); strncpy(SCen[8+sc1], "--8--", Le); strncpy(SCfi[8+sc1], "mihin ", Lf); // 'h
        strncpy(Chrd[9+sc1], ".i", Lc); strncpy(SCen[9+sc1], "first ", Le); strncpy(SCfi[9+sc1], "mit} ", Lf); // 'i
        strncpy(Chrd[10+sc1], ".j", Lc); strncpy(SCen[10+sc1], "-10--", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(Chrd[11+sc1], ".k", Lc); strncpy(SCen[11+sc1], "back ", Le); strncpy(SCfi[11+sc1], "miksi ", Lf); // 'k
        strncpy(Chrd[12+sc1], ".l", Lc); strncpy(SCen[12+sc1], "full ", Le); strncpy(SCfi[12+sc1], "mink}lainen ", Lf); // 'l
        strncpy(Chrd[13+sc1], ".m", Lc); strncpy(SCen[13+sc1], "many ", Le); strncpy(SCfi[13+sc1], "mik} ", Lf); // 'm
        strncpy(Chrd[14+sc1], ".n", Lc); strncpy(SCen[14+sc1], "on ", Le); strncpy(SCfi[14+sc1], "on ", Lf); // 'n -fin- minne?
        strncpy(Chrd[15+sc1], ".o", Lc); strncpy(SCen[15+sc1], "for ", Le); strncpy(SCfi[15+sc1], "voi ", Lf); // 'o
        strncpy(Chrd[16+sc1], ".p", Lc); strncpy(SCen[16+sc1], "important ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(Chrd[17+sc1], ".q", Lc); strncpy(SCen[17+sc1], "from ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q temp
        strncpy(Chrd[18+sc1], ".r", Lc); strncpy(SCen[18+sc1], "or ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(Chrd[19+sc1], ".s", Lc); strncpy(SCen[19+sc1], "means ", Le); strncpy(SCfi[19+sc1], "miss} ", Lf); // 's
        strncpy(Chrd[20+sc1], ".t", Lc); strncpy(SCen[20+sc1], "tried ", Le); strncpy(SCfi[20+sc1], "miten ", Lf); // 't
        strncpy(Chrd[21+sc1], ".u", Lc); strncpy(SCen[21+sc1], "must ", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(Chrd[22+sc1], ".v", Lc); strncpy(SCen[22+sc1], "five ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(Chrd[23+sc1], ".w", Lc); strncpy(SCen[23+sc1], "following ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(Chrd[24+sc1], ".x", Lc); strncpy(SCen[24+sc1], "wrote ", Le); strncpy(SCfi[24+sc1], "miksik~h}n ", Lf); // 'x
        strncpy(Chrd[25+sc1], ".y", Lc); strncpy(SCen[25+sc1], "boy ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(Chrd[26+sc1], ".z", Lc); strncpy(SCen[26+sc1], "find ", Le); strncpy(SCfi[26+sc1], "mit}}n ", Lf); // 'z optio
        strncpy(Chrd[27+sc1], ".u", Lc); strncpy(SCen[27+sc1], "brother ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(Chrd[28+sc1], ".", Lc); strncpy(SCen[28+sc1], "-28--", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(Chrd[29+sc1], ".", Lc); strncpy(SCen[29+sc1], "-29--", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(Chrd[30+sc1], ".", Lc); strncpy(SCen[30+sc1], "-30--", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(Chrd[31+sc1], "..", Lc); strncpy(SCen[31+sc1], "-31--", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '.
        strncpy(Chrd[32+sc1], ".,", Lc); strncpy(SCen[32+sc1], "message ", Le); strncpy(SCfi[32+sc1], "miksi ", Lf); // ',
        strncpy(Chrd[33+sc1], ".!", Lc); strncpy(SCen[33+sc1], "friend ", Le); strncpy(SCfi[33+sc1], "mitenk}}n ", Lf); // '!
        strncpy(Chrd[34+sc1], ".?", Lc); strncpy(SCen[34+sc1], "for ", Le); strncpy(SCfi[34+sc1], "milloin ", Lf); // '? // option
        strncpy(Chrd[35+sc1], ".-", Lc); strncpy(SCen[35+sc1], "you'll ", Le); strncpy(SCfi[35+sc1], "mill} ", Lf); // '-
        strncpy(Chrd[36+sc1], ".'", Lc); strncpy(SCen[36+sc1], "yes ", Le); strncpy(SCfi[36+sc1], "miten ", Lf); // ''
        strncpy(Chrd[37+sc1], ".U", Lc); strncpy(SCen[37+sc1], "yeah ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(Chrd[38+sc1], ".D", Lc); strncpy(SCen[38+sc1], "mean ", Le); strncpy(SCfi[38+sc1], "minne ", Lf); // 'D
        strncpy(Chrd[39+sc1], ".B", Lc); strncpy(SCen[39+sc1], "-39--", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(Chrd[40+sc1], ".F", Lc); strncpy(SCen[40+sc1], ". #", Le); strncpy(SCfi[40+sc1], ". #", Lf); // 'F // Period + Space + SHIFT
        strncpy(Chrd[41+sc1], ".S", Lc); strncpy(SCen[41+sc1], "much ", Le); strncpy(SCfi[41+sc1], "mit} ", Lf); // 'S

        sc1 = 41; // Comma         ,a
        strncpy(SCen[1+sc1], "-42--", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "ourselves ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "ever ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "-45--", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "never ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "-47--", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "gave ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "check ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "sister ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "just ", Le); strncpy(SCfi[10+sc1], "jo ", Lf); // 'j
        strncpy(SCen[11+sc1], "-52--", Le); strncpy(SCfi[11+sc1], "niink~ ", Lf); // 'k
        strncpy(SCen[12+sc1], "clear ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "most ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "not ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "-56--", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o
        strncpy(SCen[16+sc1], "-57--", Le); strncpy(SCfi[16+sc1], "niinp} ", Lf); // 'p
        strncpy(SCen[17+sc1], "come ", Le); strncpy(SCfi[17+sc1], "noin ", Lf); // 'q
        strncpy(SCen[18+sc1], "-59--", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "ever ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's Optional
        strncpy(SCen[20+sc1], "until ", Le); strncpy(SCfi[20+sc1], "ett} ", Lf); // 't
        strncpy(SCen[21+sc1], "see ", Le); strncpy(SCfi[21+sc1], "kuka ", Lf); // 'u
        strncpy(SCen[22+sc1], "everybody ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "saw ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "six ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "-66--", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "end ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "thinks ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-69--", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "these ", Le); strncpy(SCfi[29+sc1], "en}} ", Lf); // ' optio
        strncpy(SCen[30+sc1], "enough ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "everyone ", Le); strncpy(SCfi[31+sc1], "n}m} ", Lf); // '.
        strncpy(SCen[32+sc1], "-73--", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "everything ", Le); strncpy(SCfi[33+sc1], "_, ett} ", Lf); // '!
        strncpy(SCen[34+sc1], "-75--", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "came ", Le); strncpy(SCfi[35+sc1], "n}in ", Lf); // '-
        strncpy(SCen[36+sc1], "set ", Le); strncpy(SCfi[36+sc1], "kuinka ", Lf); // ''
        strncpy(SCen[37+sc1], "show ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "no ", Le); strncpy(SCfi[38+sc1], "on ", Lf); // 'D
        strncpy(SCen[39+sc1], "-80--", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], ", ", Le); strncpy(SCfi[40+sc1], ", ", Lf); // 'F // reserved
        strncpy(SCen[41+sc1], "sure ", Le); strncpy(SCfi[41+sc1], "niin ", Lf); // 'S
        
        // fin: u    = { | } ~
        sc1 = 82; // Exclamation   !a
        strncpy(SCen[1+sc1], "-83--", Le); strncpy(SCfi[1+sc1], "tarvitse ", Lf); // 'a
        strncpy(SCen[2+sc1], "themselves ", Le); strncpy(SCfi[2+sc1], "- fin - ", Lf); // 'b
        strncpy(SCen[3+sc1], "couldn't ", Le); strncpy(SCfi[3+sc1], "- fin - ", Lf); // 'c
        strncpy(SCen[4+sc1], "doesn't ", Le); strncpy(SCfi[4+sc1], "- fin - ", Lf); // 'd
        strncpy(SCen[5+sc1], "-87--", Le); strncpy(SCfi[5+sc1], "tehd} ", Lf); // 'e
        strncpy(SCen[6+sc1], "-88--", Le); strncpy(SCfi[6+sc1], "- fin - ", Lf); // 'f
        strncpy(SCen[7+sc1], "though ", Le); strncpy(SCfi[7+sc1], "- fin - ", Lf); // 'g
        strncpy(SCen[8+sc1], "thought ", Le); strncpy(SCfi[8+sc1], "tahansa ", Lf); // 'h
        strncpy(SCen[9+sc1], "till ", Le); strncpy(SCfi[9+sc1], "- fin - ", Lf); // 'i
        strncpy(SCen[10+sc1], "just ", Le); strncpy(SCfi[10+sc1], "ja ", Lf); // 'j optional
        strncpy(SCen[11+sc1], "take ", Le); strncpy(SCfi[11+sc1], "- fin - ", Lf); // 'k
        strncpy(SCen[12+sc1], "-94--", Le); strncpy(SCfi[12+sc1], "- fin - ", Lf); // 'l
        strncpy(SCen[13+sc1], "-95--", Le); strncpy(SCfi[13+sc1], "- fin - ", Lf); // 'm
        strncpy(SCen[14+sc1], "take ", Le); strncpy(SCfi[14+sc1], "- fin - ", Lf); // 'n
        strncpy(SCen[15+sc1], "too ", Le); strncpy(SCfi[15+sc1], "jos ", Lf); // 'o optio
        strncpy(SCen[16+sc1], "top ", Le); strncpy(SCfi[16+sc1], "- fin - ", Lf); // 'p
        strncpy(SCen[17+sc1], "-99--", Le); strncpy(SCfi[17+sc1], "- fin - ", Lf); // 'q
        strncpy(SCen[18+sc1], "draught ", Le); strncpy(SCfi[18+sc1], "- fin - ", Lf); // 'r
        strncpy(SCen[19+sc1], "just ", Le); strncpy(SCfi[19+sc1], "t}ss} ", Lf); // 's optional
        strncpy(SCen[20+sc1], "still ", Le); strncpy(SCfi[20+sc1], "- fin - ", Lf); // 't
        strncpy(SCen[21+sc1], "through ", Le); strncpy(SCfi[21+sc1], "tuo ", Lf); // 'u
        strncpy(SCen[22+sc1], "television ", Le); strncpy(SCfi[22+sc1], "televisio ", Lf); // 'v
        strncpy(SCen[23+sc1], "two ", Le); strncpy(SCfi[23+sc1], "- fin - ", Lf); // 'w
        strncpy(SCen[24+sc1], "text ", Le); strncpy(SCfi[24+sc1], "- fin - ", Lf); // 'x
        strncpy(SCen[25+sc1], "type ", Le); strncpy(SCfi[25+sc1], "ty~ ", Lf); // 'y temp
        strncpy(SCen[26+sc1], "twelve ", Le); strncpy(SCfi[26+sc1], "- fin - ", Lf); // 'z
        strncpy(SCen[27+sc1], "those ", Le); strncpy(SCfi[27+sc1], "t] ", Lf); // 'u
        strncpy(SCen[28+sc1], "too ", Le); strncpy(SCfi[28+sc1], "t]] ", Lf); // '
        strncpy(SCen[29+sc1], "there ", Le); strncpy(SCfi[29+sc1], "t}m} ", Lf); // '
        strncpy(SCen[30+sc1], "down ", Le); strncpy(SCfi[30+sc1], "- fin - ", Lf); // '
        strncpy(SCen[31+sc1], "-113-", Le); strncpy(SCfi[31+sc1], "arvaa ", Lf); // '.
        strncpy(SCen[32+sc1], "tell ", Le); strncpy(SCfi[32+sc1], "t}ss} ", Lf); // ',
        strncpy(SCen[33+sc1], "-115-", Le); strncpy(SCfi[33+sc1], "- fin - ", Lf); // '!
        strncpy(SCen[34+sc1], "there are ", Le); strncpy(SCfi[34+sc1], "t}}ll} on ", Lf); // '?
        strncpy(SCen[35+sc1], "there is ", Le); strncpy(SCfi[35+sc1], "t}}ll} ei ole ", Lf); // '- 
        strncpy(SCen[36+sc1], "it's ", Le); strncpy(SCfi[36+sc1], "tied} ", Lf); // ''
        strncpy(SCen[37+sc1], "taught ", Le); strncpy(SCfi[37+sc1], "tai ", Lf); // 'U
        strncpy(SCen[38+sc1], "done ", Le); strncpy(SCfi[38+sc1], "t}m}n ", Lf); // 'D
        strncpy(SCen[39+sc1], "-121-", Le); strncpy(SCfi[39+sc1], "- fin - ", Lf); // 'B
        strncpy(SCen[40+sc1], "! #", Le); strncpy(SCfi[40+sc1], "! #", Lf); // 'F // Excl.+Space + SHIFT
        strncpy(SCen[41+sc1], "tomorrow ", Le); strncpy(SCfi[41+sc1], "tulla ", Lf); // 'S
    

        sc1 = 123; // Question     ?a
        strncpy(SCen[1+sc1], "last ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "far ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "lack ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "looked ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "left ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "after ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "large ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "hers ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "life ", Le); strncpy(SCfi[9+sc1], "kai ", Lf); // 'i
        strncpy(SCen[10+sc1], "-133-", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "look ", Le); strncpy(SCfi[11+sc1], "alkaa ", Lf); // 'k
        strncpy(SCen[12+sc1], "like ", Le); strncpy(SCfi[12+sc1], "olla ", Lf); // 'l
        strncpy(SCen[13+sc1], "am ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "answer ", Le); strncpy(SCfi[14+sc1], "k]]n ", Lf); // 'n -kaan/kn
        strncpy(SCen[15+sc1], "love ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o
        strncpy(SCen[16+sc1], "really ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "requirement ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "are ", Le); strncpy(SCfi[18+sc1], "on ", Lf); // 'r
        strncpy(SCen[19+sc1], "last ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "fast ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "laugh ", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "over ", Le); strncpy(SCfi[22+sc1], "kovin ", Lf); // 'v
        strncpy(SCen[23+sc1], "which ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "what ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "away ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "found ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "father ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-151-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-152-", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "often ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "from ", Le); strncpy(SCfi[31+sc1], "arvaa ", Lf); // '.
        strncpy(SCen[32+sc1], "-155-", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "are there ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "-157-", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "already ", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "live ", Le); strncpy(SCfi[36+sc1], "-fin- ", Lf); // ''
        strncpy(SCen[37+sc1], "whome ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "another ", Le); strncpy(SCfi[38+sc1], "olen ", Lf); // 'D optio
        strncpy(SCen[39+sc1], "alias ", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "? #", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F // Quest.+ Space + SHIFT
        strncpy(SCen[41+sc1], "remember ", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S
      

        sc1 = 164; // Hyphen       -a
        strncpy(SCen[1+sc1], "easy ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "herself ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "children ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "-168-", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "easily ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "felt ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f optional
        strncpy(SCen[7+sc1], "again ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "has ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "in ", Le); strncpy(SCfi[9+sc1], "aika ", Lf); // 'i
        strncpy(SCen[10+sc1], "-174-", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "hear ", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "help ", Le); strncpy(SCfi[12+sc1], "joka ", Lf); // 'l
        strncpy(SCen[13+sc1], "made ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "land ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "home ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o
        strncpy(SCen[16+sc1], "happen ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "able ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q  //
        strncpy(SCen[18+sc1], "alright ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "house ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "easy ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't  // optional
        strncpy(SCen[21+sc1], "else ", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "-186-", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "always ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "example ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "happy ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-190-", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "these ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-192-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "they ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "-194-", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "ly ", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '.
        strncpy(SCen[32+sc1], "their ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "is there ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "her ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "-199-", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "late ", Le); strncpy(SCfi[36+sc1], "h}t} ", Lf); // ''
        strncpy(SCen[37+sc1], "have ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "language ", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "also known as ", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "ly ", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F
        strncpy(SCen[41+sc1], "am ", Le); strncpy(SCfi[41+sc1], "tuo ", Lf); // 'S // option
        

        sc1 = 205; // Apostrophe   'a
        strncpy(SCen[1+sc1], "---- ", Le); strncpy(SCfi[1+sc1], "taitaa ", Lf); // 'a -fin- paha, paljon?
        strncpy(SCen[2+sc1], "done ", Le); strncpy(SCfi[2+sc1], "et ole ", Lf); // 'b
        strncpy(SCen[3+sc1], "country ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "didn't ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "-210-", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "if ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f // optionl
        strncpy(SCen[7+sc1], "together ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "ght ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "is ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-215-", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "took ", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "play ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "took ", Le); strncpy(SCfi[13+sc1], "mutta ", Lf); // 'm
        strncpy(SCen[14+sc1], "-219-", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "do ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o // which is better of these two?
        strncpy(SCen[16+sc1], "do ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "point ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "part ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "'s ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "it ", Le); strncpy(SCfi[20+sc1], "totta ", Lf); // 't
        strncpy(SCen[21+sc1], "true ", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "you've ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "without ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "exist ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "you ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-231-", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "-222-", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-233-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "other ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // ' //optional
        strncpy(SCen[30+sc1], "-235-", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "your ", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '.
        strncpy(SCen[32+sc1], "yourselves ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "its ", Le); strncpy(SCfi[33+sc1], "itse ", Lf); // '! -fin- itse
        strncpy(SCen[34+sc1], "true ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "ph", Le); strncpy(SCfi[35+sc1], "t}t} ", Lf); // '-  -fin- tt
        strncpy(SCen[36+sc1], "-----", Le); strncpy(SCfi[36+sc1], "-fin- ", Lf); // ''
        strncpy(SCen[37+sc1], "yesterday ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "information ", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "-244- ", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "tion ", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F
        strncpy(SCen[41+sc1], "better ", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S
        

        sc1 = 246; // UpArrow = U    Ua
        strncpy(SCen[1+sc1], "way ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "himself ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "changed ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "don't ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "perhaps ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "-----", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'fc
        strncpy(SCen[7+sc1], "hadn't ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "he ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "ago ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "-----", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "place ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "-----", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "-----", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "how ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o
        strncpy(SCen[16+sc1], "people ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "home ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q // option
        strncpy(SCen[18+sc1], "hour ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "-----", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "hate ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "hug ", Le); strncpy(SCfi[21+sc1], "huono ", Lf); // 'u
        strncpy(SCen[22+sc1], "-----", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "own ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "who ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "day ", Le); strncpy(SCfi[25+sc1], "hyvin ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "-----", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "an ", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-----", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "at last ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "around ", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '. // optional
        strncpy(SCen[32+sc1], "however ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "at ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "whose ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "his ", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "about ", Le); strncpy(SCfi[36+sc1], "-fin- ", Lf); // ''
        strncpy(SCen[37+sc1], "-----", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "at least ", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "possible ", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "ation ", Le); strncpy(SCfi[40+sc1], "]] ", Lf); // 'F  -aa/
        strncpy(SCen[41+sc1], "ours ", Le); strncpy(SCfi[41+sc1], ".fin ", Lf); // 'S
        

        sc1 = 287; // DownArrow = D    Da
        strncpy(SCen[1+sc1], "-288-", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "myself ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "can't ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "needed ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "need ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "cannot ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "-----", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "-----", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "-----", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "felt ", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "call ", Le); strncpy(SCfi[12+sc1], "valmis ", Lf); // 'l
        strncpy(SCen[13+sc1], "name ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "no one ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "now ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o optional
        strncpy(SCen[16+sc1], "-----", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "-----", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "no one ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "news ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "-----", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "number ", Le); strncpy(SCfi[21+sc1], "nuo ", Lf); // 'u
        strncpy(SCen[22+sc1], "nobody ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "new ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "next ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "nyt ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y -fin- nyt (optio)
        strncpy(SCen[26+sc1], "nothing ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "things ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-----", Le); strncpy(SCfi[29+sc1], "n}m} ", Lf); // '
        strncpy(SCen[30+sc1], "everything ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "now ", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '. // option
        strncpy(SCen[32+sc1], "feel ", Le); strncpy(SCfi[32+sc1], "ne ", Lf); // ',
        strncpy(SCen[33+sc1], "called ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "soon ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "-----", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "-----", Le); strncpy(SCfi[36+sc1], "-fin- ", Lf); // ''
        strncpy(SCen[37+sc1], "-----", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "-----", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "-----", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "sition ", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F
        strncpy(SCen[41+sc1], "-----", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S
        

        sc1 = 328; // BackSpace = B      Ba
        strncpy(SCen[1+sc1], "_a ", Le); strncpy(SCfi[1+sc1], "_aa ", Lf); // 'a
        strncpy(SCen[2+sc1], "_er ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "_est ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "_d ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "_e ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "-----", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "-----", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "-----", Le); strncpy(SCfi[8+sc1], "_h]n ", Lf); // 'h -han/hn
        strncpy(SCen[9+sc1], "-----", Le); strncpy(SCfi[9+sc1], "_isi ", Lf); // 'i
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "-----", Le); strncpy(SCfi[11+sc1], "_kin ", Lf); // 'k
        strncpy(SCen[12+sc1], "-----", Le); strncpy(SCfi[12+sc1], "_ll] ", Lf); // 'l -fin- _lla/ll
        strncpy(SCen[13+sc1], "-----", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "_ing ", Le); strncpy(SCfi[14+sc1], "_n ", Lf); // 'n // optional
        strncpy(SCen[15+sc1], "_o ", Le); strncpy(SCfi[15+sc1], "__oa ", Lf); // 'o
        strncpy(SCen[16+sc1], "-----", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "-----", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "-----", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "_s ", Le); strncpy(SCfi[19+sc1], "_ss] ", Lf); // 's -fin- _ssa/ss
        strncpy(SCen[20+sc1], "-----", Le); strncpy(SCfi[20+sc1], "_st] ", Lf); // 't -fin- _sta/st
        strncpy(SCen[21+sc1], "-----", Le); strncpy(SCfi[21+sc1], "__ua ", Lf); // 'u
        strncpy(SCen[22+sc1], "-----", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "-----", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "-----", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "-----", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "_th ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-----", Le); strncpy(SCfi[29+sc1], "_]] ", Lf); // '
        strncpy(SCen[30+sc1], "This is an example of a very long string! ", Le); strncpy(SCfi[30+sc1], "T}m} on esimekki oikein pitk}st} lauseesta. ", Lf); // '
        strncpy(SCen[31+sc1], "_. #", Le);  strncpy(SCfi[31+sc1], "_. #", Lf);// '. // Automatic SHIFT
        strncpy(SCen[32+sc1], "_, ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "_! #", Le); strncpy(SCfi[33+sc1], "_! #", Lf); // '! // Automatic SHIFT
        strncpy(SCen[34+sc1], "_? #", Le); strncpy(SCfi[34+sc1], "_? #", Lf); // '? // Automatic SHIFT
        strncpy(SCen[35+sc1], "_ly ", Le); strncpy(SCfi[35+sc1], "_lle ", Lf); // '- -fin- lle
        strncpy(SCen[36+sc1], "_'s ", Le); strncpy(SCfi[36+sc1], "_lt] ", Lf); // '' _lta/lt
        strncpy(SCen[37+sc1], "_ed ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "_ing ", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "-367-", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "_ ", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F
        strncpy(SCen[41+sc1], "_ed ", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S
        

        sc1 = 369; // Space = F (Forward)     Fa
        strncpy(SCen[1+sc1], " a", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a // reserved
        strncpy(SCen[2+sc1], " b", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b // reserved
        strncpy(SCen[3+sc1], " c", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c // reserved
        strncpy(SCen[4+sc1], "-373-", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "-374-", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "-375-", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "-376-", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "-377-", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "-378-", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-379-", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "-380-", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "-381-", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "-382-", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "-383-", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], " o", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o // reserved
        strncpy(SCen[16+sc1], "-385-", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "-386-", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "-387-", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], " s", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's // reserved
        strncpy(SCen[20+sc1], "-389-", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "-39+-", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "-391-", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "-392-", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "-393-", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "-394-", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-395-", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], " th", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u // reserved
        strncpy(SCen[28+sc1], "-397-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-398-", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "-399-", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "word ", Le); strncpy(SCfi[31+sc1], "-fin- ", Lf); // '.
        strncpy(SCen[32+sc1], "words ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "world ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "work ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '?
        strncpy(SCen[35+sc1], "whole ", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "wish ", Le); strncpy(SCfi[36+sc1], "-fin- ", Lf); // ''
        strncpy(SCen[37+sc1], "-406-", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U
        strncpy(SCen[38+sc1], "went ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "-408-", Le); strncpy(SCfi[39+sc1], "-fin- ", Lf); // 'B
        strncpy(SCen[40+sc1], "  ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'F // reserved
        strncpy(SCen[41+sc1], " #", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S = Space + Shift (but faster)
        

        sc1 = 410; // Shift = S              Sa
        strncpy(SCen[1+sc1], "beautiful ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "between ", Le); strncpy(SCfi[2+sc1], "ei ole ", Lf); // 'b
        strncpy(SCen[3+sc1], "intercontinental ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "-----", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "been ", Le); strncpy(SCfi[5+sc1], "eik~ ", Lf); // 'e
        strncpy(SCen[6+sc1], "-----", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "might ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g // optional
        strncpy(SCen[8+sc1], "ight ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "#i ", Le); strncpy(SCfi[9+sc1], "ei ", Lf); // 'i  optio
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "mik} ", Le); strncpy(SCfi[11+sc1], "sek} ", Lf); // 'k -fin- mik
        strncpy(SCen[12+sc1], "black ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "me ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "ing ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "more ", Le); strncpy(SCfi[15+sc1], "ei ole ", Lf); // 'o
        strncpy(SCen[16+sc1], "up ", Le); strncpy(SCfi[16+sc1], "eip} ", Lf); // 'p
        strncpy(SCen[17+sc1], "intelligent ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q  // for fun
        strncpy(SCen[18+sc1], "before ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "use ", Le); strncpy(SCfi[19+sc1], "esimerkiksi ", Lf); // 's
        strncpy(SCen[20+sc1], "but ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "us ", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "#i've ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "below ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w // optional
        strncpy(SCen[24+sc1], "expect ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "by ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "mother ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "because ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "animal ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        strncpy(SCen[31+sc1], "my ", Le); strncpy(SCfi[31+sc1], "monta ", Lf); // '.
        strncpy(SCen[32+sc1], "our ", Le); strncpy(SCfi[32+sc1], "-fin- ", Lf); // ',
        strncpy(SCen[33+sc1], "best ", Le); strncpy(SCfi[33+sc1], "-fin- ", Lf); // '!
        strncpy(SCen[34+sc1], "international ", Le); strncpy(SCfi[34+sc1], "-fin- ", Lf); // '? temp
        strncpy(SCen[35+sc1], "back ", Le); strncpy(SCfi[35+sc1], "-fin- ", Lf); // '-
        strncpy(SCen[36+sc1], "ed ", Le); strncpy(SCfi[36+sc1], "et ", Lf); // '' -fin- et
        strncpy(SCen[37+sc1], "over ", Le); strncpy(SCfi[37+sc1], "-fin- ", Lf); // 'U optional
        strncpy(SCen[38+sc1], "under ", Le); strncpy(SCfi[38+sc1], "-fin- ", Lf); // 'D
        strncpy(SCen[39+sc1], "%", Le); strncpy(SCfi[39+sc1], "%", Lf); // 'B  Delete Word Left
        strncpy(SCen[40+sc1], "ition ", Le); strncpy(SCfi[40+sc1], "-fin- ", Lf); // 'F
        strncpy(SCen[41+sc1], "-451-", Le); strncpy(SCfi[41+sc1], "-fin- ", Lf); // 'S
        
        // ========just the SECOND CHAR is special===========
        
        sc1 = 451; // Period             a.
        strncpy(SCen[1+sc1], "around ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "beautiful ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "company ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "difficult ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "earth ", Le); strncpy(SCfi[5+sc1], "emme ", Lf); // 'e
        strncpy(SCen[6+sc1], "further ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "-----", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "hard ", Le); strncpy(SCfi[8+sc1], "hyv}} y~t} ", Lf); // 'h
        strncpy(SCen[9+sc1], "if ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "felt ", Le); strncpy(SCfi[11+sc1], "kyll} ", Lf); // 'k
        strncpy(SCen[12+sc1], "long ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "more ", Le); strncpy(SCfi[13+sc1], "my~s ", Lf); // 'm
        strncpy(SCen[14+sc1], "never ", Le); strncpy(SCfi[14+sc1], "nyt ", Lf); // 'n
        strncpy(SCen[15+sc1], "only ", Le); strncpy(SCfi[15+sc1], "on ", Lf); // 'o
        strncpy(SCen[16+sc1], "problem ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "over ", Le); strncpy(SCfi[17+sc1], "oli ", Lf); // 'q // option
        strncpy(SCen[18+sc1], "aren't ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "same ", Le); strncpy(SCfi[19+sc1], "sy~m}}n ", Lf); // 's
        strncpy(SCen[20+sc1], "try ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "usually ", Le); strncpy(SCfi[21+sc1], "siksi ", Lf); // 'u
        strncpy(SCen[22+sc1], "shall not ", Le); strncpy(SCfi[22+sc1], "voi ", Lf); // 'v
        strncpy(SCen[23+sc1], "were ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "write ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "weren't ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "things ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "they ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // ' // optional
        strncpy(SCen[30+sc1], "anyone ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '

        sc1 = 481; // Comma              a,
        strncpy(SCen[1+sc1], "ask ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "best ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "very ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "during ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "every ", Le); strncpy(SCfi[5+sc1], "en ", Lf); // 'e
        strncpy(SCen[6+sc1], "found ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "give ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "himself ", Le); strncpy(SCfi[8+sc1], "h}nen ", Lf); // 'h
        strncpy(SCen[9+sc1], "into ", Le); strncpy(SCfi[9+sc1], "isi ", Lf); // 'i
        strncpy(SCen[10+sc1], "told ", Le); strncpy(SCfi[10+sc1], "j}lkeen ", Lf); // 'j
        strncpy(SCen[11+sc1], "kiss ", Le); strncpy(SCfi[11+sc1], "kuka ", Lf); // 'k
        strncpy(SCen[12+sc1], "also ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "must ", Le); strncpy(SCfi[13+sc1], "minun ", Lf); // 'm
        strncpy(SCen[14+sc1], "knew ", Le); strncpy(SCfi[14+sc1], "nukkumaan ", Lf); // 'n
        strncpy(SCen[15+sc1], "once ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o // optional
        strncpy(SCen[16+sc1], "person ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p  
        strncpy(SCen[17+sc1], "quick ", Le); strncpy(SCfi[17+sc1], "ollut ", Lf); // 'q
        strncpy(SCen[18+sc1], "once ", Le); strncpy(SCfi[18+sc1], "ruveta ", Lf); // 'r // optional 
        strncpy(SCen[19+sc1], "seven ", Le); strncpy(SCfi[19+sc1], "sinun ", Lf); // 's
        strncpy(SCen[20+sc1], "times ", Le); strncpy(SCfi[20+sc1], "t}ytyy ", Lf); // 't
        strncpy(SCen[21+sc1], "used ", Le); strncpy(SCfi[21+sc1], "siell} ", Lf); // 'u
        strncpy(SCen[22+sc1], "very ", Le); strncpy(SCfi[22+sc1], "verran ", Lf); // 'v
        strncpy(SCen[23+sc1], "wrong ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "whatever ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "years ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "think ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "their ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "of course ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '

        sc1 = 511; // Exclamation        a!
        strncpy(SCen[1+sc1], "asked ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "by the way, ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "could ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "does ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "et ", Le); strncpy(SCfi[5+sc1], "ette ", Lf); // 'e
        strncpy(SCen[6+sc1], "friend ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f // optional
        strncpy(SCen[7+sc1], "get ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "ght ", Le); strncpy(SCfi[8+sc1], "heti ", Lf); // 'h
        strncpy(SCen[9+sc1], "its ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i // optional
        strncpy(SCen[10+sc1], "got ", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "took ", Le); strncpy(SCfi[11+sc1], "kuten ", Lf); // 'k
        strncpy(SCen[12+sc1], "let ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l // (no common keys)
        strncpy(SCen[13+sc1], "moment ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "took ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "both ", Le); strncpy(SCfi[15+sc1], "olette ", Lf); // 'o
        strncpy(SCen[16+sc1], "other ", Le); strncpy(SCfi[16+sc1], "pit}} ", Lf); // 'p
        strncpy(SCen[17+sc1], "question ", Le); strncpy(SCfi[17+sc1], "olitte ", Lf); // 'q  // optional
        strncpy(SCen[18+sc1], "often ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "should ", Le); strncpy(SCfi[19+sc1], "st] ", Lf); // 's -fin- sta/st
        strncpy(SCen[20+sc1], "shouldn't ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't 
        strncpy(SCen[21+sc1], "something ", Le); strncpy(SCfi[21+sc1], "sitten ", Lf); // 'u
        strncpy(SCen[22+sc1], "-----", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "would ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "white ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "you'd ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "wouldn't ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "that ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "anything ", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-----", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "anything ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // ' // optional


        sc1 = 541; // Question      a?
        strncpy(SCen[1+sc1], "after ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "baby ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "-544-", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "-545-", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "eleven ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "far ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "great ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "here ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "-----", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "feel ", Le); strncpy(SCfi[11+sc1], "kanssa ", Lf); // 'k
        strncpy(SCen[12+sc1], "let's ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "million ", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "enough ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "old ", Le); strncpy(SCfi[15+sc1], "ole ", Lf); // 'o
        strncpy(SCen[16+sc1], "please ", Le); strncpy(SCfi[16+sc1], "paljon ", Lf); // 'p
        strncpy(SCen[17+sc1], "-----", Le); strncpy(SCfi[17+sc1], "olleet ", Lf); // 'q
        strncpy(SCen[18+sc1], "four ", Le); strncpy(SCfi[18+sc1], "onko ", Lf); // 'r
        strncpy(SCen[19+sc1], "soon ", Le); strncpy(SCfi[19+sc1], "sanoa ", Lf); // 's
        strncpy(SCen[20+sc1], "tell ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't // no overlap
        strncpy(SCen[21+sc1], "sure ", Le); strncpy(SCfi[21+sc1], "silloin ", Lf); // 'u // optional
        strncpy(SCen[22+sc1], "slow ", Le); strncpy(SCfi[22+sc1], "vaan ", Lf); // 'v
        strncpy(SCen[23+sc1], "will ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "won't ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "you're ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "three ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "there ", Le); strncpy(SCfi[29+sc1], "}lk}} ", Lf); // '
        strncpy(SCen[30+sc1], "off ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '


        sc1 = 571; // Hyphen          a-
        strncpy(SCen[1+sc1], "all ", Le); strncpy(SCfi[1+sc1], "aivan ", Lf); // 'a
        strncpy(SCen[2+sc1], "below ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "child ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "dear ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "each ", Le); strncpy(SCfi[5+sc1], "eiv}t ", Lf); // 'e
        strncpy(SCen[6+sc1], "feel ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f // optional
        strncpy(SCen[7+sc1], "going ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "hasn't ", Le); strncpy(SCfi[8+sc1], "h}n ", Lf); // 'h
        strncpy(SCen[9+sc1], "#i'll ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "job ", Le); strncpy(SCfi[10+sc1], "joka ", Lf); // 'j
        strncpy(SCen[11+sc1], "heard ", Le); strncpy(SCfi[11+sc1], "kovin ", Lf); // 'k
        strncpy(SCen[12+sc1], "let ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l -fin- lle
        strncpy(SCen[13+sc1], "make ", Le); strncpy(SCfi[13+sc1], "mahdollista ", Lf); // 'm
        strncpy(SCen[14+sc1], "night ", Le); strncpy(SCfi[14+sc1], "en}} ", Lf); // 'n
        strncpy(SCen[15+sc1], "ole ", Le); strncpy(SCfi[15+sc1], "olen ", Lf); // 'o  -fin- ole
        strncpy(SCen[16+sc1], "percent ", Le); strncpy(SCfi[16+sc1], "paha ", Lf);// 'p
        strncpy(SCen[17+sc1], "question ", Le); strncpy(SCfi[17+sc1], "olin ", Lf); // 'q
        strncpy(SCen[18+sc1], "read ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "small ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "take ", Le); strncpy(SCfi[20+sc1], "tehd} ", Lf); // 't  // option 3
        strncpy(SCen[21+sc1], "some ", Le); strncpy(SCfi[21+sc1], "sill} ", Lf); // 'u
        strncpy(SCen[22+sc1], "-----", Le); strncpy(SCfi[22+sc1], "vain ", Lf); // 'v
        strncpy(SCen[23+sc1], "well ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "where ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "you'll ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf);// 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "this ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "-----", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "theirs ", Le); strncpy(SCfi[29+sc1], "}l} ", Lf); // '
        strncpy(SCen[30+sc1], "-----", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '


        sc1 = 601; // Apostrophe      a'
        strncpy(SCen[1+sc1], "-602-", Le); strncpy(SCfi[1+sc1], "aivan ", Lf); // 'a
        strncpy(SCen[2+sc1], "bad ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "-604-", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "did ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "eight ", Le); strncpy(SCfi[5+sc1], "et ", Lf); // 'e // option
        strncpy(SCen[6+sc1], "find ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "good ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "his ", Le); strncpy(SCfi[8+sc1], "hyv} ", Lf); // 'h // option
        strncpy(SCen[9+sc1], "isn't ", Le); strncpy(SCfi[9+sc1], "eik~ ", Lf); // 'i optio
        strncpy(SCen[10+sc1], "just ", Le); strncpy(SCfi[10+sc1], "jos ", Lf); // 'j
        strncpy(SCen[11+sc1], "keep ", Le); strncpy(SCfi[11+sc1], "kuitenkin ", Lf); // 'k
        strncpy(SCen[12+sc1], "line ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "may ", Le); strncpy(SCfi[13+sc1], "min} ", Lf); // 'm
        strncpy(SCen[14+sc1], "nice ", Le); strncpy(SCfi[14+sc1], "n}pp}imist~ ", Lf); // 'n
        strncpy(SCen[15+sc1], "out ", Le); strncpy(SCfi[15+sc1], "olet ", Lf); // 'o
        strncpy(SCen[16+sc1], "don't ", Le); strncpy(SCfi[16+sc1], "pitk}}n ", Lf); // 'p
        strncpy(SCen[17+sc1], "quit ", Le); strncpy(SCfi[17+sc1], "olit ", Lf); // 'q
        strncpy(SCen[18+sc1], "ready  ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "say ", Le); strncpy(SCfi[19+sc1], "sin} ", Lf); // 's
        strncpy(SCen[20+sc1], "said ", Le); strncpy(SCfi[20+sc1], "tied} ", Lf); // 't
        strncpy(SCen[21+sc1], "sometimes ", Le); strncpy(SCfi[21+sc1], "sit} ", Lf); // 'u
        strncpy(SCen[22+sc1], "yesterday ", Le); strncpy(SCfi[22+sc1], "voi ", Lf); // 'v
        strncpy(SCen[23+sc1], "with ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "why ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "yours ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "wanted ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "this ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "that's ", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "these ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // ' optional
        strncpy(SCen[30+sc1], "anybody ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '


        sc1 = 631; // UpArrow       aU
        strncpy(SCen[1+sc1], "-632-", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "bright ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "change ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "do ", Le); strncpy(SCfi[4+sc1], "d] ", Lf);// 'd  -da/d
        strncpy(SCen[5+sc1], "eight ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "fight ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "had ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "haven't ", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf);// 'h
        strncpy(SCen[9+sc1], "ihmisi} ", Le); strncpy(SCfi[9+sc1], "ihan ", Lf); // 'i 
        strncpy(SCen[10+sc1], "-----", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "knight ", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k -fin- kuinka paljon
        strncpy(SCen[12+sc1], "light ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l // option
        strncpy(SCen[13+sc1], "might ", Le); strncpy(SCfi[13+sc1], "mahdollisimman ", Lf); // 'm
        strncpy(SCen[14+sc1], "night ", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "ought ", Le); strncpy(SCfi[15+sc1], "-fin- ", Lf); // 'o
        strncpy(SCen[16+sc1], "possible ", Le); strncpy(SCfi[16+sc1], "paha ", Lf); // 'p
        strncpy(SCen[17+sc1], "hers ", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf);// 'q
        strncpy(SCen[18+sc1], "right ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "sight ", Le); strncpy(SCfi[19+sc1], "-fin- ", Lf); // 's
        strncpy(SCen[20+sc1], "today ", Le); strncpy(SCfi[20+sc1], "-fin- ", Lf); // 't
        strncpy(SCen[21+sc1], "such ", Le); strncpy(SCfi[21+sc1], "siihen ", Lf); // 'u
        strncpy(SCen[22+sc1], "-----", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "was ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "wasn't ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "days ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-----", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "thank ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[28+sc1], "thank you ", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-----", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "-----", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '


        sc1 = 661; // DownArrow       aD
        strncpy(SCen[1+sc1], "any ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a // optional
        strncpy(SCen[2+sc1], "-663-", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "can ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "down ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd  // optional
        strncpy(SCen[5+sc1], "#english ", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "forth ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "gain ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "hand ", Le); strncpy(SCfi[8+sc1], "heid}n ", Lf); // 'h
        strncpy(SCen[9+sc1], "international ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "join ", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "know ", Le); strncpy(SCfi[11+sc1], "kun ", Lf);// 'k
        strncpy(SCen[12+sc1], "learn ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "man ", Le); strncpy(SCfi[13+sc1], "meid}n ", Lf); // 'm
        strncpy(SCen[14+sc1], "known ", Le); strncpy(SCfi[14+sc1], "niiden ", Lf); // 'n
        strncpy(SCen[15+sc1], "open ", Le); strncpy(SCfi[15+sc1], "olen ", Lf); // 'o -fin-
        strncpy(SCen[16+sc1], "pain ", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "above ", Le); strncpy(SCfi[17+sc1], "olivat ", Lf); // 'q
        strncpy(SCen[18+sc1], "once ", Le); strncpy(SCfi[18+sc1], "olen ", Lf); // 'r
        strncpy(SCen[19+sc1], "shall ", Le); strncpy(SCfi[19+sc1], "sen ", Lf); // 's
        strncpy(SCen[20+sc1], "ten ", Le); strncpy(SCfi[20+sc1], "teid}n ", Lf); // 't
        strncpy(SCen[21+sc1], "using ", Le); strncpy(SCfi[21+sc1], "sinne ", Lf); // 'u
        strncpy(SCen[22+sc1], "meant ", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "want ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "when ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf);// 'x
        strncpy(SCen[25+sc1], "young ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "anyone ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "than ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u 
        strncpy(SCen[28+sc1], "-689-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "then ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "any ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '

// Finally, change all unused BackSpace strings to "_" to
// prevent characters appearing after careless backspaces
        sc1 = 691; // BackSpace       aB
        strncpy(SCen[1+sc1], "as ", Le); strncpy(SCfi[1+sc1], "asia ", Lf); // 'a  'as' is not final
        strncpy(SCen[2+sc1], "_", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b // special to prevent chars in repeted backspaces
        strncpy(SCen[3+sc1], "come ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c
        strncpy(SCen[4+sc1], "-695-", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "-696-", Le); strncpy(SCfi[5+sc1], "-fin- ", Lf); // 'e
        strncpy(SCen[6+sc1], "-697-", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "-698-", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "-699-", Le); strncpy(SCfi[8+sc1], "-fin- ", Lf); // 'h
        strncpy(SCen[9+sc1], "-700-", Le); strncpy(SCfi[9+sc1], "isi ", Lf); // 'i
        strncpy(SCen[10+sc1], "-701-", Le); strncpy(SCfi[10+sc1], "-fin- ", Lf); // 'j
        strncpy(SCen[11+sc1], "-702-", Le); strncpy(SCfi[11+sc1], "-fin- ", Lf); // 'k
        strncpy(SCen[12+sc1], "-703-", Le); strncpy(SCfi[12+sc1], "ll] ", Lf); // 'l  -lla/ll
        strncpy(SCen[13+sc1], "-704-", Le); strncpy(SCfi[13+sc1], "-fin- ", Lf); // 'm
        strncpy(SCen[14+sc1], "-705-", Le); strncpy(SCfi[14+sc1], "-fin- ", Lf); // 'n
        strncpy(SCen[15+sc1], "came ", Le); strncpy(SCfi[15+sc1], "osaa ", Lf); // 'o
        strncpy(SCen[16+sc1], "-707-", Le); strncpy(SCfi[16+sc1], "-fin- ", Lf); // 'p
        strncpy(SCen[17+sc1], "-708-", Le); strncpy(SCfi[17+sc1], "-fin- ", Lf); // 'q
        strncpy(SCen[18+sc1], "-709-", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "so ", Le); strncpy(SCfi[19+sc1], "ss] ", Lf); // 's  -ssa/ss
        strncpy(SCen[20+sc1], "-711-", Le); strncpy(SCfi[20+sc1], "st] ", Lf); // 't -sta/st
        strncpy(SCen[21+sc1], "-712-", Le); strncpy(SCfi[21+sc1], "-fin- ", Lf); // 'u
        strncpy(SCen[22+sc1], "-713-", Le); strncpy(SCfi[22+sc1], "-fin- ", Lf); // 'v
        strncpy(SCen[23+sc1], "-714-", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "was ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "-716-", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "-717-", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z
        strncpy(SCen[27+sc1], "they ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u  
        strncpy(SCen[28+sc1], "-719-", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "-720-", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "-721-", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '  apperas by mistake


        sc1 = 721; // Space = F (Forward)   aF
        strncpy(SCen[1+sc1], "a ", Le); strncpy(SCfi[1+sc1], "a ", Lf); // 'a // To prevent sc words at the end of the word
        strncpy(SCen[2+sc1], "b ", Le); strncpy(SCfi[2+sc1], "b ", Lf); // 'b // and to prevent characters in repeted spaces
        strncpy(SCen[3+sc1], "c ", Le); strncpy(SCfi[3+sc1], "c ", Lf); // 'c
        strncpy(SCen[4+sc1], "d ", Le); strncpy(SCfi[4+sc1], "d ", Lf); // 'd
        strncpy(SCen[5+sc1], "e ", Le); strncpy(SCfi[5+sc1], "e ", Lf); // 'e // to make sure
        strncpy(SCen[6+sc1], "f ", Le); strncpy(SCfi[6+sc1], "f ", Lf); // 'f
        strncpy(SCen[7+sc1], "ed ", Le); strncpy(SCfi[7+sc1], "edelleen ", Lf); // 'g
        strncpy(SCen[8+sc1], "h ", Le); strncpy(SCfi[8+sc1], "h]n ", Lf); // 'h
        strncpy(SCen[9+sc1], "ion ", Le); strncpy(SCfi[9+sc1], "i ", Lf); // 'i
        strncpy(SCen[10+sc1], "j ", Le); strncpy(SCfi[10+sc1], "ja niin edelleen ", Lf); // 'j
        strncpy(SCen[11+sc1], "few ", Le); strncpy(SCfi[11+sc1], "kin ", Lf); // 'k (test)
        strncpy(SCen[12+sc1], "l ", Le); strncpy(SCfi[12+sc1], "ll] ", Lf); // 'l lla/ll
        strncpy(SCen[13+sc1], "m ", Le); strncpy(SCfi[13+sc1], "kuitenkaan ", Lf); // 'm
        strncpy(SCen[14+sc1], "n ", Le); strncpy(SCfi[14+sc1], "n ", Lf); // 'n // to make sure
        strncpy(SCen[15+sc1], "o ", Le); strncpy(SCfi[15+sc1], "o ", Lf); // 'o
        strncpy(SCen[16+sc1], "p ", Le); strncpy(SCfi[16+sc1], "pitemm}lle ", Lf); // 'p //testi
        strncpy(SCen[17+sc1], "q ", Le); strncpy(SCfi[17+sc1], "oikeastaan ", Lf); // 'q
        strncpy(SCen[18+sc1], "r ", Le); strncpy(SCfi[18+sc1], "rakastan ", Lf); // 'r
        strncpy(SCen[19+sc1], "s ", Le); strncpy(SCfi[19+sc1], "s ", Lf); // 's // to make sure
        strncpy(SCen[20+sc1], "tion ", Le); strncpy(SCfi[20+sc1], "t ", Lf); // 't (test)
        strncpy(SCen[21+sc1], "ies ", Le); strncpy(SCfi[21+sc1], "u ", Lf); // 'u (test)
        strncpy(SCen[22+sc1], "v ", Le); strncpy(SCfi[22+sc1], "viimeinkin ", Lf); // 'v
        strncpy(SCen[23+sc1], "we ", Le); strncpy(SCfi[23+sc1], "vanhastaan ", Lf); // 'w
        strncpy(SCen[24+sc1], "-745-", Le); strncpy(SCfi[24+sc1], "extreemi ", Lf); // 'x
        strncpy(SCen[25+sc1], "y ", Le); strncpy(SCfi[25+sc1], "y ", Lf); // 'y // to make sure
        strncpy(SCen[26+sc1], "standardisation ", Le); strncpy(SCfi[26+sc1], "standardointi ", Lf); // 'z (TEST)
        strncpy(SCen[27+sc1], "th ", Le); strncpy(SCfi[27+sc1], "u ", Lf); // 'u  TH    // Note that some of these include
        strncpy(SCen[28+sc1], "a ", Le); strncpy(SCfi[28+sc1], "| ", Lf); // '  a_    // a space in English national layout
        strncpy(SCen[29+sc1], "a ", Le); strncpy(SCfi[29+sc1], "} ", Lf); // '  the_  // ...
        strncpy(SCen[30+sc1], "o ", Le); strncpy(SCfi[30+sc1], "~ ", Lf); // '  to_   // ...


        sc1 = 751; // Shift                 aS
        strncpy(SCen[1+sc1], "able ", Le); strncpy(SCfi[1+sc1], "-fin- ", Lf); // 'a
        strncpy(SCen[2+sc1], "be ", Le); strncpy(SCfi[2+sc1], "-fin- ", Lf); // 'b
        strncpy(SCen[3+sc1], "ice cream ", Le); strncpy(SCfi[3+sc1], "-fin- ", Lf); // 'c // for fun
        strncpy(SCen[4+sc1], "different ", Le); strncpy(SCfi[4+sc1], "-fin- ", Lf); // 'd
        strncpy(SCen[5+sc1], "believe ", Le); strncpy(SCfi[5+sc1], "ei ", Lf); // 'e
        strncpy(SCen[6+sc1], "fine ", Le); strncpy(SCfi[6+sc1], "-fin- ", Lf); // 'f
        strncpy(SCen[7+sc1], "girl ", Le); strncpy(SCfi[7+sc1], "-fin- ", Lf); // 'g
        strncpy(SCen[8+sc1], "him ", Le); strncpy(SCfi[8+sc1], "he ", Lf); // 'h
        strncpy(SCen[9+sc1], "#i'm ", Le); strncpy(SCfi[9+sc1], "-fin- ", Lf); // 'i
        strncpy(SCen[10+sc1], "just ", Le); strncpy(SCfi[10+sc1], "juttu ", Lf); // 'j  //option 3
        strncpy(SCen[11+sc1], "kind ", Le); strncpy(SCfi[11+sc1], "kuin ", Lf); // 'k
        strncpy(SCen[12+sc1], "little ", Le); strncpy(SCfi[12+sc1], "-fin- ", Lf); // 'l
        strncpy(SCen[13+sc1], "mine ", Le); strncpy(SCfi[13+sc1], "me ", Lf); // 'm
        strncpy(SCen[14+sc1], "nine ", Le); strncpy(SCfi[14+sc1], "ne ", Lf); // 'n
        strncpy(SCen[15+sc1], "one ", Le); strncpy(SCfi[15+sc1], "olemme ", Lf); // 'o
        strncpy(SCen[16+sc1], "put ", Le); strncpy(SCfi[16+sc1], "pieni ", Lf); // 'p 
        strncpy(SCen[17+sc1], "quite ", Le); strncpy(SCfi[17+sc1], "olimme ", Lf); // 'q
        strncpy(SCen[18+sc1], "right ", Le); strncpy(SCfi[18+sc1], "-fin- ", Lf); // 'r
        strncpy(SCen[19+sc1], "she ", Le); strncpy(SCfi[19+sc1], "se ", Lf); // 's
        strncpy(SCen[20+sc1], "time ", Le); strncpy(SCfi[20+sc1], "te ", Lf); // 't
        strncpy(SCen[21+sc1], "ours ", Le); strncpy(SCfi[21+sc1], "siit} ", Lf); // 'u
        strncpy(SCen[22+sc1], "-773- ", Le); strncpy(SCfi[22+sc1], "viel} ", Lf); // 'v
        strncpy(SCen[23+sc1], "woman ", Le); strncpy(SCfi[23+sc1], "-fin- ", Lf); // 'w
        strncpy(SCen[24+sc1], "while ", Le); strncpy(SCfi[24+sc1], "-fin- ", Lf); // 'x
        strncpy(SCen[25+sc1], "year ", Le); strncpy(SCfi[25+sc1], "-fin- ", Lf); // 'y
        strncpy(SCen[26+sc1], "zipper ", Le); strncpy(SCfi[26+sc1], "-fin- ", Lf); // 'z // for fun
        strncpy(SCen[27+sc1], "thing ", Le); strncpy(SCfi[27+sc1], "-fin- ", Lf); // 'u  // option
        strncpy(SCen[28+sc1], "-779- ", Le); strncpy(SCfi[28+sc1], "-fin- ", Lf); // '
        strncpy(SCen[29+sc1], "them ", Le); strncpy(SCfi[29+sc1], "-fin- ", Lf); // '
        strncpy(SCen[30+sc1], "-781- ", Le); strncpy(SCfi[30+sc1], "-fin- ", Lf); // '
        // 781 Chordon Shortcuts in total !!!

        // Special functions:
        sc1 = 781; // WordRight + ...
        strncpy(SCen[1+sc1], "##", Le); // WR Sh => CAPS until next Space
        strncpy(SCen[2+sc1], "-783- ", Le); // WR SYMB
        strncpy(SCen[3+sc1], "/", Le); // WR 123-ABC // Send nothing. Just g123abc is set to TRUE.
        strncpy(SCen[4+sc1], "+", Le); // WR Delete WordRight (SpaceShift = 4)
        strncpy(SCen[5+sc1], " ##", Le); // Sp WR => Space + CAPS until next Space 
        strncpy(SCen[6+sc1], " /", Le); // Sp 123abc => Space + 123 until next Space
        strncpy(SCen[7+sc1], "%", Le); // Sh WL => Delete WordLeft (SpaceShift = 0)
        strncpy(SCen[8+sc1], "+", Le); // Sh Delete => Delete WordRight (SpaceShift = 4) 
        strncpy(SCen[9+sc1], "=", Le); // Just send nothing.
        strncpy(SCen[10+sc1], ":", Le); //":", Lf); //    ? \ => AltGr (XShifts = 2)
        strncpy(SCen[11+sc1], ";", Le); //   Sh \ => Shift+AltGr (XShifts = 8) 
        strncpy(SCen[12+sc1], "-793- ", Le); // WL ...
        strncpy(SCen[13+sc1], "-794- ", Le); // WL ...
        strncpy(SCen[14+sc1], "-795- ", Le); // WL ...
        strncpy(SCen[15+sc1], "-796- ", Le); // WL ...
        strncpy(SCen[16+sc1], "-797- ", Le); // WL ...
        strncpy(SCen[17+sc1], "-798- ", Le); // WL ...
        strncpy(SCen[18+sc1], "-799- ", Le); // WL ...
        strncpy(SCen[19+sc1], "-800- ", Le); // WL ... // default
        strncpy(SCen[20+sc1], "-801- ", Le); strncpy(SCfi[20+sc1], "This list is only output data! It cannot be edited yet. See also gkos-shortcuts.txt.", Lf); // WL ...

        //strncpy(SCen[1023] = "|", Le); strncpy(SCfi[1023] = "|", Lf); // for the shortcut list file

    // ---------------------------------------------------------


//static char *arrayIndices[10];
//    for(int i=0; i<10; ++i )
//            {
//            //arrayIndices[i] = malloc(11*sizeof(char));  /* 10 characters + '\0' */
//            //arrayIndices[i] = malloc(11*2);  /* 10 characters + '\0' */
//            arrayIndices[i] = "123456789012345678901234567890"; //malloc(11*sizeof(char));  /* 10 characters + '\0' */
//            }


// TESTING:

   static char ChordonTEMP[3];
   int tempint;
   bool usr_found = FALSE;
   string tempstring[100];	
   char wholeline[255]; // ref to line length in data file

// ----- Collect the three items from each line of text --------

   //create the in-stream object
   ifstream fin("gkos-shortcuts.usr"); // OPEN STREAM ******
   // if this file exists, it will be used
   
   if (fin.is_open())
   {
   usr_found = TRUE;
   for (tempint = 1; tempint <= 781; tempint++) 
                  {
   fin.getline(wholeline, 255); // 255 = max length before new line
   //MessageBox(NULL, wholeline,"The line that was read.", MB_OK | MB_ICONINFORMATION);
   
   int n, m, l, o, p;
   //string str("Hello Universe!");  // OK
   string str(wholeline);
   n = str.find( '|', 0 ); // Find the next '|'
   string chordpart = str.substr(0, n);
   m = str.find( '|', n+1 ); // Find the next '|'
   string engpart = str.substr(n+1,m-n-1);
   l = str.find( '|', m+1 ); // Find the next '|'
   string finpart = str.substr(m+1,l-m-1);
   o = str.find( '|', l+1 ); // Find the next '|'
   string xxxpart = str.substr(l+1,o-l-1);
   p = str.find( '|', o+1 ); // Find the next '|'
   string yyypart = str.substr(o+1,p-o-1);
   
// Each line must have this format: oB|came |osaa |-xxx- |-yyy- |
   Lc = 2;
   strncpy(Chrd[tempint], chordpart.c_str(), Lc); Chrd[tempint][Lc]='\0';
   Le = m-n-1; if (Le >= 32){Le = 32;}
   // Comment next line for fixed English shortcuts
   strncpy(SCen[tempint], engpart.c_str(), Le); SCen[tempint][Le]='\0';
   Lf = l-m-1; if (Lf >= 32){Lf = 32;}
   // Comment next line for fixed Finnish shortcuts
   strncpy(SCfi[tempint], finpart.c_str(), Lf); SCfi[tempint][Lf]='\0';
   Lx = o-l-1;  if (Lx >= 32){Lx = 32;} //32;
   // Swedish Shortcuts from .urs file
   strncpy(SCxx[tempint], xxxpart.c_str(), Lx); SCxx[tempint][Lx]='\0';
   Ly = p-o-1; if (Ly >= 32){Ly = 32;} //32;
   // German Shortcuts from .urs file
   strncpy(SCyy[tempint], yyypart.c_str(), Ly); SCyy[tempint][Ly]='\0';

   Lc = 2; Le = 32; Lf = 32; Lx = 32; Ly = 32; // back to defaults
   
   //strncpy(Chrd[99], chordpart.c_str(), L); Chrd[tempint][L]='\0'; //TEST OK
   //strncpy(Chrd[3], "qwertyui", L); Chrd[tempint][L]='\0'; //TEST OK
                     }   
   fin.close(); // CLOSE STREAM *****
   }
// -----------------------------------------------------------------

        
     // Just a list of Chordons for the start of each line:
//    FILE *GKOSshortcutstxt = fopen("gkos-shortcuts.txt", "r+");
    // This file is only used for generating the gkos-shortcuts.dat file

// See: http://www.zeuscmd.com/tutorials/cplusplus/38-ReadingTextFiles.php

	// ------- Save Chordons and word shortcuts in a file ----------------

    FILE *GKOSshortcuts = fopen("gkos-shortcuts.dat", "w");

     for (tempint = 1; tempint <= 781; tempint++) {
     
     // in case .usr file is not found:
     if (!usr_found){strncpy(SCxx[tempint], "+xxx+ ", 6); SCxx[tempint][6]='\0';}
     if (!usr_found){strncpy(SCyy[tempint], "+yyy+ ", 6); SCyy[tempint][6]='\0';}
              
	fprintf(GKOSshortcuts, "%s",Chrd[tempint]);
    fprintf(GKOSshortcuts, "%s", "|");
	//fprintf(GKOSshortcuts, "%s\0", strncpy(SCen[tempint]);
	fprintf(GKOSshortcuts, "%s", SCen[tempint]);
	fprintf(GKOSshortcuts, "%s", "|");
	//fprintf(GKOSshortcuts, "%s\0", SCfin[tempint]);
	fprintf(GKOSshortcuts, "%s", SCfi[tempint]);
	fprintf(GKOSshortcuts, "%s", "|");
    // xxx
	fprintf(GKOSshortcuts, "%s", SCxx[tempint]);
	fprintf(GKOSshortcuts, "%s", "|");
//    // yyy
	fprintf(GKOSshortcuts, "%s", SCyy[tempint]);
	fprintf(GKOSshortcuts, "%s\n", "|");
    }
	fprintf(GKOSshortcuts, "%s\n", SCfi[801]);
    
    fclose(GKOSshortcuts);
//   	if (GKOSshortcutstxt){fclose(GKOSshortcutstxt);}
                

    // ----------------------------------------------------------


} // end of function

//-------------------- END OF CHARACTER TABLES -------------------


